package net.nitus.museu.migraCoordenades;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import net.nitus.dbcon.SQL;

/**
 * Classe que implementa la generació del fitxers de comarques i municpis 
 * dels que s'ha de convertir els vèrtexs.
 * @author xnavarro
 *
 */
public class GeneraFitxerConversio {

	/**
	 * @param args
	 */
	public static void main(String[] args) {

		String[] limits={"Municipis", "Comarques"};
		String[] limit={"Municipi", "Comarca"};
		
		Connection museu = null;
		String select = null;
		try {
			System.out.println("Carregant el driver jdbc.");
			Class.forName("org.postgresql.Driver");
			
			System.out.println("Connecant a la base de dades.");
			museu = DriverManager.getConnection("jdbc:postgresql://localhost:5432/Museu","museu","patata");
			museu.setAutoCommit(false);
			
			Statement stmt = museu.createStatement();

			for (int i=0; i<limits.length; i++){
				System.out.println("Generant el fitxer per convertir "+limits[i]+".");
				BufferedWriter fitxer = new BufferedWriter(new FileWriter("Conv"+limits[i]+".txt"));
				fitxer.write("COORDINATES: Universal Transverse Mercator (UTM)\n");
				fitxer.write("DATUM: EUR-7\n");
				fitxer.write("# ELLIPSOID: IN\n");
				fitxer.write("END OF HEADER\n");
				fitxer.write("\n");
				fitxer.write("\n");

				//Consultem els municipis/comarques
				select = "SELECT Id, Nom, UNom, " +
						        "XFrom, YFrom, " +
						        "GLatFrom, MLatFrom, SLatFrom, " +
						        "GLongFrom, MLongFrom, SLongFrom, " +
						        "XTo, YTo, "+
								"GLatTo, MLatTo, SLatTo, " +
								"GLongTo, MLongTo, SLongTo, "+ 
								"nVertexs, Comentaris "+
						 "FROM icc."+limits[i]+" " +
						 "WHERE id <> 0";
				ResultSet rs = stmt.executeQuery(select);
				
				//Per cada municipi/comarca, escrivim l'identificador, les coordenades del from i del to per a convertir-les també.
				while (rs.next()){
					int idPare = rs.getInt(1);
					System.out.println("Escrivint al fitxer "+(i==0 ? "el":"la")+" "+limit[i]+" "+idPare);
					fitxer.write("#"+limit[i]+"-"+idPare+"-"+rs.getString(2)+"\n");
					fitxer.write("31, N, "+rs.getBigDecimal(4)+", "+rs.getBigDecimal(5)+"\n");
					fitxer.write("31, N, "+rs.getBigDecimal(12)+", "+rs.getBigDecimal(13)+"\n");
					
					// Consultem els vèrtexs
					fitxer.write("#Vertexs:\n");
					Statement stvertex = museu.createStatement();
					select = "SELECT "+limit[i]+", n, " +
							         "esring, " +
							         "x, y, " +
							         "glat, mlat, slat, " +
							         "glong, mlong, slong, "+ 
								 	 "comentaris "+
							 "FROM icc.vertexs"+limits[i]+" " +
							 "WHERE "+
							         limit[i]+"="+idPare;
					ResultSet rsVertexs = stvertex.executeQuery(select);
					
					// Per cada vèrtex, escrivim les coordenades i l'identificador per tenir la referència.
					System.out.println("Escrivint al fitxer els "+rs.getInt(20)+" vèrtexs de"+(i==0 ? "l ":" la ")+limit[i]+" "+idPare+".");
					while (rsVertexs.next()){
						fitxer.write("31, N, "+rsVertexs.getBigDecimal(4)+", "+rsVertexs.getBigDecimal(5)+" #"+rsVertexs.getInt(1)+"-"+rsVertexs.getInt(2)+"\n");
					}

					fitxer.write("\n");
					rsVertexs.close();
					stvertex.close();
				}
				fitxer.close();
				rs.close();
				System.out.println("S'ha generat el fitxer de "+limits[i]+" satisfactòriament.");
			}

			System.out.println("Tancant la connexió a la base de dades.");
			museu.commit();
			stmt.close();
			museu.close();
			System.out.println("S'han generat els fitxers per convertir satisfactòriament.");
		}
		catch (SQLException e) {
			System.err.println(select);
			System.err.println("ErrorSQL: "+e.getErrorCode()+"-"+e.getSQLState()+":"+e.getLocalizedMessage());
			SQL.rollbackTry(museu);
			try {
				if (museu!=null && !museu.isClosed())museu.close();
			} catch (SQLException e1) {
				System.out.println("Error tancant la connexió a la base de dades.");
				e1.printStackTrace();
			}
			System.exit(-1);
		} catch (ClassNotFoundException e) {
			System.err.println("No s'ha trobat el driver jdbc.");
			e.printStackTrace();
			System.exit(-1);
		} catch (IOException e) {
			System.err.println("No s'ha pogut crear el fitxer.");
			e.printStackTrace();
			System.exit(-1);
		}
		System.exit(0);

	}

}
