package net.nitus.museu.migraCoordenades;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import net.nitus.dbcon.SQL;

/**
 * Genera un fitxer d'entrada amb coordenades UTM 31N (ED50) per al GeoTrans 3
 * 
 * @author xnavarro
 *
 */
public class GeneraFitxerJaciments {

	/**
	 * @param args
	 */
	public static void main(String[] args) {

		Connection museu = null;
		String select = null;
		try {
			System.out.println("Carregant el driver jdbc.");
			Class.forName("org.postgresql.Driver");
			
			System.out.println("Connecant a la base de dades.");
			museu = DriverManager.getConnection("jdbc:postgresql://localhost:5432/Museu","museu","patata");
			museu.setAutoCommit(false);
			
			Statement stmt = museu.createStatement();

			System.out.println("Generant el fitxer per convertir jaciments.");
			BufferedWriter fitxer = new BufferedWriter(new FileWriter("ConvJaciments.txt"));
			fitxer.write("COORDINATES: Universal Transverse Mercator (UTM)\n");
			fitxer.write("DATUM: EUR-7\n");
			fitxer.write("# ELLIPSOID: IN\n");
			fitxer.write("END OF HEADER\n");
			fitxer.write("\n");
			fitxer.write("\n");

			//Consultem els Jaciments
			select = "SELECT Id, Nom, x, y " +
					 "FROM calcul.jaciments " +
					 "WHERE codificacio = 'UTM 31N (ED50)'";
				ResultSet rs = stmt.executeQuery(select);
				
			//Per cada jaciment, escrivim l'identificador, el nom i les coordenades.
			while (rs.next()){
				int id = rs.getInt(1);
				System.out.println("Escrivint al fitxer el jaciment "+id);
				fitxer.write("#Jaciment-"+id+"-"+rs.getString(2)+"\n");
				fitxer.write("31, N, "+rs.getBigDecimal(3)+", "+rs.getBigDecimal(4)+"\n");
			}
			fitxer.close();
			rs.close();
			museu.commit();
			System.out.println("S'ha generat el fitxer de conversió de jaciments satisfactòriament.");
			System.out.println("Tancant la connexió a la base de dades.");
			stmt.close();
			museu.close();
			System.out.println("S'ha generat el fitxer per convertir jaciments i s'ha tancat la connexió satisfactòriament.");
		}
		catch (SQLException e) {
			System.err.println(select);
			System.err.println("ErrorSQL: "+e.getErrorCode()+"-"+e.getSQLState()+":"+e.getLocalizedMessage());
			SQL.rollbackTry(museu);
			try {
				if (museu!=null && !museu.isClosed())museu.close();
			} catch (SQLException e1) {
				System.out.println("Error tancant la connexió a la base de dades.");
				e1.printStackTrace();
			}
			System.exit(-1);
		} catch (ClassNotFoundException e) {
			System.err.println("No s'ha trobat el driver jdbc.");
			e.printStackTrace();
			System.exit(-1);
		} catch (IOException e) {
			System.err.println("No s'ha pogut crear el fitxer.");
			e.printStackTrace();
			System.exit(-1);
		}
		System.exit(0);
	}

}
