package net.nitus.museu.migraCoordenades;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.StringTokenizer;

import net.nitus.dbcon.SQL;

/**
 * Classe que guarda a la base de dades els vèrtexs obtinguts de la 
 * conversió del GeoTrans
 * 
 * @author xnavarro
 *
 */
public class GuardaJacimentsWGS84 {
	private static final int CAPSALERA=0;
	private static final int ID=1;
	private static final int COORDENADA=2;
	private static int mode=CAPSALERA;
	private static int jacimentActual = 0;
	
	/**
	 * Estableix el mode Coordenada
	 * @param linia
	 */
	private static void modeCOORDENADA(String linia){
		mode = COORDENADA;
		String token;
		int comptaTokens = 0;
		StringTokenizer st = new StringTokenizer(linia, "-");
		while (st.hasMoreTokens()) {
            token = st.nextToken();
            comptaTokens++;
            if (comptaTokens==2) jacimentActual = new Integer(token).intValue();
        }
	}

	/**
	 * @param args
	 */
	public static void main(String[] args) {

		Connection museu = null;
		String update=null;
		try {
			System.out.println("Carregant el driver jdbc.");
			Class.forName("org.postgresql.Driver");
			
			System.out.println("Connecant a la base de dades.");
			museu = DriverManager.getConnection("jdbc:postgresql://localhost:5432/Museu","museu","patata");
			museu.setAutoCommit(false);
			
			Statement stmt = museu.createStatement();
			
			System.out.println("Obrint del fitxer de jaciments convertits.");
			String file = "WGS84Jaciments.dat";
			BufferedReader br = new BufferedReader (new FileReader (file));

								
			String linia = br.readLine();
			while (linia != null) {
				System.out.println(linia);
				// Controlem el canvi de municipi/comarca
				if (linia.startsWith("#Jaciment")) modeCOORDENADA(linia);
				else if (mode==COORDENADA){
					if(linia.indexOf("N")!=-1 && linia.indexOf("E")!=-1 && linia.indexOf(",")!=-1 ){
						PuntGeografic from = new PuntGeografic(linia);
						update = "UPDATE calcul.jaciments "+
								 "SET glat="+from.getLatitud().getGraus()+", " +
								     "mlat="+from.getLatitud().getMinuts()+", " +
								     "slat="+from.getLatitud().getSegons()+", " +
								     "glong="+from.getLongitud().getGraus()+", " +
								     "mlong="+from.getLongitud().getMinuts()+", " +
								     "slong="+from.getLongitud().getSegons()+", " +
								     "XWGS84="+from.getLongitud().getWgs84()+", "+
								     "YWGS84="+from.getLatitud().getWgs84()+" "+
								 "WHERE id ="+jacimentActual;
								stmt.executeUpdate(update);
								System.out.println("S'ha guardat el jaciment: "+jacimentActual);
								mode=ID;
					}
				}
				linia = br.readLine();
			}
			museu.commit();
			System.out.println("S'ha guardat les coordenades WGS84 dels jaciments..");
			System.out.println("Tancant la connexió a la base de dades.");
			stmt.close();
			museu.close();
			System.out.println("S'ha tancat la connexió satisfactòriament.");
		}
		catch (SQLException e) {
			System.err.println(update);
			System.err.println("ErrorSQL: "+e.getErrorCode()+"-"+e.getSQLState()+":"+e.getLocalizedMessage());
			SQL.rollbackTry(museu);
			try {
				if (museu!=null && !museu.isClosed())museu.close();
			} catch (SQLException e1) {
				System.out.println("Error tancant la connexió a la base de dades.");
				e1.printStackTrace();
			}
			System.exit(-1);
		} catch (ClassNotFoundException e) {
			System.err.println("No s'ha trobat el driver jdbc.");
			e.printStackTrace();
			System.exit(-1);
		} catch (IOException e) {
			System.err.println("No s'ha pogut obrir el fitxer.");
			e.printStackTrace();
			System.exit(-1);
		}
		System.exit(0);

	}
			

}


