package net.nitus.museu.migraCoordenades;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.StringTokenizer;

import net.nitus.dbcon.SQL;

/**
 * Guarda a la base de dades els vèrtexs dels municipis i les comarques
 * obtinguts de la conversió del GeoTrasn.
 * 
 * @author xnavarro
 *
 */
public class GuardaLimitsWGS84 {
	private static final int LIMIT=0;
	private static final int FROM=1;
	private static final int TO=2;
	private static final int VERTEX=3;
	private static int limitActual=-1;
	private static int mode=LIMIT;
	
	/**
	 * Estableix el mode From
	 * @param linia
	 */
	private static void modeFROM(String linia){
		mode = FROM;
		String token;
		int comptaTokens = 0;
		StringTokenizer st = new StringTokenizer(linia, "-");
		while (st.hasMoreTokens()) {
            token = st.nextToken();
            comptaTokens++;
            if (comptaTokens==2) limitActual = new Integer(token).intValue();
        }
	}

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		String[] limits={"Municipis", "Comarques"};
		String[] limit={"Municipi", "Comarca"};

		Connection museu = null;
		String update=null;
		try {
			System.out.println("Carregant el driver jdbc.");
			Class.forName("org.postgresql.Driver");
			
			System.out.println("Connecant a la base de dades.");
			museu = DriverManager.getConnection("jdbc:postgresql://localhost:5432/Museu","museu","patata");
			museu.setAutoCommit(false);
			
			Statement stmt = museu.createStatement();
			
			for (int i=0; i<limits.length; i++){
				System.out.println("Obrint del fitxer de "+limits[i]+".");
				String file = "WGS84"+limits[i]+".dat";
				BufferedReader br = new BufferedReader (new FileReader (file));

								
				String line = br.readLine();
				while (line != null) {
					System.out.println(line);
					// Controlem el canvi de municipi/comarca
					if (line.startsWith("#Municipi-") || line.startsWith("#Comarca-")) modeFROM(line);
					if (line.startsWith("#Vertexs:")) mode=VERTEX;
					// Les línies de coordenades tenen N, E, i ","
					if(line.indexOf("N")!=-1 && line.indexOf("E")!=-1 && line.indexOf(",")!=-1 ){
						switch (mode) {
							case FROM:
								PuntGeografic from = new PuntGeografic(line);
								update = "UPDATE icc."+limits[i]+" "+
						   		 		 "SET glatfrom="+from.getLatitud().getGraus()+", " +
						   		 		 	 "mlatfrom="+from.getLatitud().getMinuts()+", " +
						   		 		 	 "slatfrom="+from.getLatitud().getSegons()+", " +
						   		 		 	 "glongfrom="+from.getLongitud().getGraus()+", " +
						   		 		 	 "mlongfrom="+from.getLongitud().getMinuts()+", " +
						   		 		 	 "slongfrom="+from.getLongitud().getSegons()+", " +
						   		 		 	 "XFromWGS84="+from.getLongitud().getWgs84()+", "+
						   		 		 	 "YFromWGS84="+from.getLatitud().getWgs84()+" "+
						   		 		 "WHERE id ="+limitActual;
								stmt.executeUpdate(update);
								System.out.println("S'ha guardat From: "+from);
								mode=TO;
								break;
							case TO:
								PuntGeografic to = new PuntGeografic(line);
								update = "UPDATE icc."+limits[i]+" "+
								   		 "SET glatto="+to.getLatitud().getGraus()+", " +
								   		 	 "mlatto="+to.getLatitud().getMinuts()+", " +
								   		 	 "slatto="+to.getLatitud().getSegons()+", " +
						   		 		 	 "glongto="+to.getLongitud().getGraus()+", " +
						   		 		 	 "mlongto="+to.getLongitud().getMinuts()+", " +
						   		 		 	 "slongto="+to.getLongitud().getSegons()+", " +
						   		 		 	 "XToWGS84="+to.getLongitud().getWgs84()+", "+
						   		 		 	 "YToWGS84="+to.getLatitud().getWgs84()+" "+
						   		 		 "WHERE id ="+limitActual;
								stmt.executeUpdate(update);
								System.out.println("S'ha guardat To: "+to);
								mode=VERTEX;
								break;
							case VERTEX:
								String token;
								String id=null;
								PuntGeografic vertex=null;
								int comptaTokens=0;
								StringTokenizer st = new StringTokenizer(line,"#");
								while (st.hasMoreTokens()){
									token = st.nextToken();
									comptaTokens++;
									if (comptaTokens==1) vertex = new PuntGeografic(token);
									else id = token;
								}
								String idPare = null;
								String n = null;
								comptaTokens=0;
								st = new StringTokenizer(id,"-");
								while (st.hasMoreTokens()){
									token = st.nextToken();
									comptaTokens++;
									if (comptaTokens==1) idPare=token;
									else n = token;
								}
								update = "UPDATE icc.vertexs"+limits[i]+" "+
										 "SET glat="+vertex.getLatitud().getGraus()+", " +
										     "mlat="+vertex.getLatitud().getMinuts()+", " +
										     "slat="+vertex.getLatitud().getSegons()+", " +
										     "glong="+vertex.getLongitud().getGraus()+", "+
										     "mlong="+vertex.getLongitud().getMinuts()+", " +
										     "slong="+vertex.getLongitud().getSegons()+", " +
											 "XWGS84="+vertex.getLongitud().getWgs84()+", "+
											 "YWGS84="+vertex.getLatitud().getWgs84()+" "+
										 "WHERE "+limit[i]+"="+idPare+" and n="+n;
								stmt.executeUpdate(update);
								System.out.println("S'ha guardat: "+idPare+"-"+n+":"+vertex);
								break;
							default:
								break;
							}
					}
					//System.out.println(line);
					
					line = br.readLine();
				}
				System.out.println("S'ha guardat les coordenades de "+limits[i]);
			}
			
			System.out.println("Tancant la connexió a la base de dades.");
			museu.commit();
			stmt.close();
			museu.close();
			System.out.println("Guardat les coordenades geogràfiques satisfactòriament.");
		}
		catch (SQLException e) {
			System.err.println(update);
			System.err.println("ErrorSQL: "+e.getErrorCode()+"-"+e.getSQLState()+":"+e.getLocalizedMessage());
			SQL.rollbackTry(museu);
			try {
				if (museu!=null && !museu.isClosed())museu.close();
			} catch (SQLException e1) {
				System.out.println("Error tancant la connexió a la base de dades.");
				e1.printStackTrace();
			}
			System.exit(-1);
		} catch (ClassNotFoundException e) {
			System.err.println("No s'ha trobat el driver jdbc.");
			e.printStackTrace();
			System.exit(-1);
		} catch (IOException e) {
			System.err.println("No s'ha pogut obrir el fitxer.");
			e.printStackTrace();
			System.exit(-1);
		}
		System.exit(0);

	}
			

}


