package net.nitus.museu.migraCoordenades;

import java.math.BigDecimal;
import java.util.StringTokenizer;
import net.nitus.widgets.StringsTools;

/**
 * Classe que modela un jaciment a efectes de la migració
 * @author xnavarro
 *
 */
public class JacimentMigracio {
	private int id;
	private String nom;
	private String uNom;
	private String poblacio;
	private String codificacio;
	private CoordenadaED50 posicio;
	private GeograficaFitxer longitud;
	private GeograficaFitxer latitud;
	private String imatge;
	private Integer xImatge;
	private Integer yImatge;
	
	/**
	 * Constructor per defecte
	 */
	public JacimentMigracio(){
		this.id=-1;
		this.nom=null;
		this.uNom=null;
		this.poblacio=null;
		this.codificacio=null;
		this.posicio=null;
		this.longitud=null;
		this.latitud=null;
		this.imatge="";
		this.xImatge=null;;
		this.yImatge=null;;
	}

	/**
	 * Constructor a partir d'una línia del fitxer
	 * @param registre
	 */
	public JacimentMigracio(String registre){
		this.posicio= new CoordenadaED50(0, 0, 0, 0);
		

		int comptaTokens=0;
		String token;
		StringTokenizer st = new StringTokenizer(registre, ";");
		while (st.hasMoreTokens()) {
			comptaTokens++;
            token = st.nextToken();
            if (comptaTokens==1) setId(token);
            else if (comptaTokens==2) setNom(token);
            else if (comptaTokens==3) setPoblacio(token);
            else if (comptaTokens==4) setCodificacio(token);
            else if (comptaTokens==6 || comptaTokens==8) {
            	if (codificacio.equals("WGS84")){
            		String cadena = "";
            		for (int i=0;i<token.length();i++){
            			if (token.charAt(i)!='º' && token.charAt(i)!='’') cadena+=token.charAt(i);
            		}
            		if (comptaTokens==6) setLongitud(new GeograficaFitxer(cadena));
            		else setLatitud(new GeograficaFitxer(cadena));
            	}
            	else {
            		if (comptaTokens==6) this.posicio.setLongitud(new BigDecimal(token).doubleValue());
            		else this.posicio.setLatitud(new BigDecimal(token).doubleValue());
            	}
            }
            else if (comptaTokens==9) setImatge(token);
            else if (comptaTokens==10) setxImatge(token);
            else if (comptaTokens==11) setyImatge(token);
        }
	}

	/**
	 * Sobreescriu el mètode toString
	 */
	public String toString(){
		String classe = "[\n";
		classe+= "Id: "+this.id+", ";
		classe+= "nom: "+this.nom+", ";
		classe+= "unom "+this.uNom+", ";
		classe+= "poblacio: "+this.poblacio+", ";
		classe+= "codificacio: "+this.codificacio+", ";
		classe+= "posicio: "+this.posicio+", ";
		classe+= "longitud: "+this.longitud+", ";
		classe+= "latitud: "+this.latitud+", ";
		classe+= "imatge: "+this.imatge+", ";
		classe+= "xImatge: "+this.xImatge+", ";
		classe+= "yImatge: "+this.yImatge+", ";
		classe+= "]";
		return classe;
	}

	/**
	 * Genera la cadena amb la comanda SQL per
	 * tal d'afegir el jaciment a la taula de treball
	 * @return String amb la cadena SQL
	 */
	public String desa(){
		String vImatge;
		if (getImatge()!=null && !getImatge().equals("")) vImatge = "'"+getImatge()+"'";
		else vImatge = "null";

		String vPoblacio; 
		if (getPoblacio()!=null && !getPoblacio().equals("")) vPoblacio = "'"+StringsTools.dq(getPoblacio())+"'";
		else vPoblacio = "''";
		String insert = "INSERT INTO calcul.jaciments("+
            					"id, nom, unom, " +
            					"poblacio, codificacio, " +
            					"x, " +
            					"y, " +
            					"glat, mlat, slat, "+ 
            					"glong, mlong, slong, " +
            					"xwgs84, ywgs84, " +
            					"imatge, xImatge, yImatge, " +
            					"comentaris) "+
            			"VALUES ("+getId()+", '"+StringsTools.dq(getNom())+"', '"+StringsTools.dq(getuNom())+"', " +
            					""+vPoblacio+", '"+getCodificacio()+"', " +
            					""+(getCodificacio().equals("WGS84") ? "0":posicio.getLongitud())+", " +
            					""+(getCodificacio().equals("WGS84") ? "0":posicio.getLatitud())+", " +
            					""+(getCodificacio().equals("WGS84") ? 
            							latitud.getGraus()+", "+latitud.getMinuts()+", "+latitud.getSegons()+", "+
            							longitud.getGraus()+", "+longitud.getMinuts()+", "+longitud.getSegons()+", 0, 0,  ":
            							"0, 0, 0, 0, 0, 0, 0, 0, ")+ 
            					""+vImatge+", "+getxImatge()+", "+getyImatge()+", null)";
		return insert;
	}

	// Els getters i els setters
	public int getId() {
		return id;
	}

	public void setId(int id) {
		this.id = id;
	}

	public void setId(String id){
		this.id=new Integer(id).intValue();
	}
	
	public String getNom() {
		return nom;
	}

	public void setNom(String nom) {
		this.nom = nom;
		this.uNom = StringsTools.U(nom);
	}

	public String getuNom() {
		return uNom;
	}

	public String getPoblacio() {
		return poblacio;
	}

	public void setPoblacio(String poblacio) {
		this.poblacio = poblacio;
	}

	public String getCodificacio() {
		return codificacio;
	}

	public void setCodificacio(String codificacio) {
		this.codificacio = codificacio;
	}

	public CoordenadaED50 getPosicio() {
		return posicio;
	}

	public void setPosicio(CoordenadaED50 posicio) {
		this.posicio = posicio;
	}

	public GeograficaFitxer getLongitud() {
		return longitud;
	}

	public void setLongitud(GeograficaFitxer longitud) {
		this.longitud = longitud;
	}

	public GeograficaFitxer getLatitud() {
		return latitud;
	}

	public void setLatitud(GeograficaFitxer latitud) {
		this.latitud = latitud;
	}

	public String getImatge() {
		return imatge;
	}

	public void setImatge(String imatge) {
		this.imatge = imatge;
	}

	public Integer getxImatge() {
		return xImatge;
	}

	public void setxImatge(int xImatge) {
		this.xImatge = xImatge;
	}
	public void setxImatge(String xImatge) {
		this.xImatge=new Integer(xImatge);
	}
	
	public Integer getyImatge() {
		return yImatge;
	}

	public void setyImatge(int yImatge) {
		this.yImatge = yImatge;
	}

	public void setyImatge(String yImatge) {
		this.yImatge=new Integer(yImatge);
	}
	
}
