package net.nitus.museu.migraCoordenades;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.text.ParseException;

/**
 * Implementa el procediment de carregar les dades 
 * del shape file a la base de dades.
 * @author xnavarro
 *
 */
public class MigraCoordenades {
	static String file = "municipis.txt";
	/**
	 * Llegeix el fitxer i carrega les dades.
	 */
	public static ShapeFile getShapeFile ()
		throws FileNotFoundException, IOException, ParseException {
		
		ShapeFile elFitxer = new ShapeFile();
		
		// Llegim el fitxer.
		System.out.println("Llegint el fitxer...");
        BufferedReader br = new BufferedReader (new FileReader (file));
        int comptalinies = 0;
    	Shape actual = null;
        String line = br.readLine();
        while (line != null) {
        	comptalinies++;
        	if (line.startsWith("Shapefile Type:")) elFitxer.liniaTipus(line); // Processem les línies Shapefile Type
        	if (line.startsWith("File Bounds:")) { // Processem les línies dels bounds.
        		elFitxer.liniaFromBounds(line); // Processem el from
        		line = br.readLine();			// Segur que trobem un to
        		comptalinies++;
        		elFitxer.liniaToBounds(line);	// Processem el to
        	}
        	if (line.startsWith("Database records ")) elFitxer.liniaDataBaseRecords(line);
        	if (line.startsWith("Field ")) elFitxer.addField(new Field(line));
        	if (line.startsWith("Shape:")) {
        		if (actual!=null) elFitxer.addShape(actual);
        		actual = new Shape(line);
        	}
        	if (line.startsWith("  Bounds:")) { // Processem les línies dels bounds del shape.
        		actual.liniaFromBounds(line); // Processem el from
        		line = br.readLine();			// Segur que trobem un to
        		comptalinies++;
        		actual.liniaToBounds(line);	// Processem el to
        	}
        	if (line.startsWith("     (")) {
        		CoordenadaED50 vertex = new CoordenadaED50(line);
        		System.out.println(vertex);
        		actual.addVertex(vertex);
        	}
        	line = br.readLine();
        }
        br.close();
        if (actual!=null) elFitxer.addShape(actual);
		return elFitxer;
	}

	public static void main(String[] args) {
		try {
			String limits[] = {"municipis", "comarques"};
			for (int i=0; i<limits.length; i++){
				System.out.println("Carregant el fitxer satisfactòriament...");
				file = limits[i]+".txt";
				ShapeFile elFitxer = getShapeFile();
				System.out.println("S'ha carregat el fitxer satisfactòriament...");
				if (elFitxer.desa(file.equals("municipis.txt"))) System.out.println("S'ha guardat el fitxer satisfactòriament.");
				else {
					System.err.println("No s'ha pogut guardar el fitxer.");
				}
			}
		} catch (FileNotFoundException e) {
			e.printStackTrace();
			System.exit(-1);
		} catch (IOException e) {
			e.printStackTrace();
			System.exit(-1);
		} catch (ParseException e) {
			e.printStackTrace();
			System.exit(-1);
		}
		System.exit(0);
	}

}
