package net.nitus.museu.migraCoordenades;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.StringTokenizer;

import net.nitus.dbcon.SQL;
import net.nitus.widgets.StringsTools;

/**
 * Classe que modela l'estructura d'un fitxer shape.
 * @author xnavarro
 *
 */
public class ShapeFile {
	private String tipus;
	private CoordenadaED50 fromBounds;
	private CoordenadaED50 toBounds;
	private Integer nRecords;
	private Integer nFields;
	private Integer nShapes;
	private ArrayList<Field> fields;
	private ArrayList<Shape> shapes;
	
	public ShapeFile(){
		this.tipus=null;
		this.fromBounds=null;
		this.toBounds=null;
		this.nRecords=null;
		this.nFields=null;
		this.nShapes=null;
		this.fields = new ArrayList<Field>();
		this.shapes = new ArrayList<Shape>();
	}
	
	/**
	 * Processa les línies: Shapefile Type: Arc   # of Shapes: 3569
	 * @param line
	 */
	public void liniaTipus(String line){
		String token;
		StringTokenizer st = new StringTokenizer(line, "#");
		while (st.hasMoreTokens()) {
            token = st.nextToken();
            if (token.startsWith("Shapefile Type:")) this.tipus=token.substring((token.indexOf(':'))+2);
            else this.nShapes=(new Integer(token.substring((token.indexOf(':'))+2))).intValue();
        }
	}

	/**
	 * Processa les línies: File Bounds: (260283.860000,4488965.370000,0,0)
	 * @param line
	 */
	public void liniaFromBounds(String line){
		this.fromBounds=new CoordenadaED50(line);
	}

	/**
	 * Processa les línies:          to  (527495.190000,4748184.120000,0,0)
	 * @param line
	 */
	public void liniaToBounds(String line){
		this.toBounds=new CoordenadaED50(line);
	}

	/**
	 * Processa les línies: Database records 3569, fields 3
	 * @param line
	 */
	public void liniaDataBaseRecords(String line){
		line = line.substring(18);
		int comptaTokens = 0;
		String token;
		StringTokenizer st = new StringTokenizer(line, ", fields ");
		while (st.hasMoreTokens()) {
            token = st.nextToken();
            comptaTokens++;
            if (comptaTokens==1) this.nRecords=new Integer(token);
            else this.nFields=new Integer(token);
        }
	}
	/**
	 * Afegeix un field a la llista de fields
	 * @param camp
	 */
	public void addField(Field camp){
		this.fields.add(camp);
	}

	/**
	 * Afegeix un shape a la llista de shapes
	 * @param shape
	 */
	public void addShape(Shape shape){
		this.shapes.add(shape);
	}

	/**
	 * Valida que el nombre de fields i shapes llegits corresponen amb 
	 * el que diu a la línia corresponent del fitxer.
	 */
	public void valida(){
		if (nShapes!=shapes.size()) System.err.println("Els shapes de la llista no coincideix amb el nombre de shapes.");
		if (nFields!=fields.size()) System.err.println("Els camps de la llista no coincideix amb el nombre de camps.");
	}
	
	/**
	 * Sobreescriu el mètode toString per obtenir quelcom llegible.
	 */
	public String toString(){
		String classe = "[\n";
		classe+= "Tipus: "+this.tipus+'\n';
		classe+= "nShapes: "+this.nShapes+", ";
		classe+= "[nShapes == shapes.size()] = "+(this.nShapes==shapes.size())+'\n';
		classe+= "fromBounds: "+this.fromBounds+'\n';
		classe+= "toBounds: "+this.toBounds+'\n';
		classe+= "nRecords: "+this.nRecords+'\n';
		classe+= "nFields: "+this.nFields+", ";
		classe+= "[nFields == fields.size()] = "+(this.nFields==fields.size())+']';
		classe+= "]";
		return classe;
	}

	/**
	 * Mostra per pantalla tota la informació del shapefile
	 */
	public void imprimeix(){
		System.out.println(this);
		System.out.println("{Fields:"+fields.size()+"}\n");
		Iterator<Field> it = fields.iterator();
		while (it.hasNext()) System.out.println(it.next().toString()+'\n');

		System.out.println("{Shapes:"+shapes.size()+"}\n");
		Iterator<Shape> itS = shapes.iterator();
		while (itS.hasNext()){
			Shape sh = itS.next();
			System.out.println(sh);
			System.out.println("{Vertex: "+sh.vertexs.size()+"}\n");
			Iterator<CoordenadaED50> itC = sh.getVertexs().iterator();
			while (itC.hasNext()) {
				System.out.println(itC.next());
			}
		}
	}
	
	/**
	 * Desa a la base de dades el contingut del fitxer shape.
	 * @param esMunicipi
	 * @return cert si ha anat bé, fals en cas contrari
	 */
	public boolean desa(boolean esMunicipi){
		int comptaLimits = 0;
		int comptaVertexs = 0;
		String[] limits={"Municipis", "Comarques"};
		String[] limit={"Municipi", "Comarca"};
		int x = (esMunicipi ? 0:1);
		Iterator<Shape> itS = shapes.iterator();
		Connection museu = null;
		String insert = null;
		try {
			System.out.println("Carregant el driver jdbc.");
			Class.forName("org.postgresql.Driver");
			
			System.out.println("Connecant a la base de dades.");
			museu = DriverManager.getConnection("jdbc:postgresql://localhost:5432/Museu","museu","patata");
			museu.setAutoCommit(false);
			
			Statement stmt = museu.createStatement();
			
			System.out.println("Guardant "+shapes.size()+" "+limits[x]+"...");
			while (itS.hasNext()){
				comptaLimits++;
				System.out.println("Afegim el "+limit[x]+" "+comptaLimits+" de "+shapes.size());
				Shape sh = itS.next();
				insert = "INSERT INTO icc."+limits[x]+" " +
						          "(" +
								    "Id, " +
								    "Nom, " +
								    "UNom, " +
								    "XFrom, YFrom, " +
								    "GLatFrom, MLatFrom, SLatFrom, GLongFrom, MLongFrom, SLongFrom, "+
								    "XFromWGS84, YFromWGS84, " +
								    "XTo, YTo, " +
								    "GLatTo, MLatTo, SLatTo, GLongTo, MLongTo, SLongTo, " +
								    "XToWGS84, YToWGS84, " +
								    "nVertexs, Comentaris) "+
								"VALUES " +
								  "(" +
								    comptaLimits+", " +
								    "'"+limit[x]+" "+comptaLimits+"', " +
								    "'"+StringsTools.U(limit[x])+" "+comptaLimits+"', " +
								    sh.getFromBounds().getLongitud()+", "+sh.getFromBounds().getLatitud()+", " +
								    "0, 0, 0, 0, 0, 0, " +
								    "0, 0, "+ 
								    sh.getToBounds().getLongitud()+", "+sh.getToBounds().getLatitud()+", " +
								    "0, 0, 0, 0, 0, 0, " +
								    "0, 0, "+
								    sh.getnVertexs()+", null)";
				stmt.executeUpdate(insert);

				System.out.println("Afegim "+sh.vertexs.size()+" vèrtexs a"+(esMunicipi ? "l Municipi.":" la Comarca."));
				int comptaVertexComarca = 0;
				Iterator<CoordenadaED50> itC = sh.getVertexs().iterator();
				while (itC.hasNext()) { // Vèrtexs
					CoordenadaED50 c = itC.next();
					comptaVertexComarca++;
					comptaVertexs++;
					insert = "INSERT INTO icc.Vertexs"+limits[x]+" " +
							  "("+
							    ""+limit[x]+", N, EsRing, " +
							    "X, Y, " +
							    "GLat, MLat, SLat, " +
							    "GLong, MLong, SLong, " +
							    "XWGS84, YWGS84, " +
							    "Comentaris) "+
							 "VALUES " +
							  "(" +
							    comptaLimits+", "+comptaVertexComarca+", '"+c.isRing()+"', " +
							    c.getLongitud()+", "+c.getLatitud()+", " +
							    "0, 0, 0, " +
							    "0, 0, 0, " +
							    "0, 0, "+
							    "null)";
					stmt.executeUpdate(insert);
				} // Vèrtexs
			} // Municipis/Comarques
			museu.commit();
			stmt.close();
			museu.close();
		}
		catch (SQLException e) {
			System.err.println(insert);
			System.err.println("ErrorSQL: "+e.getErrorCode()+"-"+e.getSQLState()+":"+e.getLocalizedMessage());
			SQL.rollbackTry(museu);
			try {
				if (museu!=null && !museu.isClosed())museu.close();
			} catch (SQLException e1) {
				System.out.println("Error tancant la connexió a la base de dades.");
				e1.printStackTrace();
			}
			return false;
		} catch (ClassNotFoundException e) {
			System.err.println("No s'ha trobat el driver jdbc.");
			e.printStackTrace();
			return false;
		}

		return true;
	}

	// Els getters i els setters
	public String getTipus() {
		return tipus;
	}

	public void setTipus(String tipus) {
		this.tipus = tipus;
	}

	public CoordenadaED50 getFromBounds() {
		return fromBounds;
	}

	public void setFromBounds(CoordenadaED50 fromBounds) {
		this.fromBounds = fromBounds;
	}

	public CoordenadaED50 getToBounds() {
		return toBounds;
	}

	public void setToBounds(CoordenadaED50 toBounds) {
		this.toBounds = toBounds;
	}

	public Integer getnRecords() {
		return nRecords;
	}

	public void setnRecords(Integer nRecords) {
		this.nRecords = nRecords;
	}

	public Integer getnFields() {
		return nFields;
	}

	public void setnFields(Integer nFields) {
		this.nFields = nFields;
	}

	public Integer getnShapes() {
		return nShapes;
	}

	public void setnShapes(Integer nShapes) {
		this.nShapes = nShapes;
	}

	public ArrayList<Field> getFields() {
		return fields;
	}

	public void setFields(ArrayList<Field> fields) {
		this.fields = fields;
	}

	public ArrayList<Shape> getShapes() {
		return shapes;
	}

	public void setShapes(ArrayList<Shape> shapes) {
		this.shapes = shapes;
	}
	
	
}
