package net.nitus.museu.scripts;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import net.nitus.dbcon.SQL;
import net.nitus.museu.context.Imatge;
import net.nitus.museu.context.Objecte;
import net.nitus.museu.context.PeriodeHistoric;
import net.nitus.museu.context.PeriodeJaciment;
import net.nitus.widgets.StringsTools;

public class BaseDeDades {

	/**
	 * Crea la base de dades de l'aplicació i carrega els
	 * registres a partir dels fitxers amb la informació.
	 * @param args
	 */
	public static void main(String[] args) {
		Connection museu = null;
		
		try {
			System.out.println("Carregant el driver jdbc.");
			Class.forName("org.postgresql.Driver");
			System.out.println("Connecant a la base de dades.");
			museu = DriverManager.getConnection("jdbc:postgresql://localhost:5432/Museu","museu","patata");
			museu.setAutoCommit(false);
			Statement stmt = museu.createStatement();

			crearEsquema(stmt);
			crearTaulaJaciments(museu);
			crearTaulaPeriodes(stmt);
			crearTaulaPeriodesJaciments(stmt);
			crearTaulaObjectes(stmt);
			crearTaulaImatges(stmt);
			
			System.out.println("S'ha creat la base de dades del museu satisfactòriament.");
			stmt.close();
			museu.commit();
			System.out.println("Tancant la connexió amb la base de dades.");
			museu.close();
			System.out.println("S'ha tancat la connexió satisfactòriament.");
		} 
		catch (SQLException e){
			System.err.println("Error SQL: "+e.getErrorCode()+"-"+e.getSQLState()+": "+e.getLocalizedMessage());
			SQL.rollbackTry(museu);
			try {
				if (museu!=null && !museu.isClosed())museu.close();
			} catch (SQLException e1) {
				System.out.println("Error tancant la connexió a la base de dades.");
				e1.printStackTrace();
			}
			System.exit(-1);
		}
		catch (FileNotFoundException e) {
			System.err.println("No s'ha localitzat el fitxer amb les dades.");
		}
		catch (IOException e) {
			System.err.println("S'ha produit un error llegint el fitxer amb les dades.");
		}
		catch (ClassNotFoundException e) {
			System.err.println("No s'ha localitzat el Driver JDBC.");
		}
		catch (Exception e) {
			e.printStackTrace();
		}

	}

	/**
	 * Crea l'esquema de les taules de l'aplicació.
	 * @param stmt
	 * @throws SQLException
	 */
	private static void crearEsquema(Statement stmt) throws SQLException{
		//L'esquema
		System.out.println("Creant l'Esquema...");
		String crearEsquema = "CREATE SCHEMA Jaciments";
		stmt.executeUpdate(crearEsquema);
		
		System.out.println("Assignat drets d'Administració a l'Esquema dels Jaciments.");
		String dretsAdminEsquema = "GRANT ALL ON SCHEMA Jaciments TO GROUP administramuseu";
		stmt.executeUpdate(dretsAdminEsquema);

		System.out.println("Assignat drets d'accés a l'Esquema dels Jaciments.");
		String dretsLectEsquema = "GRANT USAGE ON SCHEMA Jaciments TO GROUP consultamuseu";
		stmt.executeUpdate(dretsLectEsquema);
		
		System.out.println("S'ha creat l'esquema satisfactòriament.\n");
	}
	
	/**
	 * Crea la taula de jaciments i carrega les dades de la taula
	 * de treball de la conversió de les coordenades.
	 * @param con
	 * @throws SQLException
	 */
	private static void crearTaulaJaciments(Connection con) throws SQLException{
		// Taula de Jaciments
		Statement stmt = con.createStatement();
		System.out.println("Creant la taula dels jaciments.");
		String crearTaula = "CREATE TABLE Jaciments.Jaciments "+
							"("+
							  "Id integer NOT NULL, "+
							  "Nom character varying(50) not null, "+
							  "UNom character varying(50) not null, "+
							  "Poblacio character varying(50) not null, "+
							  "XWGS84 numeric(9,7) NOT NULL, "+
							  "YWGS84 numeric(9,7) NOT NULL, "+
							  "Imatge character varying(255), "+
							  "xImatge integer, "+
							  "yImatge integer, "+
							  "Comentaris character varying(255), "+
							  "CONSTRAINT PKJaciments PRIMARY KEY (Id), "+
							  "CONSTRAINT CCJacimentsIntegritatImatge CHECK " +
		                        "((imatge is null and xImatge is null and yImatge is null) or" +
		                        "(imatge is not null and xImatge is not null and yImatge is not null))" +
		                          ")" +
								"WITH ( "+
								 "OIDS = FALSE "+
								")";
			
		stmt.executeUpdate(crearTaula);
			
		System.out.println("Afegint el comentari a la taula...");
		String comentari = "COMMENT ON TABLE Jaciments.Jaciments IS 'Taula dels jaciments.'";
		stmt.executeUpdate(comentari);
			
		System.out.println("Assignant els drets d'administradors...");
		String dretsAdministracio = "GRANT ALL ON TABLE Jaciments.Jaciments TO GROUP AdministraMuseu";
		stmt.executeUpdate(dretsAdministracio);

		System.out.println("Assignant els drets de lectura...");
		String dretsLectura = "GRANT SELECT ON TABLE Jaciments.Jaciments TO GROUP ConsultaMuseu";
		stmt.executeUpdate(dretsLectura);

		System.out.println("Creant l'índex sobre uNom.");
		String indexUNom = "Create Unique Index XUJaciments_uNom on Jaciments.Jaciments(uNom) ";
		stmt.executeUpdate(indexUNom);
		
		System.out.println("Creant l'índex sobre xWGS84.");
		String indexXWGS84 = "Create Index XJaciments_XWGS84 on Jaciments.Jaciments(XWGS84) ";
		stmt.executeUpdate(indexXWGS84);

		System.out.println("Creant l'índex sobre y.");
		String indexYWGS84 = "Create Index XJaciments_YWGS84 on Jaciments.Jaciments(YWGS84) ";
		stmt.executeUpdate(indexYWGS84);

		System.out.println("Carregant les dades de la taula de treball...");
		String select = "SELECT " +
							"id, nom, unom, poblacio, xwgs84, ywgs84, imatge, xImatge, yImatge, comentaris " +
						"FROM calcul.jaciments";
		
		Statement stmtQuery = con.createStatement();
		ResultSet rs = stmtQuery.executeQuery(select);
		while (rs.next()){
			System.out.println("Afegint el jaciment: "+rs.getInt(1)+"-"+rs.getString(2));
			String vImatge = "null";
			if (rs.getString(7)!=null && !rs.getString(7).equals("")) vImatge="'"+rs.getString(7)+"'";
			String vComentaris = "null";
			if (rs.getString(10)!=null && !rs.getString(10).equals("")) vComentaris="'"+rs.getString(10)+"'";
			
			String insert = "INSERT INTO Jaciments.jaciments" +
								"(id, " +
								"nom, " +
								"unom, " +
								"poblacio, " +
								"xwgs84, ywgs84, " +
								"imatge, xImatge, yImatge, " +
								"comentaris) " +
							"VALUES " +
								"("+rs.getInt(1)+", " +
								"'"+StringsTools.dq(rs.getString(2))+"', " +
								"'"+StringsTools.dq(rs.getString(3))+"', " +
								"'"+StringsTools.dq(rs.getString(4))+"', " +
								rs.getBigDecimal(5)+", "+rs.getBigDecimal(6)+", " +
								vImatge+", " +rs.getBigDecimal(8)+", "+rs.getBigDecimal(9)+", "+
								vComentaris+")";
			stmt.executeUpdate(insert);
		}
		rs.close();
		stmtQuery.close();
		stmt.close();
		System.out.println("S'ha creat la taula de jaciments i s'ha carregat les dades satisfactòriament.\n");

	}

	/**
	 * Crea la taula dels períodes.
	 * @param stmt
	 * @throws SQLException
	 * @throws FileNotFoundException
	 * @throws IOException
	 */
	private static void crearTaulaPeriodes(Statement stmt) throws SQLException, FileNotFoundException, IOException{
		// Taula de Jaciments
		System.out.println("Creant la taula dels Periodes Històrics.");
		String crearTaula = "CREATE TABLE Jaciments.Periodes "+
							"("+
							  "Id integer NOT NULL, "+
							  "Nom character varying(50) not null, "+
							  "UNom character varying(50) not null, "+
							  "Comentaris character varying(250), "+
							  "CONSTRAINT PKPeriodes PRIMARY KEY (Id)) "+
							  "WITH ( "+
								 "OIDS = FALSE "+
								")";
			
		stmt.executeUpdate(crearTaula);
			
		System.out.println("Afegint el comentari a la taula...");
		String comentari = "COMMENT ON TABLE Jaciments.Periodes IS 'Taula dels Periodes Històrics.'";
		stmt.executeUpdate(comentari);
			
		System.out.println("Assignant els drets d'administradors...");
		String dretsAdministracio = "GRANT ALL ON TABLE Jaciments.Periodes TO GROUP AdministraMuseu";
		stmt.executeUpdate(dretsAdministracio);

		System.out.println("Assignant els drets de lectura...");
		String dretsLectura = "GRANT SELECT ON TABLE Jaciments.Periodes TO GROUP ConsultaMuseu";
		stmt.executeUpdate(dretsLectura);

		System.out.println("Creant l'índex sobre uNom.");
		String indexUNom = "CREATE UNIQUE INDEX XU_Periodes on Jaciments.Periodes(uNom)";
		stmt.executeUpdate(indexUNom);

		System.out.println("Carregant les dades del fitxer...");
		int comptalinies=0;
		BufferedReader bf = new BufferedReader(new FileReader("Periodes.txt"));
		String registre = bf.readLine();
		while (registre!=null){
        	comptalinies++;
        	if (registre.startsWith("#")) registre=bf.readLine();
        	if (!registre.trim().equals("")){
	        	System.out.println("Afegint el període "+registre);
	        	PeriodeHistoric p = new PeriodeHistoric(registre);
	        	String insert = p.afegeix();
	        	stmt.executeUpdate(insert);
        	}
        	registre = bf.readLine();
		}
		System.out.println("S'ha creat la taula de períodes històrics i s'ha afegit "+comptalinies+" períodes històrics satisfactòriament.\n");
	}

	/**
	 * Crea la taula de la relació períodes-jaciments.
	 * @param stmt
	 * @throws SQLException
	 * @throws FileNotFoundException
	 * @throws IOException
	 */
	private static void crearTaulaPeriodesJaciments(Statement stmt) throws SQLException, FileNotFoundException, IOException{
		// Taula de Jaciments
		System.out.println("Creant la taula de la relació Periodes Històrics - Jaciments.");
		String crearTaula = "CREATE TABLE Jaciments.Periodes_Jaciments "+
							"("+
							  "Periode integer NOT NULL, "+
							  "Jaciment integer NOT NULL, "+
							  "Dates character varying(30) NOT NULL, "+
							  "Descripcio text, "+
							  "Altres text, "+
							  "Comentaris character varying(250), "+
							  "CONSTRAINT PKPeriodes_Jaciments PRIMARY KEY (Periode,Jaciment)," +
							  "CONSTRAINT FKPeriodes_Jaciments_Periode FOREIGN KEY (Periode) REFERENCES jaciments.periodes (Id) ON UPDATE RESTRICT ON DELETE RESTRICT, "+
							  "CONSTRAINT FKPeriodes_Jaciments_Jaciment FOREIGN KEY (Jaciment) REFERENCES jaciments.jaciments (Id) ON UPDATE RESTRICT ON DELETE RESTRICT "+
							  ") "+
							  "WITH ( "+
								 "OIDS = FALSE "+
								")";
			
		stmt.executeUpdate(crearTaula);
			
		System.out.println("Afegint el comentari a la taula...");
		String comentari = "COMMENT ON TABLE Jaciments.Periodes_Jaciments IS 'Taula de la relació Periodes Històrics - Jaciments.'";
		stmt.executeUpdate(comentari);
			
		System.out.println("Assignant els drets d'administradors...");
		String dretsAdministracio = "GRANT ALL ON TABLE Jaciments.Periodes_Jaciments TO GROUP AdministraMuseu";
		stmt.executeUpdate(dretsAdministracio);

		System.out.println("Assignant els drets de lectura...");
		String dretsLectura = "GRANT SELECT ON TABLE Jaciments.Periodes_Jaciments TO GROUP ConsultaMuseu";
		stmt.executeUpdate(dretsLectura);

		System.out.println("Carregant les dades del fitxer...");
		int comptalinies=0;
		BufferedReader bf = new BufferedReader(new FileReader("Periode_Jaciment.txt"));
		String registre = bf.readLine();
		while (registre!=null){
        	comptalinies++;
        	if (registre.startsWith("#")) registre=bf.readLine();
        	if (!registre.trim().equals("")){
	        	System.out.println("Afegint el període del jaciment "+registre);
	        	PeriodeJaciment p = new PeriodeJaciment(registre);
	        	String insert = p.afegeix();
	        	stmt.executeUpdate(insert);
        	}
        	registre = bf.readLine();
		}
		System.out.println("S'ha creat la taula jaciments i s'ha afegit "+comptalinies+" períodes als jaciments satisfactòriament.\n");
	}

	/**
	 * Crea la taula d'objectes
	 * @param stmt
	 * @throws SQLException
	 * @throws FileNotFoundException
	 * @throws IOException
	 */
	private static void crearTaulaObjectes(Statement stmt) throws SQLException, FileNotFoundException, IOException{
		// Taula de Jaciments
		System.out.println("Creant la taula Objectes.");
		String crearTaula = "CREATE TABLE Jaciments.Objectes "+
							"("+
							  "Periode integer NOT NULL, "+
							  "Jaciment integer NOT NULL, "+
							  "n integer NOT NULL, "+
							  "Descripcio text, "+
							  "Comentaris character varying(250), "+
							  "CONSTRAINT PKObjectes PRIMARY KEY (Periode,Jaciment,N)," +
							  "CONSTRAINT FKObjectes_Periodes_Jaciments FOREIGN KEY (Periode, Jaciment) REFERENCES Jaciments.Periodes_Jaciments (Periode,Jaciment) ON UPDATE RESTRICT ON DELETE RESTRICT "+
							  ") "+
							  "WITH ( "+
								 "OIDS = FALSE "+
								")";
			
		stmt.executeUpdate(crearTaula);
			
		System.out.println("Afegint el comentari a la taula...");
		String comentari = "COMMENT ON TABLE Jaciments.Objectes IS 'Taula dels Objectes d''un Jaciment en un període històric.'";
		stmt.executeUpdate(comentari);
			
		System.out.println("Assignant els drets d'administradors...");
		String dretsAdministracio = "GRANT ALL ON TABLE Jaciments.Objectes TO GROUP AdministraMuseu";
		stmt.executeUpdate(dretsAdministracio);

		System.out.println("Assignant els drets de lectura...");
		String dretsLectura = "GRANT SELECT ON TABLE Jaciments.Objectes TO GROUP ConsultaMuseu";
		stmt.executeUpdate(dretsLectura);

		System.out.println("Carregant les dades del fitxer...");
		int comptalinies=0;
		BufferedReader bf = new BufferedReader(new FileReader("Objectes.txt"));
		String registre = bf.readLine();
		while (registre!=null){
        	comptalinies++;
        	if (registre.startsWith("#")) registre=bf.readLine();
        	if (!registre.trim().equals("")){
	        	System.out.println("Afegint l'Objecte "+registre);
	        	Objecte p = new Objecte(registre);
	        	String insert = p.afegeix();
	        	stmt.executeUpdate(insert);
        	}
        	registre = bf.readLine();
		}
		System.out.println("S'ha creat la taula d'objectes i s'ha afegit "+comptalinies+" objectes satisfactòriament.\n");
	}

	/**
	 * Crea la taula d'imatges.
	 * @param stmt
	 * @throws SQLException
	 * @throws FileNotFoundException
	 * @throws IOException
	 */
	private static void crearTaulaImatges(Statement stmt) throws SQLException, FileNotFoundException, IOException{
		// Taula de Jaciments
		System.out.println("Creant la taula d'Imatges.");
		String crearTaula = "CREATE TABLE Jaciments.Imatges "+
							"("+
							  "Periode integer NOT NULL, "+
							  "Jaciment integer NOT NULL, "+
							  "Objecte integer NOT NULL, "+
							  "n integer NOT NULL, "+
							  "url character varying(250) NOT NULL, "+
							  "xImatge integer, "+
							  "yImatge integer, "+
							  "Comentaris character varying(250), "+
							  "CONSTRAINT PKImatges PRIMARY KEY (Periode,Jaciment,Objecte,N)," +
							  "CONSTRAINT FKImatges_Objectes FOREIGN KEY (Periode, Jaciment, Objecte) REFERENCES Jaciments.Objectes (Periode,Jaciment,n) ON UPDATE RESTRICT ON DELETE RESTRICT, "+
							  "CONSTRAINT CCJacimentsIntegritatImatge CHECK " +
		                        "((url is null and xImatge is null and yImatge is null) or" +
		                        "(url is not null and xImatge is not null and yImatge is not null))" +
							  ") "+
							  "WITH ( "+
								 "OIDS = FALSE "+
								")";
			
		
		stmt.executeUpdate(crearTaula);
		
		System.out.println("Afegint el comentari a la taula...");
		String comentari = "COMMENT ON TABLE Jaciments.Imatges IS 'Taula de les imatges dels Objectes.'";
		stmt.executeUpdate(comentari);
			
		System.out.println("Assignant els drets d'administradors...");
		String dretsAdministracio = "GRANT ALL ON TABLE Jaciments.Imatges TO GROUP AdministraMuseu";
		stmt.executeUpdate(dretsAdministracio);

		System.out.println("Assignant els drets de lectura...");
		String dretsLectura = "GRANT SELECT ON TABLE Jaciments.Imatges TO GROUP ConsultaMuseu";
		stmt.executeUpdate(dretsLectura);

		System.out.println("Carregant les dades del fitxer...");
		int comptalinies=0;
		BufferedReader bf = new BufferedReader(new FileReader("Imatges_Objecte.txt"));
		String registre = bf.readLine();
		while (registre!=null){
        	comptalinies++;
        	if (registre.startsWith("#")) registre=bf.readLine();
        	if (!registre.trim().equals("")){
	        	System.out.println("Afegint la imatge "+registre);
	        	Imatge p = new Imatge(registre);
	        	String insert = p.afegeix();
	        	stmt.executeUpdate(insert);
        	}
        	registre = bf.readLine();
		}
		System.out.println("S'ha creat la taula d'imatges i s'ha afegit "+comptalinies+" imatges satisfactòriament.\n");
	}

}
