package net.nitus.museu.scripts;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.sql.Statement;

import net.nitus.dbcon.SQL;

/**
 * Crea la taula de treball per la migració dels jaciments.
 * @author xnavarro
 *
 */
public class TaulaMigraJaciments {

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		Connection museu = null;
		
		try {
			System.out.println("Carregant el driver jdbc.");
			Class.forName("org.postgresql.Driver");
			System.out.println("Connecant a la base de dades.");
			museu = DriverManager.getConnection("jdbc:postgresql://localhost:5432/Museu","museu","patata");
			museu.setAutoCommit(false);
			Statement stmt = museu.createStatement();
			
			System.out.println("Creant l'Esquema de treball.");
			String crearEsquema = "CREATE SCHEMA Calcul";
			stmt.executeUpdate(crearEsquema);
			
			System.out.println("Assignat drets d'accés a l'Esquema de treball.");
			String dretsEsquema = "GRANT ALL ON SCHEMA Calcul TO GROUP administramuseu";
			stmt.executeUpdate(dretsEsquema);

			System.out.println("Creant la taula de migració dels jaciments.");
			String crearTaula = "CREATE TABLE Calcul.Jaciments"+
								"("+
								  "Id integer NOT NULL, "+
								  "Nom character varying(50) not null, "+
								  "UNom character varying(50) not null, "+
								  "Poblacio character varying(50) not null, "+
								  "Codificacio character varying(15) not null, "+
								  "X numeric(9,2) NOT NULL, "+
								  "Y numeric(9,2) NOT NULL, "+
								  "GLat integer NOT NULL, "+
								  "MLat integer NOT NULL, "+
								  "SLat numeric(4,2) NOT NULL, "+ 
								  "GLong integer NOT NULL, "+
								  "MLong integer NOT NULL, "+
								  "SLong numeric(4,2) NOT NULL, "+
								  "XWGS84 numeric(9,7) NOT NULL, "+
								  "YWGS84 numeric(9,7) NOT NULL, "+
								  "Imatge character varying(255), "+
								  "xImatge integer, "+
								  "yImatge integer, "+
								  "Comentaris character varying(255), "+
								  "CONSTRAINT PKCalcJaciments PRIMARY KEY (Id), "+
								  "CONSTRAINT CCCalcJacimentsIntegritatImatge CHECK " +
						                        "((imatge is null and xImatge is null and yImatge is null) or" +
						                        "(imatge is not null and xImatge is not null and yImatge is not null)" +
						                          "), "+
								  "CONSTRAINT CCCalcJacimentsIntegritat CHECK " +
									            "(GLat>=0 and GLat<=60 and " +
									            " MLat>=0 and MLat<=60 and " +
									            " SLat>=0 and SLat<=60 and " +
									            " GLong>=0 and GLong<=60 and " +
									            " MLong>=0 and MLong<=60 and " +
									            " SLong>=0 and SLong<=60) "+
									") "+
									"WITH ( "+
									 "OIDS = FALSE "+
									")";
				
			stmt.executeUpdate(crearTaula);
				
			System.out.println("Afegint el comentari a la taula...");
			String comentari = "COMMENT ON TABLE Calcul.Jaciments IS 'Taula de treball per convertir les coordenades dels jaciments.'";
			stmt.executeUpdate(comentari);
				
			System.out.println("Assignant els drets d'administradors...");
			String dretsAdministracio = "GRANT ALL ON TABLE Calcul.Jaciments TO GROUP AdministraMuseu";
			stmt.executeUpdate(dretsAdministracio);
				
			System.out.println("Creant l'índex sobre x.");
			String indexX = "Create Index XCalcJaciments_X on Calcul.Jaciments(X) ";
			stmt.executeUpdate(indexX);

			System.out.println("Creant l'índex sobre y.");
			String indexY = "Create Index XCalcJaciments_Y on Calcul.Jaciments(Y) ";
			stmt.executeUpdate(indexY);

			System.out.println("Creant l'índex sobre latitud.");
			String indexLatitud = "Create Index XCalcJaciments_Latitud on Calcul.Jaciments(GLat, MLat, SLat ) ";
			stmt.executeUpdate(indexLatitud);

			System.out.println("Creant l'índex sobre longitud.");
			String indexLongitud = "Create Index XCalcJaciments_Longitud on Calcul.Jaciments(GLong, MLong, SLong ) ";
			stmt.executeUpdate(indexLongitud);

			System.out.println("Creant l'índex sobre xWGS84.");
			String indexXWGS84 = "Create Index XCalcJaciments_XWGS84 on Calcul.Jaciments(XWGS84) ";
			stmt.executeUpdate(indexXWGS84);

			System.out.println("Creant l'índex sobre y.");
			String indexYWGS84 = "Create Index XVertexCalcJaciments_YWGS84 on Calcul.Jaciments(YWGS84) ";
			stmt.executeUpdate(indexYWGS84);



			stmt.close();
			museu.commit();
			System.out.println("Tancant la connexió.");
			museu.close();
		} 
		catch (SQLException e){
			System.err.println("Error SQL: "+e.getErrorCode()+"-"+e.getSQLState()+": "+e.getLocalizedMessage());
			SQL.rollbackTry(museu);
			try {
				if (museu!=null && !museu.isClosed())museu.close();
			} catch (SQLException e1) {
				System.out.println("Error tancant la connexió a la base de dades.");
				e1.printStackTrace();
			}
			System.exit(-1);
		}
		catch (Exception e) {
			System.err.println("No s'ha localitzat el Driver JDBC.");
			System.exit(-1);
		} 

	}

}
