package net.nitus.museu.scripts;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.sql.Statement;

import net.nitus.dbcon.SQL;
import net.nitus.widgets.StringsTools;

public class TaulesLimit {

	/**
	 * Crea les taules dels municipis i de les comarques.
	 * Els registres equivalen a un shape del fitxer shapefile.
	 * @param args
	 */
	public static void main(String[] args) {
		Connection museu = null;
		String[] limits = {"Municipis","Comarques"};
		try {
			System.out.println("Carregant el driver jdbc.");
			Class.forName("org.postgresql.Driver");
			System.out.println("Connecant a la base de dades.");
			museu = DriverManager.getConnection("jdbc:postgresql://localhost:5432/Museu","museu","patata");
			museu.setAutoCommit(false);
			Statement stmt = museu.createStatement();
			for (int i=0; i<limits.length; i++){
				System.out.println("Creant la taula dels límits administratius de "+limits[i]+".");
				String crearTaula = "CREATE TABLE icc."+limits[i]+" "+
									"("+
									  "Id serial NOT NULL, "+ 
									  "Nom character varying(100) NOT NULL, "+ 
									  "UNom character varying(100) NOT NULL , "+
									  "XFrom numeric(9,2) NOT NULL, "+
									  "YFrom numeric(9,2) NOT NULL, "+
									  "GLatFrom integer NOT NULL, "+
									  "MLatFrom integer NOT NULL, "+
									  "SLatFrom numeric(4,2) NOT NULL, "+ 
									  "GLongFrom integer NOT NULL, "+
									  "MLongFrom integer NOT NULL, "+
									  "SLongFrom numeric(4,2) NOT NULL, "+
									  "XFromWGS84 numeric(9,7) NOT NULL, "+
									  "YFromWGS84 numeric(9,7) NOT NULL, "+
									  "XTo numeric(9,2) NOT NULL, "+
									  "YTo numeric(9,2) NOT NULL, "+
									  "GLatTo integer NOT NULL, "+
									  "MLatTo integer NOT NULL, "+
									  "SLatTo numeric(4,2) NOT NULL, "+ 
									  "GLongTo integer NOT NULL, "+
									  "MLongTo integer NOT NULL, "+
									  "SLongTo numeric(4,2) NOT NULL, "+ 
									  "XToWGS84 numeric(9,7) NOT NULL, "+
									  "YToWGS84 numeric(9,7) NOT NULL, "+
									  "nVertexs integer NOT NULL, "+
									  "Comentaris character varying(255), "+
									  "CONSTRAINT PK"+limits[i]+" PRIMARY KEY (Id), "+
									  "CONSTRAINT X"+limits[i]+"Unic UNIQUE (UNom), "+
									  "CONSTRAINT CC"+limits[i]+"Integritat CHECK " +
									            "(GLatFrom>=0 and GLatFrom<=60 and " +
									            " MLatFrom>=0 and MLatFrom<=60 and " +
									            " SLatFrom>=0 and SLatFrom<=60 and " +
									            " GLongFrom>=0 and GLongFrom<=60 and " +
									            " MLongFrom>=0 and MLongFrom<=60 and " +
									            " SLongFrom>=0 and SLongFrom<=60 and " +
									            " GLatTo>=0 and GLatTo<=60 and " +
									            " MLatTo>=0 and MLatTo<=60 and " +
									            " SLatTo>=0 and SLatTo<=60 and " +
									            " GLongTo>=0 and GLongTo<=60 and " +
									            " MLongTo>=0 and MLongTo<=60 and " +
									            " SLongTo>=0 and SLongTo<=60" +

									            ") "+
									") "+
									"WITH ( "+
									 "OIDS = FALSE "+
									")";
				
				stmt.executeUpdate(crearTaula);
				
				System.out.println("Afegint el comentari a la taula...");
				String comentari = "COMMENT ON TABLE icc."+limits[i]+" IS 'Taula amb els límits administratius de "+limits[i]+"'";
				stmt.executeUpdate(comentari);
				
				System.out.println("Assignant els drets d'administradors...");
				String dretsAdministracio = "GRANT ALL ON TABLE icc."+limits[i]+" TO GROUP AdministraMuseu";
				stmt.executeUpdate(dretsAdministracio);
				
				System.out.println("Assignant els drets de lectura...");
				String dretsLectura = "GRANT SELECT ON TABLE icc."+limits[i]+" TO GROUP ConsultaMuseu";
				stmt.executeUpdate(dretsLectura);

				System.out.println("Afegim el registre null.");
				String addNull = "INSERT INTO icc."+limits[i]+"" +
						                "( "+
						                "Id, " +
						                "Nom, " +
						                "UNom, " +
						                "XFrom, YFrom, " +
						                "GLatFrom, MLatFrom, SLatFrom, GLongFrom, MLongFrom, SLongFrom, " +
						                "XFromWGS84, yFromWGS84, "+ 
						                "XTo, YTo, "+
						                "GLatTo, MLatTo, SLatTo, GLongTo, MLongTo, SLongTo, " +
						                "XToWGS84, YToWGS84, "+ 
						                "nVertexs, Comentaris) "+
						         "VALUES (" +
						                "0, " +
						                "'<"+limits[i].substring(0,limits[i].length()-1)+">', " +
						                "'<"+StringsTools.U(limits[i].substring(0,limits[i].length()-1))+">'," +
						                "0, 0, " +
						                "0, 0, 0, 0, 0, 0, " +
						                "0, 0, "+
						                "0, 0, "+
						                "0, 0, 0, 0, 0, 0, "+
						                "0, 0, "+
						                "0, null)";
	 			stmt.executeUpdate(addNull);
	 			System.out.println("S'ha creat la taula de "+limits[i]+" satisfactòriament.");
	 			System.out.println("");
				
			}

			stmt.close();
			museu.commit();
			System.out.println("Tancant la connexió.");
			museu.close();
		} 
		catch (SQLException e){
			System.err.println("Error SQL: "+e.getErrorCode()+"-"+e.getSQLState()+": "+e.getLocalizedMessage());
			SQL.rollbackTry(museu);
			try {
				if (museu!=null && !museu.isClosed())museu.close();
			} catch (SQLException e1) {
				System.out.println("Error tancant la connexió a la base de dades.");
				e1.printStackTrace();
			}
			System.exit(-1);
		}
		catch (Exception e) {
			System.err.println("No s'ha localitzat el Driver JDBC.");
			System.exit(-1);
		} 

	}

}
