package net.nitus.museu.scripts;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.sql.Statement;

import net.nitus.dbcon.SQL;

public class TaulesVertexs {

	/**
	 * Crea la taula de treball per la migració dels vèrtexs.
	 * @param args
	 */
	public static void main(String[] args) {
		Connection museu = null;
		String[] limits = {"Municipis","Comarques"};
		try {
			System.out.println("Carregant el driver jdbc.");
			Class.forName("org.postgresql.Driver");
			System.out.println("Connecant a la base de dades.");
			museu = DriverManager.getConnection("jdbc:postgresql://localhost:5432/Museu","museu","patata");
			museu.setAutoCommit(false);
			Statement stmt = museu.createStatement();
			for (int i=0; i<limits.length; i++){
				System.out.println("Creant la taula dels Vertexs.");
				String crearTaula = "CREATE TABLE icc.Vertexs"+limits[i]+""+
									"("+
									  ""+(i==0? "Municipi":"Comarca")+" integer NOT NULL, "+
									  "N integer NOT NULL, "+ 
									  "EsRing boolean NOT NULL, "+
									  "X numeric(9,2) NOT NULL, "+
									  "Y numeric(9,2) NOT NULL, "+
									  "GLat integer NOT NULL, "+
									  "MLat integer NOT NULL, "+
									  "SLat numeric(4,2) NOT NULL, "+ 
									  "GLong integer NOT NULL, "+
									  "MLong integer NOT NULL, "+
									  "SLong numeric(4,2) NOT NULL, "+
									  "XWGS84 numeric(9,7) NOT NULL, "+
									  "YWGS84 numeric(9,7) NOT NULL, "+
									  "Comentaris character varying(255), "+
									  "CONSTRAINT PKVertexs"+limits[i]+" PRIMARY KEY ("+(i==0? "Municipi":"Comarca")+", N), "+
									  "CONSTRAINT FKVertexs"+limits[i]+"_"+limits[i]+" FOREIGN KEY ("+(i==0? "Municipi":"Comarca")+") REFERENCES icc."+limits[i]+" (Id) ON UPDATE RESTRICT ON DELETE RESTRICT, "+
									  "CONSTRAINT CCVertexs"+limits[i]+"Integritat CHECK " +
									            "(GLat>=0 and GLat<=60 and " +
									            " MLat>=0 and MLat<=60 and " +
									            " SLat>=0 and SLat<=60 and " +
									            " GLong>=0 and GLong<=60 and " +
									            " MLong>=0 and MLong<=60 and " +
									            " SLong>=0 and SLong<=60) "+
									") "+
									"WITH ( "+
									 "OIDS = FALSE "+
									")";
				
				stmt.executeUpdate(crearTaula);
				
				System.out.println("Afegint el comentari a la taula...");
				String comentari = "COMMENT ON TABLE icc.Vertexs"+limits[i]+" IS 'Taula amb els vertexs dels límits administratius de "+limits[i]+"'";
				stmt.executeUpdate(comentari);
				
				System.out.println("Assignant els drets d'administradors...");
				String dretsAdministracio = "GRANT ALL ON TABLE icc.Vertexs"+limits[i]+" TO GROUP AdministraMuseu";
				stmt.executeUpdate(dretsAdministracio);
				
				System.out.println("Assignant els drets de lectura...");
				String dretsLectura = "GRANT SELECT ON TABLE icc.Vertexs"+limits[i]+" TO GROUP ConsultaMuseu";
				stmt.executeUpdate(dretsLectura);

				System.out.println("Creant l'índex sobre x.");
				String indexX = "Create Index XVertex"+limits[i]+"_X on icc.Vertexs"+limits[i]+"(X) ";
				stmt.executeUpdate(indexX);

				System.out.println("Creant l'índex sobre y.");
				String indexY = "Create Index XVertex"+limits[i]+"_Y on icc.Vertexs"+limits[i]+"(Y) ";
				stmt.executeUpdate(indexY);

				System.out.println("Creant l'índex sobre latitud.");
				String indexLatitud = "Create Index XVertex"+limits[i]+"_Latitud on icc.Vertexs"+limits[i]+"(GLat, MLat, SLat ) ";
				stmt.executeUpdate(indexLatitud);

				System.out.println("Creant l'índex sobre longitud.");
				String indexLongitud = "Create Index XVertex"+limits[i]+"_Longitud on icc.Vertexs"+limits[i]+"(GLong, MLong, SLong ) ";
				stmt.executeUpdate(indexLongitud);

				System.out.println("Creant l'índex sobre xWGS84.");
				String indexXWGS84 = "Create Index XVertex"+limits[i]+"_XWGS84 on icc.Vertexs"+limits[i]+"(XWGS84) ";
				stmt.executeUpdate(indexXWGS84);

				System.out.println("Creant l'índex sobre y.");
				String indexYWGS84 = "Create Index XVertex"+limits[i]+"_YWGS84 on icc.Vertexs"+limits[i]+"(YWGS84) ";
				stmt.executeUpdate(indexYWGS84);

			}

			stmt.close();
			museu.commit();
			System.out.println("Tancant la connexió.");
			museu.close();
		} 
		catch (SQLException e){
			System.err.println("Error SQL: "+e.getErrorCode()+"-"+e.getSQLState()+": "+e.getLocalizedMessage());
			SQL.rollbackTry(museu);
			try {
				if (museu!=null && !museu.isClosed())museu.close();
			} catch (SQLException e1) {
				System.out.println("Error tancant la connexió a la base de dades.");
				e1.printStackTrace();
			}
			System.exit(-1);
		}
		catch (Exception e) {
			System.err.println("No s'ha localitzat el Driver JDBC.");
			System.exit(-1);
		} 

	}

}
