/*
 * Created on 08-jun-2005 13:19:08
 *
 * Fitxer: SQL.java
 * Autor:  xnavarro
 */
package net.nitus.dbcon;

import java.awt.Toolkit;
import java.math.BigDecimal;
import java.sql.*;
import net.nitus.missatges.MsgUtils;
import net.nitus.widgets.Strings;

/**
 * @author xnavarro
 *
 * @version
 * 08-jun-2005 13:19:08
 */
public class SQL {

    /**
     * 
     * Autor: xnavarro
     * 
     * Paràmetres: 
     * 
     */
    public SQL() {
    }

    /**
     * Executa la instrucció sqlInstruct
     *
     * @param con
     * @param sqlInstruct
     * @return  true si s'ha executat satisfactòriament
     * 			false si hi ha hagut algun error.
     */
    public static boolean sqlTry(Connection con, String sqlInstruct){
	    try{
	        Statement stmt = con.createStatement();
	        stmt.executeUpdate(sqlInstruct);
	        stmt.close();
	    }
	    catch(SQLException e){
	        Toolkit.getDefaultToolkit().beep();
	        MsgUtils.showError(null,"Error d'SQL:",e.getErrorCode()+"-"+e.getSQLState()+":\n"+e.getMessage(),true);
	        if (!SQL.rollbackTry(con)) return false;
	        return false;
	    }
        return true;
    }

    /**
     * Intenta executar una instrucció sql
     * @param con La connexió a la base de dades
     * @param sqlInstruct La instrucció que s'ha d'executar.
     * @param noTrobatPermes cert si es considera no trobat correcte, fals si no trobat és incorrecte
     * @return Enter amb -100.-si no s'ha trobat, 0.- Si ha anat bé, El codi d'error si hi ha hagut algun altre error.
     */
    public static int sqlTry(Connection con, String sqlInstruct, boolean noTrobatPermes){
	    try{
	        Statement stmt = con.createStatement();
	        int filesAfectades = stmt.executeUpdate(sqlInstruct);
	        stmt.close();
	        if (filesAfectades == 0 && !noTrobatPermes){
	            return -100;
	        }
	        return 0;
	    }
	    catch (SQLException e){
	        Toolkit.getDefaultToolkit().beep();
	        MsgUtils.showError(null,"Error d'SQL:",e.getErrorCode()+"-"+e.getSQLState()+":\n"+e.getMessage(),true);
	        if (!SQL.rollbackTry(con)) return e.getErrorCode();
	        return e.getErrorCode();
	    }
    }

    /**
     * Intenta executar una instrucció sql.
     * @param con La connexió a la base de dades
     * @param sqlInstruct La instrucció sql que s'ha d'executar
     * @param errors Una cadena amb la llista d'errors permesos
     * @param noTrobatPermes Cert si no trobat no es considera error, fals en cas contrari.
     * @return int amb 0.- si ha anat bé, -100 si no s'ha trobat cap registre, el codi d'error en cas contrari
     */
    public static int sqlTry(Connection con, String sqlInstruct, String errors, boolean noTrobatPermes){
	    try{
	        Statement stmt = con.createStatement();
	        int filesAfectades = stmt.executeUpdate(sqlInstruct);
	        stmt.close();
	        if (filesAfectades == 0 && !noTrobatPermes){
	            return -100;
	        }
	        return 0;
	    }
	    catch (SQLException e){
	        if (Strings.pos(e.getErrorCode()+"",errors)!=-1) return e.getErrorCode();
	        Toolkit.getDefaultToolkit().beep();
	        MsgUtils.showError(null,"Error d'SQL:",e.getErrorCode()+"-"+e.getSQLState()+":\n"+e.getMessage(),true);
	        if (!SQL.rollbackTry(con)) return e.getErrorCode();
	        return e.getErrorCode();
	    }
    }
    
    /**
     * Intenta executar un commit de la transacció
     * @param con La connexió que té la transacció activa
     * @return cert si ha funcionat, fals en cas contrari
     */
    public static boolean commitTry(Connection con){
        try {
            con.commit();
            return true;
        } catch (Exception e) {
            Toolkit.getDefaultToolkit().beep();
            MsgUtils.showError(null,"Error d'SQL:","S'ha perdut la connexió amb la base de dades. Tanqueu l'aplicació i torneu-la a obrir.",true);
            return false;
        }
    }
    
    /**
     * Intenta fer un roolback de la transició
     * @param con La connexió que té la transició activa
     * @return Cert si ha funcionat, fals en cas contrari
     */
    public static boolean rollbackTry(Connection con){
        try {
            con.rollback();
            return true;
        } catch (Exception e) {
            Toolkit.getDefaultToolkit().beep();
            MsgUtils.showError(null,"Error d'SQL:","S'ha perdut la connexió amb la base de dades. Tanqueu l'aplicació i torneu-la a obrir.",true);
            return false;
        }
    }

    /**
     * Retorna una cadena a partir d'una columna d'un result set, el
     * valor del qual és transformat a "" en cas que sigui null.
     * @param rs El resultset que conté la informació
     * @param columna La columna que es vol obtenir la dada
     * @return "" si la columna té un null, el valor guardat si no és null.
     * @throws SQLException
     */
	public static String getStringNoNull(ResultSet rs, int columna) throws SQLException{
	    if (rs.getString(columna) == null) return "";
	    else return rs.getString(columna); 
	}

	/**
     * Retorna un BigDecimal a partir d'una cadena o null si la cadena no té un número
	 * @param cadena La cadena que conté el número
	 * @return null si la cadena no tenia un número, el BigDecimal del número si la cadena tenia un numero.
	 */
    public static BigDecimal getBigDecimal(String cadena){
        try {
            return new BigDecimal(Strings.strip(cadena));
        }
        catch (NumberFormatException e) {
            return null;
        }
    }

	/**
     * Retorna un Integer a partir d'una cadena o null si la cadena no té un número
	 * @param cadena La cadena que conté el número
	 * @return null si la cadena no tenia un número, l'Integer del número si la cadena tenia un numero.
	 */
    public static Integer getInteger(String cadena){
        try {
            return new Integer(Strings.strip(cadena));
        }
        catch (NumberFormatException e) {
            return null;
        }
    }

}
