package net.nitus.museu.context;

import java.math.BigDecimal;

/**
 * Classe que implementa una coordenada geogràfica WGS84 en Sexagesimal
 * i manté el seu equivalent en decimal.
 * @author xnavarro
 *
 */
public class CoordenadaGeografica {
	protected Integer graus = new Integer(0);
	protected Integer minuts = new Integer(0);
	protected BigDecimal segons = new BigDecimal(0);
	protected BigDecimal wgs84 = new BigDecimal(0);
	
	/**
	 * El constructor null;
	 */
	public CoordenadaGeografica(){
		this.graus = new Integer(0);
		this.minuts = new Integer(0);
		this.segons = new BigDecimal(0);
		this.wgs84 = new BigDecimal(0);
	}
	
	/**
	 * Constructor a partir dels tipus primitius i BigDecimal per segons
	 * @param graus	Els graus
	 * @param minuts Els minuts
	 * @param segons Els segons
	 */
	public CoordenadaGeografica(int graus, int minuts, BigDecimal segons){
		this.graus=graus;
		this.minuts=minuts;
		this.segons=segons;
		this.wgs84 = toDecimal();
	}

	/**
	 * Mètode que implementa la conversió de Sexagesimal a decimal
	 * @return
	 */
	public BigDecimal toDecimal(){
		BigDecimal cgraus = new BigDecimal(graus).setScale(10);
		BigDecimal cminuts = new BigDecimal(minuts).setScale(10);
		cminuts = cminuts.divide(new BigDecimal(60), BigDecimal.ROUND_HALF_UP);
		BigDecimal csegons = new BigDecimal(segons.toString()).setScale(10);
		csegons = csegons.divide(new BigDecimal(3600), BigDecimal.ROUND_HALF_UP);
		return cgraus.add(cminuts).add(csegons);
	}
	
	/**
	 * Sobreescrivim el mètode toString.
	 */
	public String toString(){
		return graus+" "+minuts+" "+segons+" "+wgs84;
	}

	//Els getters i els setters.
	public Integer getGraus() {
		return graus;
	}

	public void setGraus(Integer graus) {
		this.graus = graus;
		this.wgs84 = toDecimal();
	}

	public Integer getMinuts() {
		return minuts;
	}

	public void setMinuts(Integer minuts) {
		this.minuts = minuts;
		this.wgs84 = toDecimal();
	}

	public BigDecimal getSegons() {
		return segons;
	}

	public void setSegons(BigDecimal segons) {
		this.segons = segons;
		this.wgs84 = toDecimal();
	}

	public BigDecimal getWgs84() {
		return wgs84;
	}

	@SuppressWarnings("unused")
	private void setWgs84(BigDecimal wgs84) {
		this.wgs84 = wgs84;
	}
	
}
