/*
 * Strings.java
 * Strings
 * Creat per: Francesc Ros�s Albiol
 * Modificat per: Xavier Navarro Ferrer
 * El dia 19-jul-04 a les 17:36:20
 * Projecte: Llibreries; Paquet: net.nitus.widgets
 */
package net.nitus.widgets;

import java.util.StringTokenizer;

/**
 * Strings
 * Creat per: Francesc Roses
 * Modificat per: Xavier Navarro
 * El dia 19-jul-04 a les 17:36:20
 * Projecte: Llibreries; Paquet: net.nitus.widgets
 * 
 * 19/07/2004.- Funci� Strip. En lloc de fer servir un for, he 
 *              fet servir un while.
 * 19/07/2004.- Funci� primeraMaj�scula.
 *  
 * The StringTools class contains several useful String functions from Rexx.
 * These functions are available from NetRexx (by Mike Cowlishaw as well as
 * Rexx) too. Rexx and NetRexx are very interesting languages. In NetRexx, these
 * string functions apply to Rexx strings not to <code>java.lang.String</code>
 * s. Frequently I need this useful functions to handle
 * <code>java.lang.String</code>s. If I convert my strings to Rexx strings, I
 * apply the function and I convert the resulting Rexx string to
 * <code>java.lang.String</code> I'll leave to use Rexx functions at second
 * attempt. So, I decided to code most of this functions from scratch as static
 * functions.
 * <p>
 * 10/09/01: Strip returns an empty String ("") instead of null if the result
 * string is empty. Creation date: (18/8/99 10:18:23)
 * <p>
 * Modified
 * </p>
 * <dl>
 * <dt>11/06/99 <code>copies(char, int)</code></dt>
 * <dd><code>if (times == 0) return null</code> becomes
 * <code>[...] return ""</code> like in Rexx.</dd>
 * <dt>11/06/99 <code>copies(String, int)</code></dt>
 * <dd>Added <code>if (times == 0) return ""</code> to do the same as in
 * <code>copies(char, int)</code>.</dd>
 * <dt>11/06/99 <code>right(String, int, char)</code></dt>
 * <dd>Added <code>if (len = s.length()) return s</code>.</dd>
 * <dt>10/04/04</dt>
 * <dd>Revised javadoc.</dd>
 * </dl>
 * 
 * @author Francesc Ros�s (www.froses.com)
 */

public class Strings {

	/**
	 * 
	 */
	public Strings() {
		super();
	}

	/**
	 * center(String,int)
	 * Returns a string of length <code>len</code> with <code>s</code>
	 * centered in it, with pad characters added as necessary to make up
	 * <code>len</code>. Creation date: (18/8/99 10:36:04)
	 * 
	 * @return A <code>String</code> with <code>s</code> centered in it.
	 * @param s
	 *          The <code>String</code> to be centered.
	 * @param len
	 *          int The length of final <code>String</code>
	 * @see StringTools#center(String, int, char)
	 */
	public static String center(String s, int len) {
		return center(s, len, ' ');
	}

	/**
	 * center(String,int,char)
	 * Returns a string of length <code>len</code> with <code>s</code>
	 * centered in it, with pad characters added as necessary to make up
	 * <code>len</code>. The len must be a positive whole number or zero. The
	 * default pad character is blank. If the string <code>s</code> is longer
	 * than <code>len</code>, it is truncated at both ends to fit. If an odd
	 * number of characters are truncated or added, the right-hand end loses or
	 * gains one more character than the left-hand end.
	 * <p>
	 * Here are some examples:
	 * <p>
	 * <code>
	 * center("abc",7)             // "  ABC  "
	 * center("abc",8,'-')         // "--ABC---"
	 * center("The blue sky",8)    // "e blue s"
	 * center("The blue sky",7)    // "e blue "
	 * </code>
	 * <note>To avoid errors because of the difference between British and
	 * American spellings, this function can be called either <code>centre</code>
	 * or <code>center</code>. Creation date: (18/8/99 10:34:05)
	 * 
	 * @return A <code>String</code> with <code>s</code> centered in it.
	 * @param s
	 *          The <code>String</code> to be centered.
	 * @param len
	 *          int The length of final <code>String</code>
	 * @param pad
	 *          The padding character
	 */
	public static String center(String s, int len, char pad) {
		if (len < 0)
			throw new IllegalArgumentException(
					"len must be a positive whole number or zero");
		if (s == null)
			return s;
		int sLen = s.length();
		if (sLen == 0)
			return s;
		if (len <= sLen) {
			int mid = (sLen - len) / 2;
			int rem = (sLen - len) % 2;
			return s.substring(mid, (sLen - mid) - rem);
		}
		int mid = (len - sLen) / 2;
		int rem = (len - sLen) % 2;
		String lr = copies(pad, mid);
		String res = lr + s + lr;
		if (rem > 0)
			res = res + copies(pad, rem);
		return res;
	}

	/**
	 * Returns a string of length len with s centered in it, with pad characters
	 * added as necessary to make up len. Creation date: (18/8/99 10:38:13)
	 * 
	 * @return A <code>String</code> with <code>s</code> centered in it.
	 * @param s
	 *          The <code>String</code> to be centered.
	 * @param len
	 *          int The length of final <code>String</code>
	 * @return A <code>String</code> with <code>s</code> centered in it.
	 * @see StringTools#center(String, int, char)
	 */
	public static String centre(String s, int len) {
		return center(s, len);
	}

	/**
	 * Returns a string of length len with s centered in it, with pad characters
	 * added as necessary to make up len. Creation date: (18/8/99 10:37:11)
	 * 
	 * @return A <code>String</code> with <code>s</code> centered in it.
	 * @param s
	 *          The <code>String</code> to be centered.
	 * @param len
	 *          int The length of final <code>String</code>
	 * @param pad
	 *          The padding character
	 * @return A <code>String</code> with <code>s</code> centered in it.
	 * @see StringTools#center(String, int, char)
	 */
	public static String centre(String s, int len, char pad) {
		return center(s, len, pad);
	}

	/**
	 * changeStr(String, String, String)
	 * Replaces all occurrences of <code>needle</code> in <code>haystack</code>
	 * with <code>newneedle</code>.
	 * <p>
	 * Examples:
	 * <p>
	 * <code>
	 * StringTools.changeStr("1","101100","")     -     '000'<br>
	 * StringTools.changeStr("1","101100","X")    -     'X0XX00<p>'
	 * </code> Creation date: (19/8/99 9:13:49)
	 * 
	 * @return A new <code>String</code> with the occurrences of
	 *         <code>needle</code> replaced
	 * @param needle
	 *          Substring to be replaced
	 * @param haystack
	 *          String to be changed
	 * @param newneedle
	 *          Substring to replace <code>needle</code>
	 */
	public static String changeStr(String needle, String haystack, String newneedle) {
		StringBuffer result = new StringBuffer();
		int tempy;
		int tempx = 0;
		while (true) {
			tempy = haystack.indexOf(needle, tempx);
			if (tempy == -1)
				break;
			String s = haystack.substring(tempx, tempx + (tempy - tempx));
			result.append(s);
			result.append(newneedle);
			tempx = tempy + needle.length();
		} // while true
		String s = haystack.substring(tempx);
		result.append(s);
		return result.toString();
	}

	/**
	 * Returns -1 if the strings, string1 and string2, are identical. Otherwise,
	 * returns the position of the first character that does not match. The
	 * shorter string is padded with blank.
	 * <p>
	 * 
	 * @return -1 if identical or the position of the first character that does
	 *         not match
	 * @param s1
	 *          First element of compare
	 * @param s2
	 *          Second element of compare
	 * @see StringTools#compare(String, String, char)
	 */
	public static int compare(String s1, String s2) {
		compare(s1, s2, ' ');
		return 0;
	}

	/**
	 * Returns -1 if the strings, string1 and string2, are identical. Otherwise,
	 * returns the position of the first character that does not match. The
	 * shorter string is padded on the right with pad if necessary. The default
	 * pad character is a blank. Here are some examples:
	 * <p>
	 * <code>
	 * COMPARE('abc','abc')         -     -1
	 * COMPARE('abc','ak')          -      1
	 * COMPARE('ab ','ab')          -     -1
	 * COMPARE('ab ','ab',' ')      -     -1
	 * COMPARE('ab ','ab','x')      -      2
	 * COMPARE('ab-- ','ab','-')    -      4
	 * </code>
	 * Creation date: (19/8/99 12:25:00)
	 * 
	 * @return -1 if identical or the position of the first character that does
	 * @param s1
	 *          First element of compare
	 * @param s2
	 *          Second element of compare
	 * @param pad
	 *          Padding for the shorter string
	 */

	public static int compare(String s1, String s2, char pad) {
		int result = -1;
		if (s1.equals(s2))
			return -1;
		if (s1.length() < s2.length())
			s1 = left(s1, s2.length(), pad);
		else if (s1.length() > s2.length())
			s2 = left(s2, s1.length(), pad);
		for (int i = 0; i < s1.length(); i++) {
			char c1 = s1.charAt(i);
			char c2 = s2.charAt(i);
			if (c1 != c2) {
				result = i;
				break;
			}
		}
		return result;
	}
	
	/**
	 * Autor: Francesc Ros�s Albiol
	 * 
	 * Returns <code>times</code> concatenated copies of <code>c</code>.<br>
	 * The n must be a positive whole number or zero.
	 * <p>
	 * Here are some examples:
	 * <p>
	 * <code>
	 * copies('a',3)    -   "aaa"
	 * copies('a',0)    -   ""
	 * </code> Creation date:
	 * (18/8/99 10:28:56)
	 * <p>
	 * Modified: (11/6/99 09:01:23): <code>if (times == 0) return null</code>
	 * becomes <code>[...] return ""</code> like in Rexx.
	 * 
	 * @return java.lang.String
	 * @param c
	 *          char
	 * @param times
	 *          int
	 */
	public static String copies(char c, int times) {
		if (times == 0)
			return "";
		StringBuffer sb = new StringBuffer();
		for (int i = 1; i <= times; i++)
			sb.append(c);
		return sb.toString();
	}

	/**
	 * Autor: Francesc Ros�s Albiol
	 * 
	 * Returns <code>times</code> concatenated copies of <code>c</code>.<br>
	 * The n must be a positive whole number or zero.
	 * <p>
	 * Here are some examples:
	 * <p>
	 * <code>
	 * copies("123",3)    -   "123123123"
	 * copies("123",0)    -   ""
	 * </code> Creation
	 * date: (18/8/99 10:30:40)
	 * <p>
	 * Modified: (11/6/99 09:02:15): Added <code>if (times == 0) return ""</code>
	 * to do the same as in <code>copies(char, int)</code>.
	 * 
	 * @return java.lang.String
	 * @param s
	 *          java.lang.String
	 * @param times
	 *          int
	 */
	public static String copies(String s, int times) {
		if (times == 0)
			return "";
		StringBuffer sb = new StringBuffer();
		for (int i = 1; i <= times; i++)
			sb.append(s);
		return sb.toString();
	}

	/**
	 * Returns a count of the occurrences of needle in haystack that do not
	 * overlap.
	 * <P>
	 * Here are several examples:
	 * <p>
	 * <code>
	 * countStr("1", "101101"))   -   4
	 * countStr("KK", "J0KKK0")   -   1
	 * </code>
	 * Creation date: (19/8/99 11:57:31)
	 * 
	 * @return int
	 * @param needle
	 *          java.lang.String
	 * @param haystack
	 *          java.lang.String
	 */
	public static int countStr(String needle, String haystack) {
		int result = 0;
		int tempx = haystack.indexOf(needle);
		while (tempx != -1) {
			result++;
			tempx = haystack.indexOf(needle, tempx + needle.length());
		}
		return result;
	}
	
	/**
	 * Checks if the string complies the type specification.
	 * 
	 * Types can be:
	 * 'W' => Integer.
	 * @param value => the string we are checking the type.
	 * @param type	=> the type we are checking the string complies.
	 * @return true if the string complies the type
	 *         false otherwise
	 */
	public static boolean dataType(String value, char type){
	    if (type=='W') {
	        try{
	            @SuppressWarnings("unused")
				Integer enter = new Integer(value);
	            return true;
	        }
	        catch (NumberFormatException e){
	            return false;
	        }
	    }
	    return false;
	}
	/**
	 * Returns string after deleting the substring that begins at the nth
	 * character to the end (including the nth character). 
	 * The n must be a positive whole number. 
	 * If n is greater than the length of string, the function
	 * returns string unchanged.
	 * Creation date: (19/8/99 14:01:04)
	 * 
	 * @return java.lang.String
	 * @param s
	 *          java.lang.String
	 * @param start
	 *          int
	 */
	public static String delStr(String s, int start) {
		return delStr(s, start, s.length() + 1);
	}

	/**
	 * Returns string after deleting the substring that begins at the nth
	 * character and is of length characters. If you omit length, or if length is
	 * greater than the number of characters from n to the end of string, the
	 * function deletes the rest of string (including the nth character). The
	 * length must be a positive whole number or zero. The n must be a positive
	 * whole number. If n is greater than the length of string, the function
	 * returns string unchanged.
	 * <p>
	 * Here are some examples:
	 * <p>
	 * <code>
	 * delStr("abcd",2)       -     "ab"
	 * delStr("abcde",2,2)    -     "abe"
	 * delStr("abcde",5)      -     "abcde"
	 * </code>
	 * Creation date: (19/8/99 13:36:13)
	 * 
	 * @return java.lang.String
	 * @param s
	 *          java.lang.String
	 * @param start
	 *          int
	 * @param len
	 *          int
	 */
	public static String delStr(String s, int start, int len) {
		if (start < 0)
			throw new IllegalArgumentException(
					"The start position is a negative number.");
		if (len < 0)
			throw new IllegalArgumentException(
					"The length specified is a negative number.");
		/* This throws an Exception, and documentation says that if
		 * start is greater than the length of string, we should return
		 * the string unchanged, so I changed to
		if (start > s.length())
			throw new IllegalArgumentException(
					"The start position is greater than the string length.");
		*/
		if (start > s.length()) return s;
		
		StringBuffer sb = new StringBuffer();
		String left = s.substring(0, start);
		int x = start + len;
		if (x >= s.length())
			return left;
		String right = s.substring(start + len);
		sb.append(left);
		sb.append(right);
		return sb.toString();
	}

	/**
	 * Returns string after deleting the substring that starts at the startth word.
	 * The start must be a positive whole number. 
	 * If start is greater than the number of words in string, the function returns 
	 * string unchanged. The string deleted includes any blanks following the final 
	 * word involved but none of the blanks preceding the first word involved.
	 * 
	 * Creation date: (20/8/99 13:50:04)
	 * 
	 * @return java.lang.String
	 * @param s
	 *          java.lang.String
	 * @param start
	 *          int
	 */
	public static String delWord(String s, int start) {
		return delWord(s, start, s.length());
	}

	/**
	 * Returns string after deleting the substring that starts at the startth word
	 * and is of len blank-delimited words. If you omit len, or if length is
	 * greater than the number of words from start to the end of string, the
	 * function deletes the remaining words in string (including the startth
	 * word). The length must be a positive whole number or zero. The start must
	 * be a positive whole number. If start is greater than the number of words in
	 * string, the function returns string unchanged. The string deleted includes
	 * any blanks following the final word involved but none of the blanks
	 * preceding the first word involved.
	 * <p>
	 * Here are some examples:
	 * <p>
	 * <code>
	 * delWord("Now is the  time",2,2)  -   'Now time'
	 * delWord("Now is the time ",3)    -   'Now is '
	 * delWord("Now is the  time",5)    -   'Now is the  time'
	 * delWord("Now is   the time",3,1) -   'Now is   time'
	 * </code>
	 * Creation date: (20/8/99 13:28:50)
	 * 
	 * @return java.lang.String
	 * @param s
	 *          java.lang.String
	 * @param start
	 *          int
	 * @param len
	 *          int
	 */
	public static String delWord(String s, int start, int len) {
		if (s == null)
			return null;
		if (start < 0)
			throw new IllegalArgumentException(
					"The start position is a negative number.");
		if (len < 0)
			throw new IllegalArgumentException(
					"The length specified is a negative number.");
		boolean inWord = false;
		int wordNum = 0;
		StringBuffer sb = new StringBuffer();
		for (int i = 0; i < s.length(); i++) {
			char c = s.charAt(i);
			if (inWord) {
				if (c == ' ') {
					inWord = false;
				}
			} else {
				if (c != ' ') {
					if (!inWord) {
						inWord = true;
						wordNum++;
					}
				}
			}
			if (wordNum < start || wordNum >= start + len)
				sb.append(c);
		} // for i
		return sb.toString();
	}

	/**
	 * Inserts the string new into the string target. 
	 * <p>
	 * Here are some examples:
	 * <p>
	 * <code>
	 * insert(" ","abcdef",3)         -     'abc def'
	 * insert("123","abc",5,6)        -     'abc  123   '
	 * insert("123","abc",5,6,'+')    -     'abc++123+++'
	 * insert("123","abc")            -     '123abc'
	 * insert("123","abc",,5,'-')     -     '123--abc'
	 * </code>
	 * Creation date: (20/8/99 14:09:02)
	 * 
	 * @return java.lang.String
	 * @param snew
	 *          java.lang.String
	 * @param target
	 *          java.lang.String
	 * @see StringTools#insert(String, String, int, int, char)
	 */
	public static String insert(String snew, String target) {
		return insert(snew, target, 0, snew.length(), ' ');
	}

	/**
	 * Inserts the string new into the string target after the nth character. 
	 * The default value for n is 0, which means  insert before the beginning of
	 * the string. n must positive whole number or zero. 
	 * If n is greater than the length of the target string, padding is added 
	 * before the string new also. 
	 * The default pad character is a blank.
	 * <p>
	 * Here are some examples:
	 * <p>
	 * <code>
	 * insert(" ","abcdef",3)         -     'abc def'
	 * insert("123","abc",5,6)        -     'abc  123   '
	 * insert("123","abc",5,6,'+')    -     'abc++123+++'
	 * insert("123","abc")            -     '123abc'
	 * insert("123","abc",,5,'-')     -     '123--abc'
	 * </code>
	 * Creation date: (20/8/99 14:11:26)
	 * 
	 * @return java.lang.String
	 * @param snew
	 *          java.lang.String
	 * @param target
	 *          java.lang.String
	 * @param n
	 *          int
	 * @see StringTools#insert(String, String, int, int, char)
	 */
	public static String insert(String snew, String target, int n) {
		return insert(snew, target, n, snew.length(), ' ');
	}

	/**
	 * Inserts the string new, padded or truncated to length len, into the string
	 * target after the nth character. The default value for n is 0, which means
	 * insert before the beginning of the string. If specified, n and length must
	 * be positive whole numbers or zero. If n is greater than the length of the
	 * target string, padding is added before the string new also. The default
	 * value for len is the length of new. If len is less than the length of the
	 * string new, then Insert truncates new to length len. The pad
	 * character is a blank.
	 * <p>
	 * Here are some examples:
	 * <p>
	 * <code>
	 * insert(" ","abcdef",3)         -     'abc def'
	 * insert("123","abc",5,6)        -     'abc  123   '
	 * insert("123","abc",5,6,'+')    -     'abc++123+++'
	 * insert("123","abc")            -     '123abc'
	 * insert("123","abc",,5,'-')     -     '123--abc'
	 * </code>
	 * Creation date: (20/8/99 14:13:10)
	 * 
	 * @return java.lang.String
	 * @param snew
	 *          java.lang.String
	 * @param target
	 *          java.lang.String
	 * @param n
	 *          int
	 * @param len
	 *          int
	 * @see StringTools#insert(String, String, int, int, char)
	 */
	public static String insert(String snew, String target, int n, int len) {
		return insert(snew, target, n, len, ' ');
	}

	/**
	 * Inserts the string new, padded or truncated to length len, into the string
	 * target after the nth character. The default value for n is 0, which means
	 * insert before the beginning of the string. If specified, n and length must
	 * be positive whole numbers or zero. If n is greater than the length of the
	 * target string, padding is added before the string new also. The default
	 * value for len is the length of new. If len is less than the length of the
	 * string new, then Insert truncates new to length len. The default pad
	 * character is a blank.
	 * <p>
	 * Here are some examples:
	 * <p>
	 * <code>
	 * insert(" ","abcdef",3)         -     'abc def'
	 * insert("123","abc",5,6)        -     'abc  123   '
	 * insert("123","abc",5,6,'+')    -     'abc++123+++'
	 * insert("123","abc")            -     '123abc'
	 * insert("123","abc",,5,'-')     -     '123--abc'
	 * </code>
	 * Creation date: (20/8/99 14:02:11)
	 * 
	 * @return java.lang.String
	 * @param snew
	 *          java.lang.String
	 * @param target
	 *          java.lang.String
	 * @param n
	 *          int
	 * @param len
	 *          int
	 * @param pad
	 *          char
	 */
	public static String insert(String snew, String target, int n, int len, char pad) {
		if (n < 0)
			throw new IllegalArgumentException(
					"n must be a positive whole number or zero");
		if (len < 0)
			throw new IllegalArgumentException(
					"len must be a positive whole number or zero");
		snew = left(snew, len, pad);
		if (target.length() < n)
			target = left(target, n, pad);
		StringBuffer sb = new StringBuffer(target);
		sb.insert(n, snew);
		return sb.toString();
	}

	/**
	 * Calls the <code>lastIndex</code> method of <code>String</code> class.
	 * <p>
	 * Creation date: (23/8/99 13:51:14)
	 * 
	 * @return int
	 * @param needle
	 *          java.lang.String
	 * @param haystack
	 *          java.lang.String
	 * @see StringTools#lastPos(char, String, int)
	 */
	public static int lastPos(char needle, String haystack) {
		return haystack.lastIndexOf(needle);
	}

	/**
	 * Calls the <code>lastIndex</code> method of <code>String</code> class.
	 * <p>
	 * I include this method for Rexx compatibility.
	 * <p>
	 * Here are some examples:
	 * <p>
	 * <code>
	 * lastPos(' ',"abc def ghi")      -     7
	 * lastPos(' ',"abcdefghi")        -     -1
	 * lastPos('xy',"efgxyz")          -     3
	 * lastPos(' ',"abc def ghi",6)    -     3
	 * </code>
	 * <p>
	 * Creation date: (23/8/99 13:51:14)
	 * 
	 * @return int
	 * @param needle
	 *          char
	 * @param haystack
	 *          java.lang.String
	 * @param start
	 *          int
	 */
	public static int lastPos(char needle, String haystack, int start) {
		return haystack.lastIndexOf(needle, start);
	}

	/**
	 * Calls the <code>lastIndex</code> method of <code>String</code> class.
	 * <p>
	 * I include this method for Rexx compatibility.
	 * <p>
	 * Here are some examples:
	 * <p>
	 * <code>
	 * lastPos(" ","abc def ghi")      -     7
	 * lastPos(" ","abcdefghi")        -     -1
	 * lastPos("xy","efgxyz")          -     3
	 * lastPos(" ","abc def ghi",6)    -     3
	 * </code>
	 * <p>
	 * Creation date: (23/8/99 13:51:14)
	 * 
	 * @return int
	 * @param needle
	 *          char
	 * @param haystack
	 *          java.lang.String
	 * @see StringTools#lastPos(String, String, int)
	 */
	public static int lastPos(String needle, String haystack) {
		return haystack.lastIndexOf(needle);
	}

	/**
	 * Calls the <code>lastIndex</code> method of <code>String</code> class.
	 * <p>
	 * I include this method for Rexx compatibility.
	 * <p>
	 * Here are some examples:
	 * <p>
	 * <code>
	 * lastPos(" ","abc def ghi")      -     7
	 * lastPos(" ","abcdefghi")        -     -1
	 * lastPos("xy","efgxyz")          -     3
	 * lastPos(" ","abc def ghi",6)    -     3
	 * </code>
	 * <p>
	 * Creation date: (23/8/99 13:51:14)
	 * 
	 * @return int
	 * @param needle
	 *          java.lang.String
	 * @param haystack
	 *          java.lang.String
	 * @param start
	 *          int
	 */
	public static int lastPos(String needle, String haystack, int start) {
		return haystack.lastIndexOf(needle, start);
	}

	/**
	 * 
	 * Autor: Francesc Ros�s Albiol
	 * 
	 * Returns the string s left aligned, len characters length and padded with
	 * blanks.
	 * <p>
	 * Creation date: (18/8/99 10:20:49)
	 * 
	 * @return java.lang.String
	 * @param s
	 *          java.lang.String
	 * @param len
	 *          int
	 * @see StringTools#left(String, int, char)
	 */
	public static String left(String s, int len) {
		return left(s, len, ' ');
	}

	/**
	 * Autor: Francesc Ros�s Albiol
	 * 
	 * Here are some examples:
	 * <p>
	 * <code>
	 * left("1234", 2)       - "12"
	 * left("1234", 6)       - "1234  "
	 * left("1234", 6, '+')  - "1234++"
	 * </code>
	 * Creation date: (18/8/99 10:19:58)
	 * 
	 * @return java.lang.String
	 * @param s
	 *          java.lang.String
	 * @param len
	 *          int
	 * @param pad
	 *          char
	 */
	public static String left(String s, int len, char pad) {
		if (len == 0)
			return s;
		if (s == null)
			return s;
		if (s.length() == 0)
			return s;
		int sLen = s.length();
		if (len < sLen)
			return s.substring(0, len);
		else {
			StringBuffer sb = new StringBuffer(s);
			for (int i = 0; i < len - sLen; i++) {
				sb.append(pad);
			} // for i
			return sb.toString();
		}
	}

	/**
	 * Returns the string target, overlaid with the string snew from 
	 * the beginning.
	 * (The overlay may extend beyond the end of the original target string.) 
	 * 
	 * Creation date: (24/8/99 13:21:11)
	 * 
	 * @return java.lang.String
	 * @param snew
	 *          java.lang.String
	 * @param target
	 *          java.lang.String
	 * @see StringTools#overlay(String, String, int, int, char)
	 */
	public static String overlay(String snew, String target) {
		return overlay(snew, target, 0, snew.length(), ' ');
	}

	/**
	 * Returns the string target, which, starting at the nth character, is
	 * overlaid with the string snew, padded or truncated to len length. (The
	 * overlay may extend beyond the end of the original target string.) 
	 * If n is greater than the length of the target string, blank padding is 
	 * added before the new string. 
	 * n must be a positive whole number.
	 * 
	 * Creation date: (24/8/99 13:23:35)
	 * 
	 * @return java.lang.String
	 * @param snew
	 *          java.lang.String
	 * @param target
	 *          java.lang.String
	 * @param n
	 *          int
	 * @see StringTools#overlay(String, String, int, int, char)
	 */
	public static String overlay(String snew, String target, int n) {
		return overlay(snew, target, n, snew.length(), ' ');
	}

	/**
	 * Returns the string target, which, starting at the nth character, is
	 * overlaid with the string snew, padded or truncated to len length. (The
	 * overlay may extend beyond the end of the original target string.) 
	 * len must be a positive whole number or zero. 
	 * n must be a positive whole number.
	 * If n is greater than the length of the target string, blank padding 
	 * is added before the new string.
	 *  
	 * Creation date: (24/8/99 13:24:33)
	 * 
	 * @return java.lang.String
	 * @param snew
	 *          java.lang.String
	 * @param target
	 *          java.lang.String
	 * @param n
	 *          int
	 * @param len
	 *          int
	 */
	public static String overlay(String snew, String target, int n, int len) {
		return overlay(snew, target, n, len, ' ');
	}

	/**
	 * Returns the string target, which, starting at the nth character, is
	 * overlaid with the string snew, padded or truncated to len length. (The
	 * overlay may extend beyond the end of the original target string.) If you
	 * specify len, it must be a positive whole number or zero. The default value
	 * for len is the length of snew. If n is greater than the length of the
	 * target string, padding is added before the new string. The default pad
	 * character is a blank, and the default value for n is 0. If you specify n,
	 * it must be a positive whole number.
	 * <p>
	 * Here are some examples:
	 * <p>
	 * <code>
	 * overlay(" ","abcdef",2);        //    'ab def'
	 * overlay(".","abcdef",2,2);      //    'ab. ef'
	 * overlay("qq","abcd");           //    'qqcd'
	 * overlay("qq","abcd",3);         //    'abcqq'
	 * overlay("123","abc",4,6,'+');   //    'abc+123+++'
	 * </code>
	 * <p>
	 * Creation date: (23/8/99 14:11:57)
	 * 
	 * @return java.lang.String
	 * @param snew
	 *          java.lang.String
	 * @param target
	 *          java.lang.String
	 * @param n
	 *          int
	 * @param len
	 *          int
	 * @param pad
	 *          char
	 */
	public static String overlay(String snew, String target, int n, int len,
			char pad) {
		if (n < 0)
			throw new IllegalArgumentException("n must be a positive whole number");
		if (len < 0)
			throw new IllegalArgumentException("len must be a positive whole number");
		if (target == null)
			throw new IllegalArgumentException("target can not be null");
		if (snew == null)
			throw new IllegalArgumentException("snew can not be null");
		int newLen = n + len;
		snew = left(snew, len, pad);
		if (target.length() < newLen)
			target = left(target, newLen, pad);
		snew = left(snew, len, pad);
		StringBuffer sb = new StringBuffer(target);
		for (int i = 0; i < snew.length(); i++, n++) {
			sb.setCharAt(n, snew.charAt(i));
		} // for i
		return sb.toString();
	}

	/**
	 * Author: Xavier Navarro Ferrer
	 * Calls the <code>index</code> method of <code>String</code> class.
	 * <p>
	 * Creation date: (19/05/2005 16:08:14)
	 * 
	 * @return int
	 * @param needle
	 *          java.lang.String
	 * @param haystack
	 *          java.lang.String
	 */
	public static int pos(char needle, String haystack) {
	    if (haystack==null || haystack.equals("")) return -1; 
		return haystack.indexOf(needle);
	}

	/**
	 * Author: Xavier Navarro Ferrer
	 * Calls the <code>lastIndex</code> method of <code>String</code> class.
	 * <p>
	 * I include this method for Rexx compatibility.
	 * <p>
	 * Here are some examples:
	 * <p>
	 * <code>
	 * pos(' ',"abc def ghi")      -     3
	 * pos(' ',"abcdefghi")        -    -1
	 * pos('x',"efgxyz")           -     3
	 * </code>
	 * <p>
	 * Creation date: (19/05/2005 16:08:14)
	 * 
	 * @return int
	 * @param needle
	 *          char
	 * @param haystack
	 *          java.lang.String
	 * @param start
	 *          int
	 */
	public static int pos(char needle, String haystack, int start) {
	    if (haystack==null || haystack.equals("")) return -1;
		return haystack.indexOf(needle, start);
	}

	/**
	 * Author: Xavier Navarro Ferrer
	 * Calls the <code>lastIndex</code> method of <code>String</code> class.
	 * <p>
	 * I include this method for Rexx compatibility.
	 * <p>
	 * Here are some examples:
	 * <p>
	 * <code>
	 * pos(" ","abc def ghi")      -     3
	 * pos(" ","abcdefghi")        -    -1
	 * pos("xy","efgxyz")          -     3
	 * </code>
	 * <p>
	 * Creation date: (19/05/2005 16:06:14)
	 * 
	 * @return int
	 * @param needle
	 *          char
	 * @param haystack
	 *          java.lang.String
	 * @see StringTools#lastPos(String, String, int)
	 */
	public static int pos(String needle, String haystack) {
	    if (haystack==null || haystack.equals("")) return -1;
		return haystack.indexOf(needle);
	}

	/**
	 * Author: Xavier Navarro Ferrer
	 * Calls the <code>indexOf</code> method of <code>String</code> class.
	 * <p>
	 * I include this method for Rexx compatibility.
	 * <p>
	 * Here are some examples:
	 * <p>
	 * <code>
	 * pos(" ","abc def ghi")      -     3
	 * pos(" ","abcdefghi")        -    -1
	 * pos("xy","efgxyz")          -     3
	 * pos(" ","abc def ghi",5)    -     7
	 * </code>
	 * <p>
	 * Creation date: (19/05/2005 16:04:14)
	 * 
	 * @return int
	 * @param needle
	 *          java.lang.String
	 * @param haystack
	 *          java.lang.String
	 * @param start
	 *          int
	 */
	public static int pos(String needle, String haystack, int start) {
	    if (haystack==null || haystack.equals("")) return -1;
		return haystack.indexOf(needle, start);
	}

	/**
	 * Returns string, swapped end for end. Here are some examples:
	 * <p>
	 * <code>
	 * reverse("cBA")   - "ABc"
	 * reverse("XYZ ")  - " ZYX"
	 * </code>
	 * <p>
	 * Creation date: (24/8/99 11:10:00)
	 * 
	 * @return java.lang.String
	 * @param s
	 *          java.lang.String
	 */
	public static String reverse(String s) {
		StringBuffer sb = new StringBuffer(s);
		return sb.reverse().toString();
	}

	/**
	 * Returns a string of length len containing the rightmost len characters of
	 * string. The string returned is blank padded with (or truncated) on
	 * the left as needed. The length must be a positive whole number or zero.
	 * 
	 * Creation date: (18/8/99 10:26:50)
	 * 
	 * @return java.lang.String
	 * @param s
	 *          java.lang.String
	 * @param len
	 *          int
	 * @see StringTools#right(String, int, char)
	 */
	public static String right(String s, int len) {
		return right(s, len, ' ');
	}

	/**
	 * Returns a string of length len containing the rightmost len characters of
	 * string. The string returned is padded with pad characters (or truncated) on
	 * the left as needed. The default pad character is a blank. The len must be a
	 * positive whole number or zero.
	 * <p>
	 * Here are some examples:
	 * <p>
	 * <code>
	 * right("abc  d",8)     // '  abc  d'
	 * right("abc def",5)    // 'c def'
	 * right("12",5,'0')     // '00012'
	 * </code>
	 * <p>
	 * Creation date: (18/8/99 10:26:21)
	 * <p>
	 * Modified: (11/6/99 08:56:43): Added
	 * <code>if (len = s.length()) return s</code>.
	 * 
	 * @return java.lang.String
	 * @param s
	 *          java.lang.String
	 * @param len
	 *          int
	 * @param pad
	 *          char
	 */
	public static String right(String s, int len, char pad) {
		if (len < 0)
			throw new IllegalArgumentException("len must be a positive whole number");
		if (len == 0)
			return s;
		if (s == null)
			return s;
		if (s.length() == 0)
			return s;
		int sLen = s.length();
		if (len == s.length())
			return s;
		if (len < sLen)
			return s.substring(sLen - len);
		else {
			String sl = copies(pad, len - sLen);
			return sl + s;
		}
	}

	/**
	 * Returns the blank-delimited words in the receiving string, with one blank
	 * character between each word. 
	 * Creation date: (24/8/99 14:24:29)
	 * 
	 * @return java.lang.String
	 * @param s
	 *          java.lang.String
	 * @see StringTools#space(String, int, char)
	 */
	public static String space(String s) {
		return space(s, 1, ' ');
	}

	/**
	 * Returns the blank-delimited words in the receiving string, with one
	 * pad character between each word. 
	 * Creation date: (24/8/99 14:23:55)
	 * 
	 * @return java.lang.String
	 * @param s
	 *          java.lang.String
	 * @param pad
	 *          char
	 * @see StringTools#space(String, int, char)
	 */
	public static String space(String s, char pad) {
		return space(s, 1, pad);
	}

	/**
	 * Returns the blank-delimited words in the receiving string, with
	 * n blank characters between each word. 
	 * n must be a positive whole number or zero. If it is 0, all blanks are
	 * removed. Leading and trailing blanks are always removed. 
	 * Creation date: (24/8/99 14:19:31)
	 * 
	 * @return java.lang.String
	 * @param s
	 *          java.lang.String
	 * @param n
	 *          int
	 * @see StringTools#space(String, int, char)
	 */
	public static String space(String s, int n) {
		return space(s, n, ' ');
	}

	/**
	 * Returns the blank-delimited words in the receiving string, with
	 * <code>n</code> <code>pad</code> characters between each word. If you
	 * specify n, it must be a positive whole number or zero. If it is 0, all
	 * blanks are removed. Leading and trailing blanks are always removed. The
	 * default for n is 1, and the default pad character is a blank. Creation
	 * date: (24/8/99 13:55:44)
	 * 
	 * @return java.lang.String
	 * @param s
	 *          java.lang.String
	 * @param n
	 *          int
	 * @param pad
	 *          char
	 */
	public static String space(String s, int n, char pad) {
		if (s == null)
			return s;
		if (s.length() == 0)
			return s;
		if (n < 0)
			throw new IllegalArgumentException(
					"n is a negative number and must be positive");
		s = strip(s);
		StringBuffer sb = new StringBuffer();
		for (int i = 1; i <= n; i++) {
			sb.append(pad);
		} // for i
		String p = sb.toString();
		StringTokenizer st = new StringTokenizer(s, " ");
		sb = new StringBuffer();
		while (st.hasMoreElements()) {
			String word = st.nextToken();
			if (sb.length() != 0)
				if (n != 0)
					sb.append(p);
			sb.append(word);
		}
		return sb.toString();
	}

	/**
	 * Opci� per defecte de strip. Veure documentaci� de strip(string cadena, string deOn, char charEsborrat)
	 * Default option of strip. See strip(string cadena, string deOn, char charEsborrat) documentation
	 * @param cadena
	 * @return  La cadena sense blancs davant i darrera.
	 * 			The String cadena without leading and traling blanc chars.
	 */
	public static String strip(String cadena){
		return strip(cadena,"B");	
	}

	/**
	 * Opci� per defecte de strip quan no s'especifica d'on es treuen els car�cters. 
	 * Veure documentaci� de strip(string cadena, string deOn, char charEsborrat).
	 * Default option of strip when you don't specify from where yo are removing the chars. 
	 * See strip(string cadena, string deOn, char charEsborrat) documentation.
	 * @param cadena
	 * @return  La cadena sense <param>charEsborrat<param> davant i darrera.
	 * 			The String cadena without leading and traling <param>charEsborrat<param> chars.
	 */
	public static String strip(String cadena, char charEsborrat){
		return strip(cadena,"B",charEsborrat);
	}
	
	/**
	 * Opci� per defecte de strip quan no s'especifica el car�cter que s'elimina. 
	 * Veure documentaci� de strip(string cadena, string deOn, char charEsborrat).
	 * Default option of strip when you don't specify the char you are removing. 
	 * See strip(string cadena, string deOn, char charEsborrat) documentation.
	 * @param cadena
	 * @return  La cadena sense <param>charEsborrat<param>.
	 * 			The String cadena without <param>charEsborrat<param> chars.
	 */
	public static String strip(String cadena, String deOn){
		return strip(cadena,deOn, ' ');
	}
	
	/**
	 * Port de la funci� strip de REXX.
	 * Aquesta funci� retorna una cadena a la qual se li han eliminat els caracters
	 * <param>charEsborrat<param> que tingui a:
	 * Si <param>deOn<param> = 'I' o 'L' del principi.
	 * Si <param>deOn<param> = 'F' o 'T' del final.
	 * Si <param>deOn<param> = 'D' o 'B' de l'inici i del final.
	 * <param>deOn<param> es pot especificar en min�scules i amb maj�scules.
	 * 
	 * Port of strip function in REXX.
	 * This method returns a String without the <param>charEsborrat<param> at
	 * If <param>deOn<param> = 'I' o 'L' from the beggining.
	 * Si <param>deOn<param> = 'F' o 'T' from the end.
	 * Si <param>deOn<param> = 'D' o 'B' from both.
	 * You can <param>deOn<param> in UpperCase and LowerCase.
	 * 
	 * @param cadena
	 * @param deOn
	 * @param charEsborrat
	 * @return
	 */
	public static String strip(String cadena, String deOn, char charEsborrat){
		if ((cadena == null)) return cadena;
		deOn = deOn.trim().toUpperCase();
		if (deOn != "B" && deOn != "L" && deOn != "T" && deOn != "I" && deOn != "F")
			throw new IllegalArgumentException("Invalid mode '" + deOn + "'");
		if ((deOn == "B") || (deOn == "D") || (deOn == "I") || (deOn == "L")) {
			while (cadena.length()!=0 && cadena.charAt(0)==charEsborrat) {
				cadena = cadena.substring(1);
			}
		}
		if ((deOn == "B") || (deOn == "D") || (deOn == "F") || (deOn == "T")) {  
			while (cadena.length()!=0 && cadena.charAt(cadena.length()-1)==charEsborrat) {
				cadena = cadena.substring(0,cadena.length()-1);
			}
		}
		return cadena;
	}

	/**
	 * Returns the substring of the receiving string that begins at the nth
	 * character.  
	 * The n must be a positive whole number. 
	 * Creation date: (19/8/99 13:10:02)
	 * 
	 * @return java.lang.String
	 * @param s
	 *          java.lang.String
	 * @param start
	 *          int
	 * @see StringTools#subStr(String, int, int, char)
	 */
	public static String subStr(String s, int start) {
		return s.substring(start);
		//return subStr(s, start, s.length());
	}

	/**
	 * Returns the sub-string of String s that begins at the start character, 
	 * and is of length len [padded with blank characters if necessary]. 
	 * start must be a positive whole number and len must be a 
	 * non-negative whole number. 
	 * If start is greater than s.length(), then 
	 * only pad characters can be returned.
	 * 
	 * @return java.lang.String
	 * @param s
	 *          java.lang.String
	 * @param start
	 *          int
	 * @param len
	 *          int
	 * @see StringTools#subStr(String, int, int, char)
	 */
	public static String subStr(String s, int start, int len) {
		if (s == null)
			return null;
		return subStr(s, start, len, ' ');
	}

	/**
	 * Returns the sub-string of String <code>s</code> that begins at the
	 * <code>start</code> character, and is of length <code>len</code> [padded
	 * with <code>pad</code> characters if necessary]. <code>start</code> must
	 * be a positive whole number and <code>len</code> must be a non-negative
	 * whole number. If <code>start</code> is greater than
	 * <code>s.length()</code>, then only pad characters can be returned.
	 * <p>
	 * If <code>len</code> is omitted it defaults to be the rest of the string.
	 * The default pad character is a blank.
	 * <p>
	 * Here are some examples:
	 * <p>
	 * 
	 * <pre>
	 *       subStr("abc", 1)          // "bc"
	 *       subStr("abc", 1, 4)       // "bc  "
	 *       subStr("abc", 4, 4)       // "    "
	 *       subStr("abc", 1, 6, '.')  // "bc...."
	 *       subStr("abc", 4, 6, '.')  // "......"
	 * </pre>
	 * 
	 * Creation date: (31/3/00 18:58:28)
	 * 
	 * @return java.lang.String
	 * @param s
	 *          java.lang.String
	 * @param start
	 *          int
	 * @param len
	 *          int
	 * @param pad
	 *          char
	 */
	public static String subStr(String s, int start, int len, char pad) {
		if (s == null)
			return null;
		if (start < 0)
			throw new IllegalArgumentException(
					"The start position is a negative number.");
		if (len < 0)
			throw new IllegalArgumentException(
					"The length specified is a negative number.");
		if (start > s.length())
			return copies(pad, len);
		int x = start + len;
		if (x > s.length())
			return s.substring(start) + copies(pad, x - s.length());
		return s.substring(start, x);
	}

	/**
	 * Returns the substring of string that starts at the nth word, and is up to
	 * remaining blank-delimited words. 
	 * The firstWordNum must be a positive whole  number greater than 1.
	 * The returned string never has leading or trailing blanks, but includes 
	 * all blanks between the selected words.
	 * 
	 * Creation date: (25/8/99 10:19:11)
	 * 
	 * @return java.lang.String
	 * @param s
	 *          java.lang.String
	 * @param firstWordNum
	 *          int
	 * @see StringTools#subWord(String, int, int)
	 */
	public static String subWord(String s, int firstWordNum) {
		int nWords = words(s);
		return subWord(s, firstWordNum, (nWords - firstWordNum) + 1);
	}

	/**
	 * Returns the substring of string that starts at the nth word, and is up to
	 * length blank-delimited words. The firstWordNum must be a positive whole
	 * number greater than 1. If you omit numWords, it defaults to the number of
	 * remaining words in string. The returned string never has leading or
	 * trailing blanks, but includes all blanks between the selected words.
	 * <p>
	 * Here are some examples:
	 * <p>
	 * <code>
	 * subWord("Now is the  time",2,2);    // "is the"
	 * subWord("Now is the  time",3);      // "the  time"
	 * subWord("Now is the  time",5);      // ""
	 * </code>
	 * Creation date: (19/8/99 9:07:30)
	 * 
	 * @return java.lang.String
	 * @param s
	 *          java.lang.String
	 * @param firstWordNum
	 *          int
	 * @param numWords
	 *          int Lenght in words.
	 */
	public static String subWord(String s, int firstWordNum, int numWords) {
		StringBuffer subWord = new StringBuffer();
		boolean inWord = false;
		int currWordNum = 0;
		if (firstWordNum < 0)
			throw new IllegalArgumentException(
					"firstWordNum must be a positive whole number.");
		if (firstWordNum < 1)
			throw new IllegalArgumentException("firstWordNum must be greater than 1.");
		for (int i = 0; i < s.length(); i++) {
			char c = s.charAt(i);
			if (c != ' ') {
				if (!inWord) {
					currWordNum++;
					inWord = true;
				}
			} else {
				inWord = false;
			}
			if (currWordNum >= firstWordNum
					&& currWordNum <= (firstWordNum + numWords) - 1)
				subWord.append(c);
		} // for i
		if (currWordNum < (firstWordNum + numWords) - 1)
			return null;
		else
			return strip(subWord.toString());
	}


	/**
	 * Returns s with each character translated to another character or unchanged.
	 * You can also use this function to reorder the characters in string.
	 * <p>
	 * The output table is tableo and the input translation table is tablei.
	 * TRANSLATE searches tablei for each character in s. If the character is
	 * found, then the corresponding character in tableo is used in the result
	 * string; if there are duplicates in tablei, the first (leftmost) occurrence
	 * is used. If the character is not found, the original character in s is
	 * used. The result string is always the same length as string s.
	 * <p>
	 * The tables can be of any length. If you specify neither translation table
	 * and omit pad, string is simply translated to uppercase (that is, lowercase
	 * a-z to uppercase A-Z), but, if you include pad, the language processor
	 * translates the entire string to pad characters. tablei defaults (when null)
	 * to xrange(0x00, 0xFF), and tableo defaults to the null string and is padded
	 * with pad or truncated as necessary. The default pad is a blank.
	 * <p>
	 * Here are some examples:
	 * <p>
	 * <code>
	 * translate("abcdef");                       // 'ABCDEF'
	 * translate("abcdef", '+');                  // '++++++'
	 * translate("abcdef","12","ec");             // 'ab2d1f'
	 * translate("abcdef","12","abcd",'.');       // '12..ef'
	 * translate("APQRV",null,"PR");              // 'A Q V'
	 * translate("4123","abcd","1234");           // 'dabc'
	 * translate("APQRV",xrange((char)0x0,'Q'));  // 'APQ  '
	 * </code>
	 * <p>
	 * <note>The last example shows how to use the TRANSLATE function to reorder
	 * the characters in a string. In the example, the last character of any
	 * four-character string specified as the second argument would be moved to
	 * the beginning of the string. </note> Creation date: (25/8/99 10:48:11)
	 * 
	 * @return java.lang.String
	 * @param s
	 *          java.lang.String
	 * @see StringTools#translate(String, String, String, char)
	 */
	public static String translate(String s) {
		return s.toUpperCase();
	}

	/**
	 * Returns s with each character translated to another character or unchanged.
	 * You can also use this function to reorder the characters in string.
	 * <p>
	 * The output table is tableo and the input translation table is tablei.
	 * TRANSLATE searches tablei for each character in s. If the character is
	 * found, then the corresponding character in tableo is used in the result
	 * string; if there are duplicates in tablei, the first (leftmost) occurrence
	 * is used. If the character is not found, the original character in s is
	 * used. The result string is always the same length as string s.
	 * <p>
	 * The tables can be of any length. If you specify neither translation table
	 * and omit pad, string is simply translated to uppercase (that is, lowercase
	 * a-z to uppercase A-Z), but, if you include pad, the language processor
	 * translates the entire string to pad characters. tablei defaults (when null)
	 * to xrange(0x00, 0xFF), and tableo defaults to the null string and is padded
	 * with pad or truncated as necessary. The default pad is a blank.
	 * <p>
	 * Here are some examples:
	 * <p>
	 * <code>
	 * translate("abcdef");                       // 'ABCDEF'
	 * translate("abcdef", '+');                  // '++++++'
	 * translate("abcdef","12","ec");             // 'ab2d1f'
	 * translate("abcdef","12","abcd",'.');       // '12..ef'
	 * translate("APQRV",null,"PR");              // 'A Q V'
	 * translate("4123","abcd","1234");           // 'dabc'
	 * translate("APQRV",xrange((char)0x0,'Q'));  // 'APQ  '
	 * </code>
	 * <p>
	 * <note>The last example shows how to use the TRANSLATE function to reorder
	 * the characters in a string. In the example, the last character of any
	 * four-character string specified as the second argument would be moved to
	 * the beginning of the string. </note> Creation date: (25/8/99 10:51:27)
	 * 
	 * @return java.lang.String
	 * @param s
	 *          java.lang.String
	 * @param pad
	 *          char
	 * @see StringTools#translate(String, String, String, char)
	 */
	public static String translate(String s, char pad) {
		if (s == null)
			return s;
		StringBuffer sb = new StringBuffer(s);
		for (int i = 0; i < sb.length(); i++) {
			sb.setCharAt(i, pad);
		}
		return sb.toString();
	}

	/**
	 * Returns s with each character translated to another character or unchanged.
	 * You can also use this function to reorder the characters in string.
	 * <p>
	 * The output table is tableo and the input translation table is tablei.
	 * TRANSLATE searches tablei for each character in s. If the character is
	 * found, then the corresponding character in tableo is used in the result
	 * string; if there are duplicates in tablei, the first (leftmost) occurrence
	 * is used. If the character is not found, the original character in s is
	 * used. The result string is always the same length as string s.
	 * <p>
	 * The tables can be of any length. If you specify neither translation table
	 * and omit pad, string is simply translated to uppercase (that is, lowercase
	 * a-z to uppercase A-Z), but, if you include pad, the language processor
	 * translates the entire string to pad characters. tablei defaults (when null)
	 * to xrange(0x00, 0xFF), and tableo defaults to the null string and is padded
	 * with pad or truncated as necessary. The default pad is a blank.
	 * <p>
	 * Here are some examples:
	 * <p>
	 * <code>
	 * translate("abcdef");                       // 'ABCDEF'
	 * translate("abcdef", '+');                  // '++++++'
	 * translate("abcdef","12","ec");             // 'ab2d1f'
	 * translate("abcdef","12","abcd",'.');       // '12..ef'
	 * translate("APQRV",null,"PR");              // 'A Q V'
	 * translate("4123","abcd","1234");           // 'dabc'
	 * translate("APQRV",xrange((char)0x0,'Q'));  // 'APQ  '
	 * </code>
	 * <p>
	 * <note>The last example shows how to use the TRANSLATE function to reorder
	 * the characters in a string. In the example, the last character of any
	 * four-character string specified as the second argument would be moved to
	 * the beginning of the string. </note> Creation date: (25/8/99 11:04:13)
	 * 
	 * @return java.lang.String
	 * @param s
	 *          java.lang.String
	 * @param tableo
	 *          java.lang.String
	 * @see StringTools#translate(String, String, String, char)
	 */
	public static String translate(String s, String tableo) {
		StringBuffer tablei = new StringBuffer();
		for (int i = 0; i <= 255; i++) {
			tablei.append((char) i);
		}
		return translate(s, tableo, tablei.toString(), ' ');
	}

	/**
	 * Returns s with each character translated to another character or unchanged.
	 * You can also use this function to reorder the characters in string.
	 * <p>
	 * The output table is tableo and the input translation table is tablei.
	 * TRANSLATE searches tablei for each character in s. If the character is
	 * found, then the corresponding character in tableo is used in the result
	 * string; if there are duplicates in tablei, the first (leftmost) occurrence
	 * is used. If the character is not found, the original character in s is
	 * used. The result string is always the same length as string s.
	 * <p>
	 * The tables can be of any length. If you specify neither translation table
	 * and omit pad, string is simply translated to uppercase (that is, lowercase
	 * a-z to uppercase A-Z), but, if you include pad, the language processor
	 * translates the entire string to pad characters. tablei defaults (when null)
	 * to xrange(0x00, 0xFF), and tableo defaults to the null string and is padded
	 * with pad or truncated as necessary. The default pad is a blank.
	 * <p>
	 * Here are some examples:
	 * <p>
	 * <code>
	 * translate("abcdef");                       // 'ABCDEF'
	 * translate("abcdef", '+');                  // '++++++'
	 * translate("abcdef","12","ec");             // 'ab2d1f'
	 * translate("abcdef","12","abcd",'.');       // '12..ef'
	 * translate("APQRV",null,"PR");              // 'A Q V'
	 * translate("4123","abcd","1234");           // 'dabc'
	 * translate("APQRV",xrange((char)0x0,'Q'));  // 'APQ  '
	 * </code>
	 * <p>
	 * <note>The last example shows how to use the TRANSLATE function to reorder
	 * the characters in a string. In the example, the last character of any
	 * four-character string specified as the second argument would be moved to
	 * the beginning of the string. </note> Creation date: (25/8/99 10:59:13)
	 * 
	 * @return java.lang.String
	 * @param s
	 *          java.lang.String
	 * @param tableo
	 *          java.lang.String
	 * @param tablei
	 *          java.lang.String
	 * @see StringTools#translate(String, String, String, char)
	 */
	public static String translate(String s, String tableo, String tablei) {
		return translate(s, tableo, tablei, ' ');
	}

	/**
	 * Returns s with each character translated to another character or unchanged.
	 * You can also use this function to reorder the characters in string.
	 * <p>
	 * The output table is tableo and the input translation table is tablei.
	 * TRANSLATE searches tablei for each character in s. If the character is
	 * found, then the corresponding character in tableo is used in the result
	 * string; if there are duplicates in tablei, the first (leftmost) occurrence
	 * is used. If the character is not found, the original character in s is
	 * used. The result string is always the same length as string s.
	 * <p>
	 * The tables can be of any length. If you specify neither translation table
	 * and omit pad, string is simply translated to uppercase (that is, lowercase
	 * a-z to uppercase A-Z), but, if you include pad, the language processor
	 * translates the entire string to pad characters. tablei defaults (when null)
	 * to xrange(0x00, 0xFF), and tableo defaults to the null string and is padded
	 * with pad or truncated as necessary. The default pad is a blank.
	 * <p>
	 * Here are some examples:
	 * <p>
	 * <code>
	 * translate("abcdef");                       // 'ABCDEF'
	 * translate("abcdef", '+');                  // '++++++'
	 * translate("abcdef","12","ec");             // 'ab2d1f'
	 * translate("abcdef","12","abcd",'.');       // '12..ef'
	 * translate("APQRV",null,"PR");              // 'A Q V'
	 * translate("4123","abcd","1234");           // 'dabc'
	 * translate("APQRV",xrange((char)0x0,'Q'));  // 'APQ  '
	 * </code>
	 * <p>
	 * <note>The last example shows how to use the TRANSLATE function to reorder
	 * the characters in a string. In the example, the last character of any
	 * four-character string specified as the second argument would be moved to
	 * the beginning of the string. </note> Creation date: (25/8/99 10:37:56)
	 * 
	 * @return java.lang.String
	 * @param s
	 *          java.lang.String
	 * @param tableo
	 *          java.lang.String
	 * @param tablei
	 *          java.lang.String
	 * @param pad
	 *          char
	 */
	public static String translate(String s, String tableo, String tablei,
			char pad) {
		if (s == null)
			return s;
		if (s.length() == 0)
			return s;
		if (tableo == null)
			tableo = copies(pad, tablei.length());
		else
			tableo = left(tableo, tablei.length(), pad);
		StringBuffer sb = new StringBuffer(s);
		for (int i = 0; i < s.length(); i++) {
			char c = sb.charAt(i);
			int ndx = tablei.indexOf(c);
			if (ndx != -1)
				sb.setCharAt(i, tableo.charAt(ndx));
		} // for i
		return sb.toString();
	}
	
	public static String UAcc(String s){
		if ((s=="") || (s==null) || (s.length()==0)) return s; 
		return translate(s);
	}

	/**
	 * Returns the nth blank-delimited word in string or returns the null string
	 * if fewer than n words are in string. The n must be a positive whole number.
	 * This function is exactly equivalent to <code>subWord(string,n,1)</code>.
	 * <p>
	 * Here are some examples:
	 * <p>
	 * <code>
	 * word("Now is the time",3);    // 'the'
	 * word("Now is the time",5);    // ''
	 * </code>
	 * <p>
	 * Creation date: (19/8/99 9:03:25)
	 * 
	 * @return java.lang.String
	 * @param s
	 *          java.lang.String
	 * @param n
	 *          int Word number
	 */
	public static String word(String s, int n) {
		if (s == null)
			return null;
		if (s.length() == 0)
			return null;
		if (n < 1)
			throw new IllegalArgumentException("n must be a positive whole number.");
		boolean inWord = false;
		int numWords = 0;
		StringBuffer lastWord = new StringBuffer();
		for (int i = 0; i < s.length(); i++) {
			char c = s.charAt(i);
			if (c == ' ') {
				if (inWord) {
					inWord = false;
					if (numWords == n) {
						return lastWord.toString();
					}
				}
			} else {
				if (!inWord) {
					inWord = true;
					lastWord = new StringBuffer();
					numWords++;
				}
				lastWord.append(c);
			}
		} // for i
		return lastWord.toString();
	}

	/**
	 * Returns the position of the first character in the nth blank-delimited word
	 * in s or returns -1 if fewer than n words are in s or s is null or s haves
	 * length 0. The n must be a positive whole number.
	 * <p>
	 * Here are some examples:
	 * <p>
	 * <code>
	 * wordIndex("Now is the time",3);    // 7
	 * wordIndex("Now is the time",6);    // -1
	 * </code>
	 * <p>
	 * Creation date: (25/8/99 13:15:33)
	 * 
	 * @return int
	 * @param s
	 *          java.lang.String
	 * @param n
	 *          int
	 */
	public static int wordIndex(String s, int n) {
		if (s == null)
			return -1;
		if (s.length() == 0)
			return -1;
		if (n <= 0)
			throw new IllegalArgumentException("n must be a positive whole number.");
		boolean inWord = false;
		int currentWord = 0;
		for (int i = 0; i < s.length(); i++) {
			char c = s.charAt(i);
			if (c == ' ') {
				inWord = false;
				continue;
			}
			if (!inWord) {
				if (c != ' ') {
					inWord = true;
					currentWord++;
					if (currentWord == n)
						return i;
				}
			}
		} // for i
		// if (currentWord < n) return -1;
		return -1;
	}

	/**
	 * Returns the length of the nth blank-delimited word in string s or returns 0
	 * if fewer than n words are in string s. The n must be a positive whole
	 * number.
	 * <p>
	 * Here are some examples:
	 * <p>
	 * <code>
	 * wordLength("Now is the time",2);      //  2
	 * wordLength("Now comes the time",2);   //  5
	 * wordLength("Now is the time",6);      //  0
	 * </code>
	 * <p>
	 * Creation date: (25/8/99 13:28:28)
	 * 
	 * @return int
	 * @param s
	 *          java.lang.String
	 * @param n
	 *          int
	 */
	public static int wordLength(String s, int n) {
		if (n < 1)
			throw new IllegalArgumentException("n must be a positive whole number");
		String w = word(s, n);
		return w.length();
	}

	/**
	 * Returns the number of blank-delimited words in string.
	 * <p>
	 * Here are some examples:
	 * <p>
	 * <code>
	 * words("Now is the time");   //  4
	 * words(" ");                 //  0
	 * </code>
	 * <p>
	 * In NetRexx the tab character is not a word delimiter. Only the blank
	 * character is a word delimited.
	 * <p>
	 * Creation date: (18/8/99 11:10:17)
	 * 
	 * @return int
	 * @param s
	 *          java.lang.String
	 */
	public static int words(String s) {
		if (s == null)
			return 0;
		if (s.length() == 0)
			return 0;
		int numWords = 0;
		boolean inWord = false;
		for (int i = 0; i < s.length(); i++) {
			char c = s.charAt(i);
			if (c != ' ') {
				if (!inWord) {
					inWord = true;
					numWords++;
				}
			} else {
				inWord = false;
			}
		} // for i
		return numWords;
	}

	/**
	 * Returns a string of all valid 1-byte encodings (in ascending order) between
	 * and including the values start (0x00) and end (0xFF).
	 * <p>
	 * Here are some examples:
	 * <p>
	 * <code>
	 * xrange();      // [0x00 .. 0xFF]
	 * </code>
	 * <p>
	 * Creation date: (25/8/99 12:33:12)
	 * 
	 * @return java.lang.String
	 */
	public static String xrange() {
		return xrange((char) 0, (char) 255);
	}

	/**
	 * Returns a string of all valid 1-byte encodings (in ascending order) between
	 * and including the values start and 0xFF. start must be a single character.
	 * <p>
	 * Here are some examples:
	 * <p>
	 * <code>
	 * xrange('a');    // [a .. 0xFF]
	 * </code>
	 * <p>
	 * Creation date: (25/8/99 12:32:11)
	 * 
	 * @return java.lang.String
	 * @param start
	 *          char
	 */
	public static String xrange(char start) {
		return xrange(start, (char) 255);
	}

	/**
	 * Returns a string of all valid 1-byte encodings (in ascending order) between
	 * and including the values start and end. The default value for start is
	 * 0x00, and the default value for end is 0xFF. If start is greater than end,
	 * the values wrap from 0xFF to 0x00. If specified, start and end must be
	 * single characters.
	 * <p>
	 * Here are some examples:
	 * <p>
	 * <code>
	 * xrange('a','f')      -    'abcdef'
	 * xrange('i','j')      -    'ij'
	 * </code>
	 * <p>
	 * Creation date: (25/8/99 12:22:15)
	 * 
	 * @return java.lang.String
	 * @param start
	 *          char
	 * @param end
	 *          char
	 */
	public static String xrange(char start, char end) {
		if (start < 0)
			throw new IllegalArgumentException(
					"start must be a whole number between 0 and 255");
		if (end < 0)
			throw new IllegalArgumentException(
					"end must be a whole number between 0 and 255");
		StringBuffer sb = new StringBuffer();
		if (start <= end) {
			for (char i = start; i <= end; i++) {
				sb.append(i);
			} // for i
		} else { // start > end
			for (char i = start; i <= 255; i++) {
				sb.append(i);
			} // for i
			for (char i = 0; i < end; i++) {
				sb.append(i);
			} // for i
		}
		return sb.toString();
	}
}
