package net.nitus.museu;

import java.io.IOException;
import java.io.PrintWriter;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import javax.naming.NamingException;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.jdom.Attribute;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;

/**
 * Servlet que retorna les peticions AJAX dels jaciments.
 */
public class Jaciment extends HttpServlet {
	private static final long serialVersionUID = 1L;
       
    /**
     * @see HttpServlet#HttpServlet()
     */
    public Jaciment() {
        super();
        // TODO Auto-generated constructor stub
    }

	/**
	 * @see HttpServlet#doGet(HttpServletRequest request, HttpServletResponse response)
	 */
	protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		response.setContentType("text/xml;charset=utf-8");
		PrintWriter resposta = response.getWriter();

		Connection con = null;
		Document doc;
		
		try {
			ConnexioDataSource cds = new ConnexioDataSource("java:/comp/env/jdbc/postgres");
			con = cds.getConnexio();

			String metode=request.getParameter("metode");
			if (metode != null && metode.equals("idJaciment")) {
				Integer id = new Integer(request.getParameter("id"));
				doc = getJaciment(id,con);
			}
			else if (metode != null && metode.equals("eresJaciments")){
				String eres = request.getParameter("eres");
				doc = getJaciments(eres, con);
			}
			else if (metode != null && metode.equals("llistaJaciments")){
				doc = getJaciments(null, con);
			}
			else if (metode != null && metode.equals("diapositives")){
				Integer id = new Integer(request.getParameter("id"));
				String eres = request.getParameter("eres");
				doc = getDiapositives(id, eres, con);
			}
			else {
				Element root = new Element("Error").setText("Crida incorrecta al servlet.");
				Element metodes = new Element("Metodes_Permesos");
				metodes.addContent(new Element("metode").setAttribute(new Attribute("metode", "idJaciment")).addContent(new Element("parametre").setAttribute(new Attribute("id", "n"))));
				metodes.addContent(new Element("metode").setAttribute(new Attribute("metode", "eresJaciment")).addContent(new Element("parametre").setAttribute(new Attribute("eres", "n, n, ..."))));
				metodes.addContent(new Element("metode").setAttribute(new Attribute("metode", "llistaJaciments")));
				Element diapositives = new Element("metode").setAttribute(new Attribute("metode", "diapositives"));
				diapositives.addContent(new Element("parametre").setAttribute(new Attribute("id", "n")));
				diapositives.addContent(new Element("parametre").setAttribute(new Attribute("eres", "id, id, ..., n")));
				metodes.addContent(diapositives);
				root.addContent(metodes);
				doc = new Document(root);
			}
			con.close();
		}
		catch (NumberFormatException e) {
			Element root = new Element("error").setText("L'identificador de jaciment passat com a paràmetre era incorrecte.");
			doc = new Document(root);
		}catch (SQLException e) {
			Element root = new Element("error").setText("Error SQL:"+e.getMessage()+"<br>"+e.getLocalizedMessage()+"<br>"+e.getStackTrace());
			try {
				if (con!=null && !con.isClosed()) con.close();
			} catch (SQLException e1) {
				root.addContent(new Element("error tancant: ").setText("Error SQL tancant la connexio: "+e1.getErrorCode()+"\n"+e.getLocalizedMessage()));
			}
			doc = new Document(root);
		} catch (NamingException e) {
			doc = new Document(new Element("error").setText("Error DataSource:"+e.getLocalizedMessage()+"\n"+e.getExplanation()));
		} catch (Exception e) {
			doc = new Document(new Element("error").setText("Error:"+e.getMessage()));
		}
	
		XMLOutputter out=new XMLOutputter(Format.getPrettyFormat());
		out.output(doc,resposta);
		//out.output(doc,System.out);
	}

	/**
	 * @see HttpServlet#doPost(HttpServletRequest request, HttpServletResponse response)
	 */
	protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		doGet(request, response);
	}

	/**
	 * Retorna un document xml amb totes les dades del jaciment buscat.
	 * @param id L'identificador del jaciment que s'està buscant.
	 * @param con La connexió a la base de dades que s'utilitzarà per fer la consulta
	 */
	protected Document getJaciment(int id, Connection con) throws SQLException{
		Statement stmt = con.createStatement();
		String select = "Select id, nom, poblacio, xwgs84, ywgs84, imatge, xImatge, yImatge from jaciments.jaciments where id = "+id;
		ResultSet rs = stmt.executeQuery(select);
		Element root=new Element("jaciment");
		while (rs.next()){
			root=new Element("jaciment");
			root.addContent(new Element("id").setText(rs.getString(1)));
			root.addContent(new Element("nom").setText(rs.getString(2)));
			root.addContent(new Element("poblacio").setText(rs.getString(3)));
			Element imatge = new Element("imatgeJaciment");
			imatge.addContent(new Element("nomImatgeJaciment").setText(rs.getString(6)));
			imatge.addContent(new Element("xImatgeJaciment").setText(rs.getInt(7)+""));
			imatge.addContent(new Element("yImatgeJaciment").setText(rs.getInt(8)+""));
			root.addContent(imatge);
			Element posicio = new Element("posicio");
			posicio.addContent(new Element("latitud").setText(rs.getBigDecimal(5).toString()));
			posicio.addContent(new Element("longitud").setText(rs.getBigDecimal(4).toString()));
			root.addContent(posicio);
		}
		rs.close();
		// Afegim els Períodes
		Element periodes = new Element("periodes");
		select = "Select p.nom, pj.dates, pj.descripcio, pj.altres, p.id " +
				 "from jaciments.periodes_jaciments pj, jaciments.periodes p " +
				 "where p.id = pj.periode and pj.jaciment = "+id+
				 "Order by p.id";
		rs=stmt.executeQuery(select);
		while (rs.next()){
			Element periode = new Element("periode"); //Creem el periode
			periode.addContent(new Element("nomPeriode").setText(rs.getString(1)));
			periode.addContent(new Element("dates").setText(rs.getString(2)));
			periode.addContent(new Element("descripcio").setText(rs.getString(3)));
			periode.addContent(new Element("altres").setText(rs.getString(4)));
			int idPeriode = rs.getInt(5);
			
			// Afegim els objectes;
			Element objectes = new Element("objectes");
			String buscaObjectes = "Select n, descripcio " +
								   "from jaciments.objectes " +
								   "where jaciment="+id+" and periode="+idPeriode;
			Statement stmtObjectes = con.createStatement();
			ResultSet rsObjectes = stmtObjectes.executeQuery(buscaObjectes);
			while (rsObjectes.next()) {
				Element objecte = new Element("objecte"); //Creem l'objecte
				objecte.addContent(new Element("descripcio").setText(rsObjectes.getString(2)));
				int idObjecte = rsObjectes.getInt(1);
				//Afegim les imatges
				Element imatges = new Element("imatges");
				String buscaImatges = "Select url, xImatge, yImatge " +
						              "from jaciments.imatges " +
						              "where jaciment = "+id+" and periode = "+idPeriode+" and objecte="+idObjecte;
				Statement stmtImatges = con.createStatement();
				ResultSet rsImatges= stmtImatges.executeQuery(buscaImatges);
				while (rsImatges.next()) {
					Element imatge = new Element("imatge"); //Creem la imatge
					imatge.addContent(new Element("url").setText(rsImatges.getString(1)));
					imatge.addContent(new Element("xImatge").setText(rsImatges.getString(2)));
					imatge.addContent(new Element("yImatge").setText(rsImatges.getString(3)));
					imatges.addContent(imatge); //Afegim la imatge a les imatges de l'objecte
				} //Final imatges
				rsImatges.close();
				stmtImatges.close();
				objecte.addContent(imatges); //Afegim les imatges a l'objecte
				objectes.addContent(objecte); //Afegim l'objecte als objectes del període
			} // Final Objectes*/
			periode.addContent(objectes); //Afegim els objectes del període al període
			periodes.addContent(periode); //Afegim el període als períodes del jaciment
			rsObjectes.close();
			stmtObjectes.close();
		} //Final Periodes
		root.addContent(periodes); //Afegim els períodes al jaciment
			
		Document doc = new Document(root);
		stmt.close();
		return doc;
	}

	/**
	 * Retorna un document xml amb l'identificador i el nom dels jaciments 
	 * de les eres especificades a la llista que es passa com a paràmetre.
	 * @param eres Si és null, es retorna la llista amb tots els jaciments de totes les eres. 
	 *             Si no és null, es retorna la llista amb els jaciments que pertanyen a les
	 *             eres especificades.
	 * @param con La connexió a la base de dades que s'utilitzarà per fer la consulta
	 */
	private Document getJaciments(String eres, Connection con) throws SQLException {
		Statement stmt = con.createStatement();
		String whereEres="";
		if (eres!=null) whereEres = " and pj.periode in ("+eres+")";
		String select = "Select distinct id, nom, xwgs84, ywgs84 from jaciments.jaciments j, jaciments.periodes_jaciments pj where pj.jaciment=j.id "+whereEres;
		ResultSet rs = stmt.executeQuery(select);
		Element root=new Element("jaciments");
		while (rs.next()){
			Element jaciment =new Element("jaciment");
			jaciment.addContent(new Element("id").setText(rs.getString(1)));
			jaciment.addContent(new Element("nom").setText(rs.getString(2)));
			jaciment.addContent(new Element("latitud").setText(rs.getString(4)));
			jaciment.addContent(new Element("longitud").setText(rs.getString(3)));
			root.addContent(jaciment);
		}
		rs.close();
		stmt.close();
		return new Document(root);
	}

	/**
	 * Retorna un document xml amb totes les dades del jaciment buscat
	 * en forma de diapositives.
	 * @param id L'identificador del jaciment que s'està buscant.
	 * @param con La connexió a la base de dades que s'utilitzarà per fer la consulta
	 */
	protected Document getDiapositives(int id, String eres, Connection con) throws SQLException{
		Statement stmt = con.createStatement();
		String select = "Select id, nom, poblacio, imatge, xImatge, yImatge from jaciments.jaciments where id = "+id;
		ResultSet rs = stmt.executeQuery(select);
		Element root=new Element("diapositives");
		String nomJaciment="";
		String poblacio="";
		String foto="";
		int x=0;
	    int y=0;
		while (rs.next()){
			if (rs.getInt(1)!=0){
				Element diapositiva = new Element("diapositiva");
				diapositiva.addContent(new Element("tipus").setText("jaciment"));
				nomJaciment = rs.getString(2);
				diapositiva.addContent(new Element("nom").setText(rs.getString(2)));
				poblacio = rs.getString(3);
				diapositiva.addContent(new Element("poblacio").setText(rs.getString(3)));
				diapositiva.addContent(new Element("urlImatge").setText(rs.getString(4)));
				diapositiva.addContent(new Element("xImatge").setText(rs.getInt(5)+""));
				diapositiva.addContent(new Element("yImatge").setText(rs.getInt(6)+""));
				if (rs.getInt(5)!=0) {
					foto = rs.getString(4);
					x= rs.getInt(5);
					y= rs.getInt(6);
				}
					
			}
		}
		rs.close();
		String wherePeriodes = "";
		if (eres!=null && !eres.equals("")) wherePeriodes = " and pj.periodes in ("+eres+")";
		// Afegim els Períodes
		select = "Select p.nom, pj.dates, pj.descripcio, pj.altres, p.id " +
				 "from jaciments.periodes_jaciments pj, jaciments.periodes p " +
				 "where p.id = pj.periode and pj.jaciment = "+id+" "+wherePeriodes+" "+
				 "Order by p.id";
		rs=stmt.executeQuery(select);
		while (rs.next()){
			boolean periodeAfegit = false;
			String nomPeriode = rs.getString(1);
			String dates = rs.getString(2);
			String descripcio = rs.getString(3) ;
			String altres = rs.getString(4);
			int idPeriode = rs.getInt(5);
			
			// Afegim els objectes;
			String buscaObjectes = "Select n, descripcio " +
								   "from jaciments.objectes " +
								   "where jaciment="+id+" and periode="+idPeriode;
			Statement stmtObjectes = con.createStatement();
			ResultSet rsObjectes = stmtObjectes.executeQuery(buscaObjectes);

			while (rsObjectes.next()) {
				boolean objecteAfegit = false;
				String descripcioObjecte = rsObjectes.getString(2);
				int idObjecte = rsObjectes.getInt(1);

				//Afegim les imatges
				String buscaImatges = "Select url, ximatge, yimatge " +
						              "from jaciments.imatges " +
						              "where jaciment = "+id+" and periode = "+idPeriode+" and objecte="+idObjecte;
				Statement stmtImatges = con.createStatement();
				ResultSet rsImatges= stmtImatges.executeQuery(buscaImatges);
				while (rsImatges.next()) {
					String url = rsImatges.getString(1);
					int xImatge = rsImatges.getInt(2);
					int yImatge = rsImatges.getInt(3);
					Element diapositiva = new Element("diapositiva");
					diapositiva.addContent(new Element("tipus").setText("imatge"));
					diapositiva.addContent(new Element("nom").setText(nomJaciment));
					diapositiva.addContent(new Element("poblacio").setText(poblacio));
					diapositiva.addContent(new Element("periode").setText(nomPeriode));
					diapositiva.addContent(new Element("dates").setText(dates));
					diapositiva.addContent(new Element("descripcioPeriode").setText(descripcio));
					diapositiva.addContent(new Element("altres").setText(altres));
					diapositiva.addContent(new Element("descripcioObjecte").setText(descripcioObjecte));
					diapositiva.addContent(new Element("urlImatge").setText(url));
					diapositiva.addContent(new Element("xImatge").setText(xImatge+""));
					diapositiva.addContent(new Element("yImatge").setText(yImatge+""));
					root.addContent(diapositiva);
					objecteAfegit=true;
					periodeAfegit=true;
				} //Final imatges
				rsImatges.close();
				stmtImatges.close();
				
				if (!objecteAfegit){
					Element diapositiva = new Element("diapositiva");
					diapositiva.addContent(new Element("tipus").setText("objecte"));
					diapositiva.addContent(new Element("nom").setText(nomJaciment));
					diapositiva.addContent(new Element("poblacio").setText(poblacio));
					diapositiva.addContent(new Element("periode").setText(nomPeriode));
					diapositiva.addContent(new Element("dates").setText(dates));
					diapositiva.addContent(new Element("descripcioPeriode").setText(descripcio));
					diapositiva.addContent(new Element("altres").setText(altres));
					diapositiva.addContent(new Element("descripcioObjecte").setText(descripcioObjecte));
					diapositiva.addContent(new Element("urlImatge").setText(""));
					diapositiva.addContent(new Element("xImatge").setText(""));
					diapositiva.addContent(new Element("yImatge").setText(""));
					root.addContent(diapositiva);
					objecteAfegit=true;
					periodeAfegit=true;
				}
			} // Final Objectes*/
			if (!periodeAfegit){
				Element diapositiva = new Element("diapositiva");
				diapositiva.addContent(new Element("tipus").setText("periode"));
				diapositiva.addContent(new Element("nom").setText(nomJaciment));
				diapositiva.addContent(new Element("poblacio").setText(poblacio));
				diapositiva.addContent(new Element("periode").setText(nomPeriode));
				diapositiva.addContent(new Element("dates").setText(dates));
				diapositiva.addContent(new Element("descripcioPeriode").setText(descripcio));
				diapositiva.addContent(new Element("altres").setText(altres));
				diapositiva.addContent(new Element("descripcioObjecte").setText(""));
				if (foto.equals("")) {
					diapositiva.addContent(new Element("urlImatge").setText(""));
					diapositiva.addContent(new Element("xImatge").setText(""));
					diapositiva.addContent(new Element("yImatge").setText(""));
				}
				else {
					diapositiva.addContent(new Element("urlImatge").setText(foto));
					diapositiva.addContent(new Element("xImatge").setText(x+""));
					diapositiva.addContent(new Element("yImatge").setText(y+""));
				}
				root.addContent(diapositiva);
				periodeAfegit=true;
			}
			
			rsObjectes.close();
			stmtObjectes.close();
		} //Final Periodes
		Document doc = new Document(root);
		stmt.close();
		return doc;
	}
	

}