package net.nitus.museu;

import java.io.IOException;
import java.io.PrintWriter;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import javax.naming.InitialContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;

/**
 * Servlet que desenvolupa la pàgina principal de l'aplicació
 */
public class Jaciments extends HttpServlet {
	private static final long serialVersionUID = 1L;
    
    /**
     * @see HttpServlet#HttpServlet()
     */
    public Jaciments() {
        super();
    }

	/**
	 * @see HttpServlet#doGet(HttpServletRequest request, HttpServletResponse response)
	 */
	protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		response.setContentType("text/html;charset=utf-8");
		PrintWriter resposta = response.getWriter();

		DataSource ds = null;
		try {
			InitialContext cxt = new InitialContext();
			if ( cxt == null ) {
			   throw new Exception("Uh oh -- no context!");
			}

			ds = (DataSource) cxt.lookup( "java:/comp/env/jdbc/postgres" );
			if ( ds == null ) {
				   throw new Exception("Data source not found!");
				}
			
			Connection con = ds.getConnection();
			resposta.println(Web.getHeader());
			resposta.println(getMapsHeader());
			resposta.println(Web.getMarc());
			resposta.println(getMapsBody(con));
			resposta.println(Web.getFooter());
			con.close();

		} catch (SQLException e) {
			resposta.println("S'ha produit l'error d'SQL: "+e.getErrorCode()+"-"+e.getSQLState()+"<br>"+e.getLocalizedMessage());
		} catch (Exception e) {
			resposta.println(e.getMessage()+"<br>"+e.getLocalizedMessage()+"<br>"+e.getStackTrace());
		}
	}

	/**
	 * @see HttpServlet#doPost(HttpServletRequest request, HttpServletResponse response)
	 */
	protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		// Redirigim la petició cap al get.
		doGet(request, response);
	}
	
	/**
	 * Genera la cadena que conté el codi html i javascript necessari per l'aplicació web.
	 * @param con
	 * @return
	 */
	private String getMapsHeader(){
		String resposta="";
		resposta += "<script src=\"http://maps.google.com/maps?file=api&amp;v=2&amp;key=ABQIAAAAcb4Yq65GNf0jP1GLEd1XAhQxGoLwwlb8Pp8tHIAGtxnwO4W_0xRdM8HwqHDCTmsNsIZ7_J_4mkceqw&sensor=false\""+"\n"+
					"type=\"text/javascript\"></script>"+"\n"+
					"<script src=\"/museu/js/extlargemapcontrol.js\" type=\"text/javascript\"></script>"+"\n"+
					"<script src=\"/museu/js/diapositives.js\" type=\"text/javascript\"></script>"+"\n"+
					"\n"+
            		"<script type=\"text/javascript\">"+"\n"+
					"\n"+
					"// Conté els jaciments que cal mostrar.\n" +
					"var lJaciments = [];\n" +
					"// Designa si s'està en mode museu o mode indicacions\n" +
					"var modeIndicacions = false;\n" +
					"// Conté el mapa per poder-hi accedir des de qualsevol lloc.\n" +
					"var map;\n" +
					"// Conté el gestor de peticions AJAX\n"+
					" var req = GXmlHttp.create();\n"+
					"// Controls del moviment \n"+
					" var moure = false"+"\n" +
					" var minLatitud=41.150000;\n"+
					" var maxLatitud=41.700000;\n"+
					" var minLongitud=1.660000;\n"+
					" var maxLongitud=2.370000;\n"+
					"// Contenen les polilínies \n"+
					" var baixLlobregat;\n"+
					" var molins;\n"+
					"// Contenen els marcadors del museu (antic i nou) \n"+
					" var museu;\n"+
					" var ametller;\n"+
					"// Les Icones \n"+
					" var museuIcon;\n"+
					" var jacimentIcon;\n"+
					" var TabbedPanels1; \n"+ 
					"\n"+
					"// Aquesta és la funció que inicialitza el mapa i el mashup \n"+
					"function initialize() {"+"\n"+
		            "  if (GBrowserIsCompatible()) {"+"\n"+ 
		            "     map = new GMap2(document.getElementById(\"map_canvas\"));"+"\n"+ 
		            "     map.setCenter(new GLatLng(41.41275, 2.017322), 10);"+"\n"+ 
		            "\n"+
		            "     // L'escoltador dels events dels moviments\n"+
		            "     GEvent.addListener(map, 'moveend', onMoveEnd);"+"\n"+
		            "\n"+
		            "     //Afegim el selector de vista\n " +
		            "  	  var mapControl = new GMapTypeControl();"+"\n"+ 
		            "     map.addControl(mapControl);"+"\n"+ 
		            "\n"+
		            "     var opts2 = {"+"\n"+ 
		            "                zoomInBtnTitle : \"Augmentar\","+"\n"+ 
		            "                zoomOutBtnTitle : \"Reduir\","+"\n"+ 
		            "                moveNorthBtnTitle : \"Vista panor&agrave;mica amunt\","+"\n"+ 
		            "                moveSouthBtnTitle : \"Vista panor&agrave;mica abaix\","+"\n"+ 
		            "                moveEastBtnTitle : \"Vista panor&agrave;mica a la dreta\","+"\n"+ 
		            "                moveWestBtnTitle : \"Vista panor&agrave;mica a l'esquerra\","+"\n"+ 
		            "                homeBtnTitle : \"Tornar a l'&uacute;ltim resultat\""+"\n"+ 
		        	"     };"+"\n"+ 
		        	"     var extLargeMapControl2 = new ExtLargeMapControl(opts2);"+"\n"+ 
		        	"     map.addControl(extLargeMapControl2);"+"\n"+ 
		            "\n"+
		            "     //Definim les icones \n" +
		            "     //La icona dels museus  \n" +
		            "     museuIcon = new GIcon(G_DEFAULT_ICON);\n"+
		            "     museuIcon.image = '/museu/image/marcaMuseu.png';\n"+
		            "     museuIcon.iconSize = new GSize(37,27);\n"+
		            "     museuIcon.shadow = '/museu/image/marcaMuseu_sombra.png';\n"+
		            "     museuIcon.shadowSize = new GSize(57,27);\n"+
		            "     museuIcon.iconAnchor = new GPoint(18,13);\n"+
		            "     museuIcon.infoWindowAnchor = new GPoint(18,3);\n"+
		            "     museuIcon.imageMap = [ 0,27, 1,6, 19,0,  36,6,  37,27];\n"+
		            "\n"+
		            "     //La icona dels jaciments  \n" +
		            "     jacimentIcon = new GIcon(G_DEFAULT_ICON);\n"+
		            "     jacimentIcon.image = '/museu/image/marcaJaciment.png';\n"+
		            "     jacimentIcon.iconSize = new GSize(20,25);\n"+
		            "     jacimentIcon.shadow = '/museu/image/marcaJaciment_sombra.png';\n"+
		            "     jacimentIcon.shadowSize = new GSize(38,25);\n"+
		            "     jacimentIcon.iconAnchor = new GPoint(10,13);\n"+
		            "     jacimentIcon.infoWindowAnchor = new GPoint(10,13);\n"+
		            "     jacimentIcon.imageMap = [ 2,25, 2,14, 3,10,  0,9,  0,4,  5,0,  25,0,  29,4,  29,9,  27,11,  29,17,  29,25];\n"+
		            "\n"+
		            "     //Afegim el museu actual\n" +
		            "     var museuPos = new GLatLng(41.415138, 2.01712);\n"+
		            "     var textMuseu = \"<table>\"+" +"\n" +
		            "                        \"  <tr>\"+"+"\n" +
		            "                        \"    <td>\"+"+"\n" +
		            "                        \"      <strong><a href=\'JavaScript:obreReproductor(\\\"reproductor\\\")\'>Museu Municipal de<br>Molins de Rei</a></strong><br>\"+"+"\n" +
		            "                        \"      Carrer del Pintor Fortuny, 55<br>\"+"+"\n" +
		            "                        \"      <u>08750 Molins De Rei</u><br>\"+"+"\n" +
		            "                        \"    </td>\"+"+"\n" +
		            "                        \"    <td>\"+"+"\n" +
		            "                        \"      <img src='/museu/image/elmuseu.jpg' alt='Museu' width='137' height='91'>\"+"+"\n" +
		            "                        \"    </td>\"+"+"\n" +
		            "                        \"  </tr>\"+"+"\n" +
		            "                        \"</table>\";"+"\n" +
		            "     museu = new GMarker(museuPos,{title:\"Museu Municipal de Molins de Rei\", icon:museuIcon});"+"\n" +
					"     GEvent.addListener(museu, 'click', function() {"+"\n"+
					"         museu.openInfoWindowHtml(textMuseu);"+"\n"+
					"     });"+"\n" +
		            "\n"+
		            "     //Afegim el futur museu \n" +
		            "     var ametllerPos = new GLatLng(41.414024, 2.016185);\n"+
		            "     var textAmetller = \"<table>\"+" +"\n" +
		            "                        \"  <tr>\"+"+"\n" +
		            "                        \"    <td>\"+"+"\n" +
		            "                        \"      <strong>Ca n\'Ametller</strong><br>\"+"+"\n" +
		            "                        \"      Plaça de Catalunya<br>\"+"+"\n" +
		            "                        \"      <u>Molins de Rei</u><br>\"+"+"\n" +
		            "                        \"      <br>\"+"+"\n" +
		            "                        \"      Ubicació futura del museu.\"+" +"\n" +
		            "                        \"    </td>\"+"+"\n" +
		            "                        \"    <td>\"+"+"\n" +
		            "                        \"      <img src='/museu/image/ametller.jpg' alt='Ca n\'Ametller' width='137' height='91'>\"+"+"\n" +
		            "                        \"    </td>\"+"+"\n" +
		            "                        \"  </tr>\"+"+"\n" +
		            "                        \"</table>\";"+"\n" +
		            "     ametller = new GMarker(ametllerPos,{title:\"Ca n'Ametller\", icon:museuIcon});"+"\n" +
					"     GEvent.addListener(ametller, 'click', function() {"+"\n"+
					"         ametller.openInfoWindowHtml(textAmetller);"+"\n"+
					"     });"+"\n" +
		            "\n"+
		            "     //Afegim la comarca. \n" +
		            "     baixLlobregat = new GPolyline.fromEncoded({ \n"+
		            "    	 color: \"#ff0000\", \n"+
		            "    	 weight: 3, \n"+
		            "    	 opacity: 0.8, \n"+
		            "    	 points: \"e{r{FasmJmc@bSuSkHyC{Cq@[_CDcA[qDoBuCu@{@?yAHu@Oq@sAmAgAa@w@Ek@_C{Cu@{DU{CoBqImA}@iBa@}ADeDuHmEbKwMxJqDsAeDgBoGqCg@mBgA{CaAyBw@mAmA}Aq@q@aEsFqHyFoG_C_GcAaEqAsBdLkCEwDgAgFmEuCiFcBwA}DmF}Az@}A`@oBEgBgA_BcBOw@EiCj@aDJiBNqA}IaImAg@}Aa@iCCyEpAsJtFaAfAe@^mAJsANw@f@gErFiChF}@fAiCbCiB~AiCjAuC~BqCtCmBbBcBp@uCpCiBhCeCzCcBtCmSbScRlMgEpEoCrEk@pAuB~AkLtH[j@_@ZeClAiBrB[P{@fAq@N[P[^a@T_@?a@Ua@Ee@De@E[ImA}@[OO?m@p@q@j@u@Jm@?wAHoB_@q@?q@NsBvAk@Jw@E}Av@a@?a@U_D{C[E[T[f@[Z{@NsAIaAWgAV}@a@wAK[?QPq@KUUO[?IsAa@gANq@q@aA?aAk@cA`@qAz@iAhBu@?}@Z{@v@q@NuBe@aA?}ExByBZw@hCaD~FUpDe@bByAdD[rAk@`AcB~GaAnBaAjDg@`@wAj@kHrNQ`AYjHQnBq@xBqHpPqAnDgAhB{BtBcBrBaArAyB`J}@hBu@`A_BdCoCvD{CbGuChFUpDiCrAuGpA{@bBa@TkL~AaATcAd@qA`AqCnFqAbF}@xBEvA?v@JlAwDuCeDDyBiBgAoBm@u@_@q@gAUq@cA}DqAe@Kw@E[e@_@Eg@`@gA^q@IgAk@}@w@cB_CoCyBsAmA?uKIwIq@aEZqCB}AImEH{@rAeC`@cBq@eD[eDOiA?cBKk@Uk@k@EiBf@gAK[Uk@q@Q[_PsFcFrByB~BwAtCm@ZU?uDk@iBa@g@[_@_@Um@UUgI~BcJnCkCl@CTa@j@k@Zq@Ha@POTE^?rANjCPd@Ej@KZq@f@u@t@Kp@DT`AhBDZETq@bAUj@OfAgAtCQlBD`@E`@OTa@NqADw@`@k@xBUpAKvHKj@DxAa@hCU|@[^e@ZgAE{CxAUTKj@`@v@Nv@Iv@OX[VgABq@Cg@N[TO`@O~BBp@Ut@UPmAKq@f@{@NiBNa@JwAz@g@l@e@fA}@xCIp@Ul@QlBIp@Uf@q@z@}@|@oClB}@bA[lAe@hCsAvD_@^}@ZmAxAk@^sAp@qATgAl@cAnCI`@c[pMuGnP{LoLyAdD?fA`@nBU`Ag@f@q@`A}A~FuClFwAhB{ClAgEfAiGdDoBrA}Ad@kDdCyAj@iG_Dw@Ee@p@gAd@eCu@w@j@kHnKmAtDOfEv@nFNtCk@~B_BrFwAvH[`AEz@JlAj@~Av@|ADvAKv@k@p@gIzPK|@v@hCp@tCEpD_@hBuCtGw@vA{CbBaErBw@v@aAxAO~BQtH_DcFeCmF}EoF}Aq@_Cq@oC}M_OqRmF_F_BkAkDiBu]mMmViGooCgRe@a@aAcBkCeCrEZz@IbAiBv@aEBiBv@uCrAqDtHgIv@_CT{CTwAf@}A|AuBlAaAhBsBbBgAvAa@`@?t@[f@a@rAsBz@aAv@m@dDwAjCwAj@m@p@{@bBaDp@}Ad@yB?cBD}@TgBlAmEp@mAZ}@`@u@Tm@H{@IcAQ_@_@k@oB{Bq@_DOkCDk@H[?q@Iu@Bm@u@wE[wAQiAIcG?}@H_DJ}@rAaEHk@Tw@Jq@Dk@Kq@Pg@[_CHe@Pa@Hk@?w@De@Hg@ZUTe@?k@Cm@q@aAUe@Km@?k@Je@^mA?k@_@gAHm@ZUT_@Nm@JqADkCQmBJiBEiFDiBDkIcFs{@hB{DrBcF`@yB^kCp@}Av@gAnCwAxA}IEiFOoCsAeKk@yGKgINgEp@}AtB_C|A_BdCcBxBkAxBcAjDqATgEp@qCf@}E`AmEd@aF`@aD`@cBhG_PnCuB|AgBpC_BnL|i@bEiS|k@w}Bp@eC`@mBU}Q[{CUqLW{DCaMQ}D?kDlAO|A[bA`@TEZa@Hk@Ta@hAwAt@a@l@cBBmBJk@Nm@ZUZe@|@k@z@ErAZ^nC?v@Jz@d@p@|@E`AVpDhCj@|@v@j@f@?d@KZ[Ne@Iq@q@q@Ug@?e@Hk@p@k@f@WvA_@ZDT^DjC`@^d@D`A?v@DZN?v@Ep@J|@ZTZKd@[d@OhADj@Tj@DZKZ[Ze@ZyAEq@sAmAU[Jk@TO|A?dCp@t@p@v@fAv@vAvEbIbA`Aj@`@^E`@KDw@[YUg@Ek@Og@De@^E|@PfAKTk@?k@Iq@HkCNq@l@iBNwAKwDDk@Je@vEeCNa@xAgArFqDp@q@N_@?m@u@{@m@w@Uk@UgADw@TUfAHrBv@|@j@v@hBH|@DbCNbB`@p@f@O^UbAgATq@HkC`@qAv@g@fAOZKT[^gAJcBN_@f@g@TEJJ?`@BNPJXBVO^}@`@Cd@Nf@d@NZDl@NT`@H`@I^a@|@}ApAcA|@If@?j@Tt@Nl@U~BUz@a@Nk@Iq@e@{@Bg@`@U`@I|BX~BCxAq@HUJm@UmAEe@Jk@TUd@BnB|@pAfAp@?bAOZNNZHp@ZZf@Hd@IZO`AcBbA_Cz@iB~AiB^EZJZZj@~Cj@rAZ`@`@?TUT}@P}ANU`AUdC[dC}@j@[p@w@^q@rA_DrAoBlAaAvA}@nCYxArBv@Hp@e@d@}@j@{DxAsBxFyE?[a@_@[iBJsBTgAj@mAZgA?yANa@l@q@t@Yj@[ZuBl@k@Xq@?{@q@_CIyADqA[m@nGqUnOkLjIO`MmA~s@JjDbx@j@nGJj@Ep@TrANpAf@tB?t@QrADj@f@|@Nd@?j@g@~B?l@|@Tp@d@lF|@hBBnBJfATvATp@TZZv@hBp@dDp@fAZ`@~FjCpANhAEbCaAxA[fBDZBbAl@|AbBZHp@ENP|ADTTp@|Al@d@jA`@v@|@f@Xp@v@ZP`ApAd@JZE`@DZNT`@`ApDD^El@lAtCTz@D`AIbAQ`ADp@Jf@Bd@Ud@k@Zk@v@g@N_@f@yBlBXZbArANNZf@NNlAnBbAZ`B?PBNJTTPTXp@`@`@p@d@j@Z`@DzDHVDd@Pp@?j@EbBUbAKz@O|Ag@p@_@bBcBp@w@l@g@j@k@T[`AiCbAmEp@iCj@qCv@}Fd@aD`@_DZiBTiB`AiFZ{@N[Ta@l@w@XUVIhC[lAa@xBgAnBw@j@Id@E|A?`@DnBj@p@Nt@D`@?`@Ev@UlBaAxB}@`@UHOr@sBNQbBmA^O`@UZK`EcCv@m@j@k@p@mA`@e@`@KjA_@lAU`@E`@Kp@UNONKZ[PUN[Z{@^}@l@w@vAgA|@k@lAaAfF}DbBwAj@k@JETa@?aADKTUJET?TCfA}@d@[~BoCxAoB^k@Zq@Je@Ng@?[DYEW?UOgAUe@QQcBaAuCsA{@{@sBeDsAcByAcBcBuCaAmAsAgA~NmDrBtBdC?|IeG|AdCbFcJtCmEvDeHbBoCfBqDjC_GxBqErEaIsEcIiBcCw@mAq@sAwAeD}AeD{CkGmA{CU{@?[D[TuGD_@dDz@jC[jDgArFyF~BqDzDkInCwHbEiFfFsBpH[fENhCxAVHt@?TDPDNNPJN?TETIZETKZ?TEZDT?d@If@Ej@KTITQ~FyFTY|@oB^k@`@a@p@e@jDsA|@a@p@_@v@w@`AcBhB}Dd@{@rFcJPOZUXQf@_@HQv@sB|@iBd@w@rAgAv@u@Za@|AkC`AgBxA{C|A{CfAsBbBuCp@w@~CaDnBcBrFqEaA[eD}AiByAUCcA?YWa@CsANe@J[EQKq@k@_@OUEQDu@?[EQEUUw@[O?IDq@`@g@Be@J}@DwA[sAa@q@Ew@U[DUJUToC^a@?e@IEEe@D[Hg@Ue@?[ZE?gEe@[BmB?}@JiB`@q@T_@Dg@?cB[k@NO?QJoCTaA?}@BaA?a@CqAa@w@[mAk@k@g@EIKK?I_@iBKEEI_@a@m@}@Uu@q@wD_@gA}@k@k@[kCiB{@w@a@k@O[a@sA[k@YOW?UDk@T[DIHQDOHsAzCU`@_@d@m@f@wA|Aa@TODU?UDw@?cAQOOIEa@aA[g@[YOEEEU?rRy|@kCqDqAeDq@eDEkC[eDk@yEQk@IyBDsBfAuGEmIN{C~AuHNwEdCqCfBmAl@I|B`@tCp@`DQfEwA|Aw@pDa@dCxBZCvMaIfNkKbAkLTwAv@{CrB{@~FB|Fq@bEcG?cBKkDk@uCcA{GCoCm@wEe@yA{D}EyBwAByECsBa@yBk@cB?k@TsA?uCq@eHQkC?yBT}AJsADcBNcBaAgJDsAKw@[qAEm@?yBDw@Eo@}AcFlAsEd@yBl@mNa@_OfAsB|A_ChBsBEmApC_BjA_@|@QZ?vEbBjDtBdKvDfE~Ck@p@a@|@k@nCtCrBjD[lF`@_DvIT?xBTrBItCUfDa@dHyBfFQ`D`@zHdCjDDjC^bBdC~BzGtCzL`AlBjCyFbB_DhCqDrAmAvA}@dHgAtBOlAk@bBa@t@EbA?|Fl@zCjAjHhJrEq@xBNhCe@zCaAlAsA`@}AZ_CWaEsB_CiB}@mBEeCv@E{@TgECuFk@Ym@s@aAqIcBiByFuCoGaAgAUyBfAa@[?sEKyCmA}LUuCUkAaAuBlA{CrFuLhB[ZTZf@B`A?lATbBlAvDbBjE`@`@|@d@|ADpAk@hAa@pEfAlA?d@YZw@U}Aw@iAsByBsAuCu@_@sAa@cBcByB_D_CiB_CmAfAcBPuBKyCDq@d@k@|AsAxAeCK_CxBPlE`AxAj@pAfAxB|Af@xFj@rENfEgAlErAtClA`A`ExArEj@bCa@JzGNv@fAHlIiF~KiHl@sABsFrAkLZ_Gq@kC}@sBwAiBq@mAyBcBmEcB_CuCsFqIsBuCmA[cA?u@j@cAkDIqETsAj@cB|AaE~AgIj@oFj@wNNuF[{HaAuCaAe@g@}@UmBp@yFhBgHbBoCnBaAnCq@fBQpDk@p@qEhBiBlEw@hFlAtChBxC^vDzBnBlBfBxBzCtBvHpElA`A|E~AlATz@?fAKtB_@lBsA~BoClEQbC[tBu@bBBfEcClAsAp@sBpAcBl@xBTp@z@`@fEcF|@w@hBKpATrAl@fEfFfAcBlEoHzD|A~FsAbBaAtCpEj@e@fAHv@`D~BnKdLoGv@?nXnOtGuBlIn[kCdHqEPmAnH?bRpL`I`AJjDHbBk@hBmApEaMxAa@lFiCjDsBp@w@lAaEtBuCzDwDZk@Dm@Kq@k@uC~CcBvHkEvATlYaQzLh[vI`RnBpDpExFhBpDnKyV`EwMhNzTH[PU?IBK?gACa@BUJ[ZYHEDK`@sBv@eDbByET[NIZK`@q@j@wAHmAPw@Tq@Nq@lAyBbBsB|@}@vAeCp@{@d@w@r@iBn@}AVw@Nu@v@_ClAiBfAgAjDaEpAmEv@oBrF{H~FoFj@ObEsBdHgE|I}HbB}B~BgEhB}DfEoO`AwEv@sEfAoHfA_Kz@sJbAmI^}EJoCf@sJHuG?yFIwDa@aIQyNJoL?uFj@g[IoFNDT?`@JbBpATTj@`@p@T`@d@pAl@PN`A`@tCbB`@ZrR__@lFrE?ZED?BU?_DoCa@IOBQJaQh\\\\zC~B`@PTBZPZTlArAd@^j@ThAZd@TrAv@d@`@`@d@|A|AZP^?VDp@t@^p@TTTPl@NxFxBfAv@fAfA`EtCj@p@p@j@p@v@tBhB^?ZKZUNEJ?TJTTPTxBnBlA|AXj@zOkUbOtP_PlUJHlBv@Td@`@`@p@`ArAtBd@z@JJrOeTfAsApDkD~AmAlBw@vAe@dBUH??DD??BD??PE??HE??DcBTwAf@iBt@}AhAmDjDgAlAINDd@?bAH`EJdCj@pEd@jCfAvEp@xBl@xBd@lAzCvIp@bB`@rAnDqDD??ED?DDD??DB??^gEfEJnBHfAZbAxBlFxAdClAbBT^z@tBhAlBt@lAl@bBhCnGxB|Dj@t@J`@v@|ATv@j@rAv@bCvAvD`@rAd@lAbAdDd@vAfAbFbB~F`@bBN|@`@vANv@TTf@NKZJ`@N^JhA`A|ElAbFvAjD|@~CvAxEnBjHv@xC`AxEdHpZlEvQv@~Bp@fDlAvExCfNp@|D|@hGrAjH`Jda@~A`ITv@ZET`@?ZQZCN?tBZnHNnB`@tCrB|Id@tCThBDbCPlA^tB`@NDT^rE|@pEHxAl@nDj@hF`@hBp@pE^vDlAlIxAzH`A|ErAvIpAxFT~APrB^`DJxBTfATp@f@EHN{@f@Ed@DlAj@xJp@hFNj@Jj@HbBZlATfEZtCT|@TrBZ`EPv@`ApHz@bGPxBTv@?`@TzCj@hCPnBHd@rApL`@hCNnCj@tFNhC`@jD|@vLTvE^dD?lATrBJbB?xEj@fIJj@?rANjAJrET|ADbBH|A`@jLZbENnDDzBj@`RTnFJxCBjCZjIDfHJfBNhFNtOZpM?nBD|AKxJD|A?bBIvD?`EHnCJnG?bFE`IDpLK`IDhBEtGDrF?fEK|A?dCD`AItCBf@Jd@K|A?bBItBKjA?|QD`EI`DEvDEdLDlB?fDEfB?xADhB?jEDlIH`DEbG?jGDnD`@~NH`QPpEBtC`@vHDrAqQhK{CfEoBpD}ApEa@l@u@t@kD[qCq@gBrAkDhBcJfEcExBgB?_Ba@kAmAeCaEyA_D}IaUk@k@k@Uw@EyBp@k@p@q@pDoBnGoCxKgN|^cAtFyCjEwTp]{HjDgAbAaAnC`AdGxBjUzCzPDnBfErNzYbTzHlIU`AgAhSrR`U}Nhx@}EzTfBlBrAdCDjC?zHnP[xAjHfFlFwAt[iBzPsAtGC`ArBhKp@xA}Id@}AdCq@pD{}@sJcOgImMrVJ^?|MH|@DtDJxB?lDUbCUbBElAa@fAq@bAqAj@w@Ha@CmA[a@?aBz@cAbBq@hBOp@Dd@Tf@ZT^JbA?`ATz@p@Pd@?j@}@`DIj@kCbF[hBDd@v@|AJp@Ql@yBbCKf@k@UaAq@q@q@w@QgA|DeC~Kk]`]_Cm@qIkEmVj@oGuCgQmIsAIUBgFlEiBnCmA~Bv@_DZoFEmBKuBe@mBsAsAcC{@q]{Pk@Ka@UQa@_@Og@KsBgAa@I_@U}@a@aAoC[Qg@q@Yk@w@q@cAOu@T[ZQKCwAmAw@Ue@Hg@`@k@HUCk@Ka@[g@}AYa@iA_@Oa@D[Z}Aj@kC?_@J}@?{@a@[a@a@qAe@a@U?w@k@k@q@w@w@g@w@_@wAw@_Cq@{CyAuC{HuS_HcTsAKk@ZuGfAmI`EoCcBsBmB{CyJa@}DJUd@kMw@oO`@uC|@qPaEqDkCcBwAcCsAgHoBwEgAkDiBwEqA{CmEeGeCqAoCcEa@iCCsEq@gEq@cB}@cBmF}B_KuFsFbFe\\\\`YgEvDwExCe\\\\l@qU`M_L`g@g@~BaA~BuL|U}QzPqD~FUbCK~BqAvD{GvAyFnByV~BmBiFwLkj@\", \n"+
		            "    	 levels: \"PGCADCABC@AHAAABDABECBDGDEGAE@B@D@@CGBBFDFCDBBEADCFBDBA@FD@BJCB?C?BEBAD?BBA@BBD@ACDDB@E@B@BB@@B@A@HA@A@C@?AE?C@?BADACABAE@DB@?BC@BEB?AAC@@CBCBBF@BCABDBADBIBCBCAABBADACEA@BEACAA?DBD?@B@DGBCAC@C@FBAD?AJDDB??BBEA@BDBACADAB?G@CC@BDAADB?AA@HABD@@CFCBBE@@D?A@F?AC@B@@J@ABAADA@BD@B@E@BAC@@D@BBCEBAAB@F@BC@EB@BD@B@B@DAAAEABB@?CAEA@@B?D@B@AD?AHAA@CAAD?DEGGCABD@BBAFACABBAFDBADEAHBDAD?B@AEB@CAEABE@CFB@D?C@E@JADC@DDF@BDHEEAAMAEBAC@DDB@CBF@ACAACA@CA@B?F@CB?ACB@??HAC@ACDA@A@@C??FA?DA@?C?AABC??A@?B@D?B?B@D?CABA@AC@EBA@?DF@C?BDBDFABACAHB?D@B?FBAAA?AEDABHHB?@GBAAB@@K?CAEA@BADBA?D@ABGCA@BDACAAF@C@DB@C@D@AFABBD??AD?BAF@?CA@C@EACB?CAEBF@B?D@C?HB@B@ADAACE@AB?ACA?FBAAB@EB?ACBAGBAEB@@BG?ACAADA?D@BB@FB@?B@DACAD@AABEA@BD@B@CAFBCACBEABCE?B?BAFABC@CA@ACAFAB?BFB@CACAFA@AD@B@E@AAC@EDBHACDBEAABD@@BADA?CBAEABABEGEBJHC?AB?@C@AC@C@AHABB?B?@EABB?DEACACEABCAAAG@BAB@?@AC@@AD@F@B@CA?B??EAAAA@E????AGA@D?A?@B?D?A?E?A??B@K???C@E??B????AC?A?G?AAB@D?AC@?B?D@A??F?@C@??A@C@@D?@??B@?D?A??AC???@??DA@C?A?A?C@A?H?A?C??AE?AADA?AB@HECEEG@A@B??AHA?C???@E@B??HDBFBCACGCDC?B?B@?D?@??A?@?A?B@?GB@A?DA?B@E@?AC???B?@?C@?C?@?@?E?A?J@BD@AAB?AC?@B@@?B@@F?@B?@D?@@C?AAC@?B@AAB?BB@C@@@C@C@@AB?@?F?A@D?A?B??@B@AD??@C?A@AG?A@@??D?C??@F??@AD?A@BA?@JFBDBA?@AFBCACBHBAE@CDABDBFBFB@FCBEH@C@BAEBCE@CAC@AC@DAA?@DBA@@B?@CF?DCE@ACKBA@EBAFE@BFDBFG?B?CBCEBCAGCBBH@CAEBABAG?BCEFBCAJBDBFBDBFAADACGBD?CCF@B?CAH@DG@C?AC@AFBC?CBDAHB@BD@CACAGCA@D@BDHADA?F@ADFCBECEAGC?CIBACGA@DBCA@EACEBHA?BDAEBDBBCDGBEAB@ECJDBBEB@B@AD@G@ADBDABBECABF?CE@CEACF?EDBGDBE?FBFEIFDECFH@CECEAAFBCAAEA?F?CCHBDBBHBGJ?@?B?@BA??B?CBA?B@CA??EA@A@?C?A?@CA@C@FBD@ACEA@HB@DA@@@F?ACA@AC@ABH??B??AAB@?A?GMB??FACA?FHA??B??B?A?B??BA?C@@?C?C@@B??@DA?@B?D?@A@GHFF@C@@@??HB@EACBA@???I@@??BB?CA@?F???BG@A?C??@F??B????EBD?F@B@?AA?B@B@A@D?@@??@?C@A???C@FB?@A@CB?@C?@AAC?@B?@B@CBBGA?ACA@D@C@ABBACAA@@A@A@C?AAC?@AC?CAEBD?D@?AAAA@A?B??AAA?AA@?@C@@@A@AJA@AB@A@AAA??A@C?@?@?BA?A@E?@@?@@B?A?@BA@@?A@?AB?C@@?B@A?A@?@?D??@A@?CAB?@?JEBAB@FAEA@@HBCEABEBACHBAACCF@DDBIABBDFCFAFHBGAD?FFDGCAAEBLEBFEIFAA@@AC?@FABDB@@CDA@FA@C@ACAFA@BBDABAGAE@AAE@EJBEE?BGAC@HBADACGAABAA?A@@?EBA?ADBABFCADA?C?DBBEABAB@@EABB@AA?GA?@BABGBACEAIBEACCAGADBC@@@ECBEBBC@FAJ@?BFEH?BEDCE@CIBCGBP\", \n"+
		            "    	 zoomFactor: 2, \n"+
		            "    	 numLevels: 18 \n"+
		            "     }); \n"+
		            "\n" +
		            "     //Afegim el municipi. \n" +
		            "     molins = new GPolyline.fromEncoded({ \n"+
		            "    	 color: \"#7534aa\", \n"+
		            "    	 weight: 3, \n"+
		            "    	 opacity: 0.8, \n"+
		            "    	 points: \"ea}{F_`lK|Aj@lBJjGUfAfArBxF|@vAbBlA`M~FfAxBbGKbE`MnDtBf@^`AjHxBhFpCdCiAbCvChJxCaAv@xB`@pA|FrAeDdGhSxx@tOuFw@hNpDvENE|@e@z@a@f@Oz@Kv@OnCw@nFmBj@Qj@Ij@UZ[fAk@tBa@nCUzCKfF?~EfAtDaIP?d@Ope@s[xBm@j@UrA[j@UvAaAv@a@|AmAj\\\\aAlF}@j@aAlAqEbEuKXa@zBmAt@e@tB}@t@Of@OzCg@lBUpDk@j@Q~BaAhB{@uHsNwDcFgEiJk@wAgHePgEmIyC{GgDyGaAeCeCrBkDfD_DvE}Q_T{CmEeDiByAqD{@aAcA[sBkDkDiByB_@oC?EuCTyBDuCa@yAaAmAQ}AJeCO}A}DgAO{DUsAU[w@w@gEgIyAxAU`ADfATbAqDQsBe@uC}Aw@yAOsBlB}AcC_Cw@}DUiCTuG`AqDkHiJ{CkA}Fm@cA?u@DcB`@mAj@uBNeHfAwA|@sAlAiCpDcB~CkCxFaAmBuC{L_C{GcBeCkC_@kDE{HeCaDa@gFPeHxBgD`@uCTsBHyBUU?~CwImFa@kDZuCsBj@oC`@}@j@q@gE_DeKwDkDuBwEcB[?}@PkA^qC~ADlAiBrB}A~BgArB`@~Nm@lNe@xBmArE|AbFDn@Ev@?xBDl@ZpAJv@ErA`AfJObBEbBKrAU|A?xBPjCp@dH?tCUrA?j@j@bB`@xBBrBCxExBvAzD|Ed@xAl@vEBnCbAzGj@tCJjD?bB\", \n"+
		            "    	 levels: \"PBCEBACECDGC@DDCEDFD?DFEFFEJ??B@?@BA?B@@BE@ACFD@BF@?@C@@ECFABAF?@C?A??C@?KBC@?AA@@G@BGBCCCBBECBGAADBCAAEDAD?BFBDAFBDBFCEDADBEICAEAABAFABC@HBBCGACBECBC?B?GFBEEB@FCBBAJ@BDCA@FCD?GC@?B@@ABD@?AAD@CA@CAC@ECBEAB@C@P\", \n"+
		            "    	 zoomFactor: 2, \n"+
		            "    	 numLevels: 18 \n"+
		            "     }); \n"+
		            "\n" +
		            "\n" +
		            "     //Afegim els jaciments\n " +
		            "     carregaJaciments('');\n";
		resposta+=  "\n"+
		            "\n"+
		            "     //Afegim el gestor de rutes.\n " +
		            "     gdir = new GDirections(map, document.getElementById(\"map_directions\"));"+"\n"+
		            "     GEvent.addListener(gdir, \"error\", errorsDireccions);"+"\n"+
		  		  	"\n"+
		      		"  }"+"\n"+
		    		"}"+"\n"+ 
		            "\n"+
		            "// Entra a mode indicacions i carrega les indicacions obtingudes de Google Maps\n"+
		            "function setDirections(fromAddress, toAddress) {"+"\n"+
		            "     if (!modeIndicacions) map.savePosition();\n"+
		            "     modeIndicacions=true;\n"+
		            "     gdir.load(\"from: \" + fromAddress + \" to: \" + toAddress);"+"\n"+
		            "}"+"\n"+
		            "\n"+
		            "//Neteja les indicacions\n"+
		            "function clearDirections() {"+"\n"+
		            "     gdir.clear();"+"\n"+
		            "     map.returnToSavedPosition();"+"\n"+
		            "}"+"\n" +
		            "\n"+
		            "//Control d'errors de les indicacions de Google Maps\n"+
		            "function errorsDireccions(){\n " +
		            "     modeMuseu();" +
		            "     if (gdir.getStatus().code == G_GEO_UNKNOWN_ADDRESS)\n " +
		            "       alert(\"No  s'ha trobat cap lloc que correspongui a l'adreça introduïda. Això pot ser degut a que l'adreça sigui relativament nova o bé que sigui incorrecta. \");\n " +
		            "     else if (gdir.getStatus().code == G_GEO_SERVER_ERROR)\n " +
		            "       alert(\"L'adreça no s'ha pogut processar per una causa desconeguda.\");\n " +
		            "     else if (gdir.getStatus().code == G_GEO_MISSING_QUERY)\n " +
		            "       alert(\"S'ha introduit una consulta buida.\");\n " +
		            "     else if (gdir.getStatus().code == G_GEO_BAD_KEY)\n " +
		            "       alert(\"Clau de google incorrecta. Contacteu amb el WebMaster.\");\n " +
		            "     else if (gdir.getStatus().code == G_GEO_BAD_REQUEST)\n " +
		            "       alert(\"No s'ha pogut processar la consulta.\");\n " +
		            "     else alert(\"Error desconegut.\");\n " +
		            "}\n " +
		            "\n"+
		            "// Consulta el servlet la llista dels jaciments de les eres seleccionades\n"+
		            "function carregaJaciments(eres) {"+"\n" +
					"   //var req = GXmlHttp.create();\n"+
					"   var url;\n" +
					"   if (eres=='') url=\"/museu/jaciment?metode=llistaJaciments\";\n" +
		            "   else url ='/museu/jaciment?metode=eresJaciments&eres='+eres;\n" +
		            "   req.open(\"GET\", url, true);\n" +
		            "   req.onreadystatechange = getCallbackFunction(req, processaJaciments);"+"\n" +
		            "   req.send(null);"+"\n" +
        			"}\n" +
		            "\n"+
        			"// Retorna una funció que aten la instància XMLHttpRequest \n"+
        			"function getCallbackFunction(req, processData) {"+"\n" +
        			"  return function () {"+"\n" +
        			"    // Si l'estat és completat"+"\n" +
        			"    if (req.readyState == 4) {"+"\n" +
        			"      if (req.status == 200) {"+"\n" +
        			"        // Pasa l'xml a la funció que el gestionarà\n"+
        			"        processData(req.responseXML);"+"\n" +
					"      } else {"+"\n" +
					"        // Error HTTP"+"\n" +
					"        alert(\"HTTP error: \"+req.status);"+"\n" +
              		"      }"+"\n" +
            		"    }"+"\n" +
          			"  }"+"\n" +
        			"}\n" +
          			"" +"\n" +
          			"// Funció que processa l'xml amb la llista dels jaciments \n"+
        			"function processaJaciments(xmlDoc){"+"\n" +
        			"  // Obté la llista dels jaciments i els processa; "+"\n" +
        			"  jaciments = xmlDoc.documentElement.getElementsByTagName(\"jaciment\");"+"\n" +
        			"  mostraOverlays();"+"\n" +
        			"}"+"\n" +
          			"" +"\n" +
          			"// Funció que mostra totes les capes que han de ser visibles \n"+
        			"function mostraOverlays() {"+"\n" +
        			"  // Reiniciem totes les capes i afegim les fixes"+"\n" +
        			"  map.clearOverlays();"+"\n" +
        			"  map.addOverlay(baixLlobregat); \n"+
        			"  map.addOverlay(molins); \n"+
        			"  map.addOverlay(ametller); \n"+
        			"  map.addOverlay(museu); \n"+
        			"  lJaciments=[];\n"+
        			"  for (var i = 0; i < jaciments.length; i++) {"+"\n" +
        			"    var lat = parseFloat(jaciments[i].getElementsByTagName(\"latitud\")[0].firstChild.nodeValue);"+"\n" +
        			"  	 var lng = parseFloat(jaciments[i].getElementsByTagName(\"longitud\")[0].firstChild.nodeValue);"+"\n" +
        			"  	 var id = jaciments[i].getElementsByTagName(\"id\")[0].firstChild.nodeValue;"+"\n" +
        			"    var nom = jaciments[i].getElementsByTagName(\"nom\")[0].firstChild.nodeValue;"+"\n" +
        			"    creaJaciment(new GLatLng(lat,lng),nom,id);"+"\n" +
            		"  }"+"\n" +
            		"}"+"\n" +
          			"" +"\n" +
          			"// Funció que crea la marca dels jaciments amb els escoltadors d'events, ... \n"+
        			"function creaJaciment(posicio, nom, id) {"+"\n" +
        			"  var jaciment = new GMarker(posicio,{title:nom,icon:jacimentIcon});"+"\n" +
        			"  jaciment.id = id;"+"\n" +
					"  GEvent.addListener(jaciment, 'click', function() {"+"\n"+
					"       jaciment.openInfoWindowHtml(\"Consultant el jaciment \"+id);"+"\n"+
					"       consultaJaciment(''+id);"+"\n"+
					"  });"+"\n" +
					"  lJaciments.push(jaciment);\n"+
					"  map.addOverlay(jaciment);\n"+
					"}"+"\n" +
          			"" +"\n" +
          			"// Funció que consulta la informació del globus d'un jaciment al servlet \n"+
		            "function consultaJaciment(id) {"+"\n" +
		            "   modeMuseu();\n"+
		            "   req.open(\"GET\", \"http://portatil01.nitus.net/museu/jaciment?metode=idJaciment&id=\"+id, true);\n" +
		            "   req.onreadystatechange = getCallbackFunction(req, mostraGlobus);"+"\n" +
		            "   req.send(null);"+"\n" +
        			"}\n" +
					""+"\n" +
          			"// Funció que processa l'xml corresponent a un globus d'un jaciment \n"+
					"function processaGlobus(xmlInfo) {"+"\n" +
					"    var html;\n" +
					"    // Consultem el nom i l'identificador del jaciment.\n" +
					"    var nom = xmlInfo.getElementsByTagName(\"nom\")[0].firstChild.nodeValue;"+"\n" +
					"    var id = xmlInfo.getElementsByTagName(\"id\")[0].firstChild.nodeValue;"+"\n" +
        			"    html = '<strong>'+'<a href=\"javascript:consultaDiapositives('+id+')\">'+nom+'</a></strong><br>';"+"\n"+
        			"\n"+
        			"    // Consultem la població.\n" +
        			"    var poblacio;"+"\n" +
        			"    if (xmlInfo.getElementsByTagName(\"poblacio\")[0].childNodes.length!=0) {\n" +
        			"      poblacio = xmlInfo.getElementsByTagName(\"poblacio\")[0].firstChild.nodeValue;"+"\n" +
        			"    }\n"+
        			"    else poblacio='';\n"+
        			"    html=html+poblacio+'<br>';\n"+
        			"\n"+
        			"    // Llegim els atributs dels períodes  \n" +
        			"    var eres = xmlInfo.getElementsByTagName(\"periodes\"); \n" +
        			"    for (var i = 0; i < eres.length; i++) {"+"\n" +
        			"      var periode = eres[i].getElementsByTagName(\"periode\"); \n" +
        			"      if (periode.length!=0) html = html+'<u><strong>Periode'+(periode.length==1 ? '':'s')+'</strong></u>'; \n"+
        			"      for (var j=0; j<periode.length; j++){ \n"+
        			"        // Recuperem el nom del període. \n"+
        			"        var nomPeriode = periode[j].getElementsByTagName(\"nomPeriode\")[0].firstChild.nodeValue; \n" +
        			"        html = html +'<br>'+nomPeriode; \n"+
        			"\n"+
        			"        // Recuperem les dates del període.\n"+
        			"        var dates = ''; \n" +
        			"        if (periode[j].getElementsByTagName(\"dates\")[0].childNodes.length!=0) { \n" +
        			"           dates = periode[j].getElementsByTagName(\"dates\")[0].firstChild.nodeValue; \n" +
        			"           html=html + ' ' + dates; \n" +
        			"        } \n" +
        			"      } \n" +
            		"    } \n" +
            		"    return new Array(id,html); \n"+
					"}\n" +
          			"" +"\n" +
          			"// Funció que obra el globus d'un jaciment i engega la consulta de la informació \n"+
        			"function mostraGlobus(xmlInfo) {"+"\n" +
        			"    var retorn= processaGlobus(xmlInfo);\n"+
        			"    var jaciment = buscaJaciment(retorn[0]);\n"+
            		"    jaciment.openInfoWindowHtml(retorn[1]);"+"\n"+
					"}"+"\n" +
          			"" +"\n" +
          			"// Funció consulta la informació en format diapositives d'un jaciment\n"+
		            "function consultaDiapositives(id) {"+"\n" +
		            "   modeMuseu();\n"+
		            "   req.open(\"GET\", \"http://portatil01.nitus.net/museu/jaciment?metode=diapositives&id=\"+id, true);\n" +
		            "   req.onreadystatechange = getCallbackFunction(req, mostraDiapositives);"+"\n" +
		            "   req.send(null);"+"\n" +
        			"}\n" +
          			"" +"\n" +
          			"// Funció que obre el reproductor de diapositives amb la informació d'un jaciment en format diapositives. \n"+
					"function mostraDiapositives(xmlInfo){\n" +
					"    obreReproductor('reproductor', xmlInfo);\n"+
					"}"+"\n" +
          			"" +"\n" +
          			"// Funció que busca un jaciment a la llista de jaciments que es mostren \n"+
					"function buscaJaciment(id) {"+"\n" +
        			"    for (var i=0; i<lJaciments.length;i++){"+"\n" +
        			"      var jaciment = lJaciments[i];"+"\n" +
        			"      if (jaciment.id==id) return jaciment;"+"\n"+
        			"    }"+"\n"+
        			"    return jaciment;"+"\n"+
					"}"+"\n" +
          			"" +"\n" +
          			"// Funció que carrega la llista de períodes seleccionats i consulta els jaciments d'aquests períodes \n"+
					"function actualitzaPeriodes(){\n" +
					"   modeMuseu();\n"+
					"   var camps = document.getElementById('selector');" +"\n" +
					"   //Utilitzem el -1 per diferenciar la consulta de tots de la consulta buida," +"\n" +
					"   //aprofitant que el disseny no assigna números negatius als jaciments." +"\n" +
					"   ids = '-1';\n" + 
		    		"   for (var i=0; i<(camps.length)-3; i++){" +"\n" +
		    		"     if (camps[i].checked) {" +"\n" +
		    		"       ids = ids + ',' + camps[i].name;" +"\n" +
		    		"     }" +"\n" +
		    		"   }" +"\n" +
		    		"   carregaJaciments(ids);" +"\n" +
		    		"}\n "+
          			"" +"\n" +
          			"// Desactiva tots els checkboxes del selector de períodes \n"+
		    		"function netejaSelector() {"+"\n" +
		    		"   modeMuseu();\n"+
		    		"   var camps = document.getElementById('selector');" +"\n" +
		    		"   for (var i=0; i<camps.length; i++){" +"\n" +
		    		"     camps[i].checked=false; " +"\n" +
		    		"   }" +"\n" +
		    		"}"+"\n" +
          			"" +"\n" +
          			"// Selecciona tots els checkboxes del selector de períodes \n"+
		    		"function seleccionaTots() {"+"\n" +
		    		"   modeMuseu();\n"+
		    		"   var camps = document.getElementById('selector');" +"\n" +
		    		"   for (var i=0; i<camps.length; i++){" +"\n" +
		    		"     camps[i].checked=true; " +"\n" +
		    		"   }" +"\n" +
		    		"}"+"\n" +
          			"" +"\n" +
          			"// Control de restricció de moviments fora de la comarca del Baix Llobregat \n"+
		    		"function onMoveEnd() {"+"\n" +
                    "  if (modeIndicacions == false) {"+"\n" +
                    "    var bounds = map.getBounds();"+"\n" +
                    "    var southWest = bounds.getSouthWest();"+"\n" +
                    "    var northEast = bounds.getNorthEast();"+"\n" +
                    "    if (southWest.lat() > minLatitud &&"+"\n" +
                    "        northEast.lat() < maxLatitud &&"+"\n" +
                    "        southWest.lng() > minLongitud &&"+"\n" +
                    "        northEast.lng() < maxLongitud) {"+"\n" +
                    "        map.savePosition();\n" +
                    "    }"+"\n" +
                    "    else {"+"\n" +
                    "        map.returnToSavedPosition();"+"\n" +
                    "        alert(\"Aquest mapa està pensat per mostrar exclusivament\\n\"+\n" +
                    "              \"els jaciments del Museu de Molins de Rei.\\n\"+\n" +
                    "              \"Si no s'està mirant una ruta, no es pot sortir\\n\"+\n" +
                    "              \"de l'àmbit territorial de la comarca del Baix Llobregat.\");"+"\n" +
                    "    }"+"\n" +
                    "  }"+"\n" +
                    "}"+"\n" +
          			"" +"\n" +
          			"// Funció que activa el mode museu \n"+
                    "function modeMuseu() {"+"\n" +
                    "  map.returnToSavedPosition();"+"\n"+
                    "  modeIndicacions=false;"+"\n" +
                    "  clearDirections();"+"\n" +
                    "}"+"\n" +
          			"" +"\n" +
          			"// Funció que activa el mode indicacions \n"+
                    "function modeIndicacions() {"+"\n" +
                    "  map.savePosition();"+"\n" +
                    "  modeIndicacions=true;"+"\n" +
                    "}"+"\n" +
                    "</script>"+"\n" +

		    		"</head>"+"\n" +
		            "<body onload=\"initialize()\" onunload=\"GUnload()\" topmargin=\"10\" leftmargin=\"0\" rightmargin=\"0\" bottommargin=\"0\" marginwidth=\"0\" marginheight=\"0\">"+"\n"+ 
		    		"";
		return resposta;
	}

	/**
	 * Genera la cadena html corresponent al body del document html
	 * @param con Necessita la connexió a la base de dades per consultar-la
	 * @return String amb la cadena html corresponent al body
	 */
	private String getMapsBody(Connection con){
		String resposta = "<div id=\"content\"><p></div>";
		try {
			Statement stmt=con.createStatement();
			resposta += "<div id=\"content\">"+"\n";
			resposta += "   <div id=\"map_utils\">"+"\n";
			resposta += "\n";
			resposta += "	<h1>Periodes Històrics</h1>"+"\n";
			resposta += "	<form name='selector' action='' method='post' id='selector'>"+"\n";

			String select = "Select id, nom from jaciments.periodes order by id";
			ResultSet rs = stmt.executeQuery(select);
			while (rs.next()) resposta += "		<input type='Checkbox' name='"+rs.getInt(1)+"' CHECKED>"+rs.getString(2)+"</input><br>\n";
			rs.close();
			resposta += "	<center><table cellspacing='3'>" +"\n"+
		    			"     <tr>" +"\n"+
		    			"       <td>" +"\n"+
		    			"         <input type=\"button\" value=\"Tots\" class=\"boto\" onClick=\"seleccionaTots()\" />"+"\n"+
		    			"       </td>" +
		    			"       <td>" +"\n"+
		    			"         <input type=\"button\" value=\"Cap\" class=\"boto\" onClick=\"netejaSelector()\"/>"+"\n"+
		    			"       </td>" +
		    			"       <td>" +"\n"+
		    			"         <input type=\"button\" value=\"Actualitza\" class=\"boto\" onClick='actualitzaPeriodes()'/>"+"\n"+
		    			"       </td>" +
		    			"     </tr>" +"\n"+
		    			"	</table></center>" +"\n"+
		    			"	</form>"+"\n";
			resposta += "	<form action='#' id='directionsForm' name='directionsForm' onsubmit=\"setDirections(this.carrer.value+' '+this.num.value+' '+this.ciutat.value+' '+this.pais.value, this.to.value); return false;\">"+"\n"+
						" 	<h1>Com arribar-hi</h1>"+"\n"+
						" 	<table border='0' cellspacing='0' cellpadding='0' width='95%' align='left'>"+"\n"+
		    			"     <tr>" +"\n"+
		    			"       <th>" +"\n"+
						" 	    Des de"+"\n"+
						"       </th>" +"\n"+
						"     </tr>" +"\n"+
		    			"     <tr>" +"\n"+
		    			"       <td>" +"\n"+
			 			"	      Carrer:"+"\n"+
						"       </td>" +"\n"+
		    			"       <td colspan='3'>" +"\n"+
			 			"         <input type=\"text\" name=\"carrer\" size=\"30\" maxlength=\"60\" class=\"camp_form1\">\n"+
						"       </td>" +"\n"+
						"     </tr>" +"\n"+
						"     </tr>" +"\n"+
		    			"     <tr>" +"\n"+
		    			"       <td>" +"\n"+
			 			"	      Num:"+"\n"+
						"       </td>" +"\n"+
		    			"       <td>" +"\n"+
			 			"         <input type=\"text\" name=\"num\" size=\"3\" maxlength=\"5\" class=\"camp_form1\">\n"+
						"       </td>" +"\n"+
						"       <td>" +"\n"+
			 			"	      Ciutat:"+"\n"+
						"       </td>" +"\n"+
		    			"       <td>" +"\n"+
			 			"         <input type=\"text\" name=\"ciutat\" size=\"15\" maxlength=\"60\" class=\"camp_form1\">\n"+
						"       </td>" +"\n"+
						"     </tr>" +"\n"+
		    			"     <tr>" +"\n"+
		    			"       <td>" +"\n"+
			 			"	      Pais:"+"\n"+
						"       </td>" +"\n"+
		    			"       <td colspan='3'>" +"\n"+
			 			"         <input type=\"text\" name=\"pais\" size=\"30\" maxlength=\"40\" class=\"camp_form1\">\n"+
						"       </td>" +"\n"+
						"     </tr>" +"\n"+
		    			"     <tr>" +"\n"+
		    			"       <th>" +"\n"+
						" 	    Fins a"+"\n"+
						"       </th>" +"\n"+
						"     </tr>" +"\n"+
		    			"     <tr>" +"\n"+
		    			"       <td  colspan='4'>" +"\n"+
						"   <select id='to' name='to' class=\"camp_form1\">"+"\n"+
			 			"		<option value='0,0' class=\"select\">&lt;---------------Jaciment----------------&gt;</option>\n"+
			 			"		<option value='41.415138,2.01712' class=\"select\">Museu Municipal de Molins de Rei</option>\n"+
			 			"		<option value='41.414024,2.016185' class=\"select\">Futura ubicació del Museu (Ca n'Ametller)</option>\n";
			select = "Select id, nom, yWGS84, xWGS84 from jaciments.jaciments order by id";
			rs = stmt.executeQuery(select);
			while (rs.next()) resposta += "		<option value='"+rs.getBigDecimal(3)+","+rs.getBigDecimal(4)+"' class=\"select\">"+rs.getString(2)+"</option>\n";
			rs.close();
			resposta += "   </select>"+"\n"+
						"       </td>" +"\n"+
						"     </tr>" +"\n"+
		    			"     <tr>" +"\n"+
		    			"       <td  colspan='4'>" +"\n"+
		    			"	      <center><table>" +"\n"+
						"           <tr>" +"\n"+
						"             <td>" +"\n"+
						"               <input type=\"reset\" value=\"Neteja\" class=\"boto\" onclick='modeMuseu();'/>"+"\n"+
						"             </td>" +
						"             <td>" +"\n"+
						"               <input type=\"submit\" value=\"Busca\" class=\"boto\" />"+"\n"+
						"             </td>" +
						"           </tr>" +"\n"+
						"	      </table></center>" +"\n"+
						"       </td>" +"\n"+
						"     </tr>" +"\n"+
						" 	</table>"+"\n";
			resposta += "	</form>"+"\n";
			resposta += "\n";
			resposta +="     </div>";
			resposta +="   <div id=\"map_canvas\" style=\"width: 500px; height: 500px\"></div>"+"\n";
			resposta +="<div id=\"reproductor\" name='reproductor' class=\"reproductor\">\n"+
			   		   "  <!-- La barra de títol -->\n"+
			   		   "  <div id=\"titolReproductor\" name='titolReproductor' class=\"titolReproductor\">\n"+
			   		   "  </div>\n"+
			   		   "  <!-- L'àrea de diapositives-->\n"+
			   		   "  <div id=\"areaReproduccio\" name='areaReproduccio' class=\"areaReproduccio\">\n"+
			   		   "  <!-- Aquí hi van les diapositives -->\n"+
			   		   "  </div>\n"+
			   		   "  <!-- Area Controls -->\n"+
			   		   "  <div id=\"controlsReproduccio\" name='controlsReproduccio' class=\"controlsReproduccio\">\n"+
			   		   "  <center>\n"+
			   		   "  <table cellspacing='7' cellpaddign='0' border='0'> \n"+
			   		   "    <tr> \n"+
			   		   "      <td> \n"+
			   		   "        <a href='javascript:anteriorDiapositiva();' onMouseOver=\"botoOver('esquerra');return true;\"\n"+ 
			   		   "                                                    onMouseOut=\"botoOut('esquerra');return true;\">\n"+
			   		   "          <img name='esquerra' src='image/esquerraOut.png' heigh='35' width='36' alt='Anterior' border='0'>\n" +
			   		   "        </a>\n"+ 
			   		   "      </td> \n"+
			   		   "      <td> \n"+
			   		   "        <a href='javascript:seguentDiapositiva();' onMouseOver=\"botoOver('dreta');return true;\" \n"+
			   		   "                                                   onMouseOut=\"botoOut('dreta');return true;\">\n"+
			   		   "          <img name='dreta' src='image/dretaOut.png' heigh='35' width='36' alt='Seguent' border='0'>\n" +
			   		   "        </a>\n"+ 
			   		   "      </td> \n"+
			   		   "    </tr> \n"+
			   		   "  </table> \n"+
			   		   "  </center>\n"+
			   		   "  </div>\n"+
			   		   "</div>";
			resposta +="   <div id=\"map_directions\">" +
					   "   </div>"+"\n";
			resposta +="</div>";

			stmt.close();
		} catch (SQLException e) {
			return "S'ha produit un l'error d'SQL "+e.getErrorCode()+"-"+e.getSQLState()+": "+e.getLocalizedMessage();
		}
		return resposta;
	}

}
